package com.vykon.vykonPro.alarm;

import javax.baja.alarm.BAlarmRecord;
import javax.baja.data.BIDataValue;
import javax.baja.naming.SlotPath;
import javax.baja.sys.Action;
import javax.baja.sys.BComponent;
import javax.baja.sys.BFacets;
import javax.baja.sys.BIcon;
import javax.baja.sys.BString;
import javax.baja.sys.BValue;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Topic;
import javax.baja.sys.Type;
import javax.baja.util.BFormat;
import javax.baja.nre.util.TextUtil;
import javax.baja.util.IFuture;
import javax.baja.util.Invocation;

import com.vykon.vykonPro.util.BVykonProService;

public class BAlarmDataFilter
    extends BComponent
{
  /*-
   class BAlarmDataFilter
   {
     properties
     {
       keysToInclude:String
         -- Comma separated list of alarmData keys to keep in filtered BAlarmRecord output
         default{[ "enter alarmData keys to keep separated by commas" ]}
     }
     actions
     {
       receive(alarm: BAlarmRecord)
         -- Route an alarm record
         flags { summary, async }
         default {[ new BAlarmRecord() ]}
     }
     topics
     {
       alarm: BAlarmRecord
        flags { summary }
        -- fired whenever an alarm of this class is detected
      
     }
   }
   -*/
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.vykon.vykonPro.alarm.BAlarmDataFilter(4178490436)1.0$ @*/
/* Generated Tue Mar 16 15:45:48 EDT 2010 by Slot-o-Matic 2000 (c) Tridium, Inc. 2000 */

////////////////////////////////////////////////////////////////
// Property "keysToInclude"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>keysToInclude</code> property.
   * Comma separated list of alarmData keys to keep in filtered BAlarmRecord output
   * @see com.vykon.vykonPro.alarm.BAlarmDataFilter#getKeysToInclude
   * @see com.vykon.vykonPro.alarm.BAlarmDataFilter#setKeysToInclude
   */
  public static final Property keysToInclude = newProperty(0, "enter alarmData keys to keep separated by commas",null);
  
  /**
   * Get the <code>keysToInclude</code> property.
   * @see com.vykon.vykonPro.alarm.BAlarmDataFilter#keysToInclude
   */
  public String getKeysToInclude() { return getString(keysToInclude); }
  
  /**
   * Set the <code>keysToInclude</code> property.
   * @see com.vykon.vykonPro.alarm.BAlarmDataFilter#keysToInclude
   */
  public void setKeysToInclude(String v) { setString(keysToInclude,v,null); }

////////////////////////////////////////////////////////////////
// Action "receive"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>receive</code> action.
   * Route an alarm record
   */
  public static final Action receive = newAction(Flags.SUMMARY|Flags.ASYNC,new BAlarmRecord(),null);
  
  /**
   * Invoke the <code>receive</code> action.
   * @see com.vykon.vykonPro.alarm.BAlarmDataFilter#receive
   */
  public void receive(BAlarmRecord alarm) { invoke(receive,alarm,null); }

////////////////////////////////////////////////////////////////
// Topic "alarm"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>alarm</code> topic.
   * @see com.vykon.vykonPro.alarm.BAlarmDataFilter#fireAlarm
   */
  public static final Topic alarm = newTopic(Flags.SUMMARY,null);
  
  /**
   * Fire an event for the <code>alarm</code> topic.
   * @see com.vykon.vykonPro.alarm.BAlarmDataFilter#alarm
   */
  public void fireAlarm(BAlarmRecord event) { fire(alarm, event, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BAlarmDataFilter.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  public void started() throws Exception
  {
    initializeKeys();
  }

  public void changed(Property property, Context context) 
  {
   super.changed(property, context);
    if( !Sys.atSteadyState() || !isRunning() )
     return;
    
    if(property.equals(keysToInclude))
    {
      initializeKeys();
    }
  }
  
  private void initializeKeys()
  {
    //make sure the string has something in it
    if(getKeysToInclude().length()>0)
    {
      //check if there is a comma which means more than one item and we need to split the list
      if(getKeysToInclude().indexOf(",")>-1)
      {  
        filteredkeys = TextUtil.splitAndTrim(getKeysToInclude(), ',');
      }
      else //only one item in the list
      {
        filteredkeys = new String[1];
        filteredkeys[0] = getKeysToInclude().trim();
      }
    }
  }
  
  public IFuture post(Action action, BValue argument, Context cx)
  {
    if(action == receive)
      return BVykonProService.getInstance().getWorker().post(new Invocation(this, action, argument, cx));
    else
      return super.post(action, argument, cx);
  }
  
  public void doReceive(BAlarmRecord alarm)
  {
    //get a new copy of the alarm reocrd just to be safe
    BAlarmRecord myalarm = (BAlarmRecord)alarm.newCopy();
    
    //set filtered data values to new array the same size as filterekeys
    filteredvalues = new BIDataValue[filteredkeys.length];
    
    for(int i=0; i<filteredkeys.length; i++)
    {
      //check to make sure that the key exist in the alarmData
      if(myalarm.getAlarmFacet(filteredkeys[i])!= null)
      {
        if(filteredkeys[i].equals("msgText"))
        {
          String s = ((BString) myalarm.getAlarmFacet("msgText")).toString(); 
          s = (SlotPath.unescape(BFormat.make(s).format(myalarm)));
          filteredvalues[i] = BString.make(s);
            
        }
        else
        {
          //store the current facet for use later
          filteredvalues[i] = (BIDataValue)myalarm.getAlarmFacet(filteredkeys[i]);
        }
      }
    }
    
    //create a new BFacet using the filteredkeys and filtervalues
    BFacets b = BFacets.make(filteredkeys, filteredvalues);
    
    
    
    //assign the new facet to myalarm
    myalarm.setAlarmData(b);
    
    //fire the topic to send out the filtered alarm record
    fireAlarm(myalarm);
  }
  
  public BIcon getIcon() { return icon; }
  private static final BIcon icon = BIcon.std("filter.png");

  
////////////////////////////////////////////////////////////////
//local variables
////////////////////////////////////////////////////////////////
 
 private String[] filteredkeys;
 private BIDataValue[] filteredvalues;
}
