package com.vykon.vykonPro.alarm;

import com.vykon.vykonPro.hvac.BRollingAvg;
import com.vykon.vykonPro.hvac.BRollingAvgMaster;

import javax.baja.alarm.BAlarmRecord;
import javax.baja.alarm.ext.BAlarmSourceExt;
import javax.baja.alarm.ext.offnormal.BTwoStateAlgorithm;
import javax.baja.control.BControlPoint;
import javax.baja.data.BIDataValue;
import javax.baja.nre.annotations.NiagaraProperty;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.status.BStatusNumeric;
import javax.baja.status.BStatusValue;
import javax.baja.sys.*;

/**
 * BMaxDeviationAlgorithm implements a standard out-of-range algorithm for the maximumDeviation slot value.
 *
 * Created by james on 12/19/2018.
 */

@NiagaraType

@NiagaraProperty(
        name = "deviationLimit",
        type = "double",
        defaultValue = "0"
)

@NiagaraProperty(
        name = "deadband",
        type = "double",
        defaultValue = "0"
)
public class BMaxDeviationAlgorithm
    extends BTwoStateAlgorithm
{
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.vykon.vykonPro.alarm.BMaxDeviationAlgorithm(3491268178)1.0$ @*/
/* Generated Wed Dec 19 10:43:17 EST 2018 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Property "deviationLimit"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code deviationLimit} property.
   * @see #getDeviationLimit
   * @see #setDeviationLimit
   */
  public static final Property deviationLimit = newProperty(0, 0, null);
  
  /**
   * Get the {@code deviationLimit} property.
   * @see #deviationLimit
   */
  public double getDeviationLimit() { return getDouble(deviationLimit); }
  
  /**
   * Set the {@code deviationLimit} property.
   * @see #deviationLimit
   */
  public void setDeviationLimit(double v) { setDouble(deviationLimit, v, null); }

////////////////////////////////////////////////////////////////
// Property "deadband"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code deadband} property.
   * @see #getDeadband
   * @see #setDeadband
   */
  public static final Property deadband = newProperty(0, 0, null);
  
  /**
   * Get the {@code deadband} property.
   * @see #deadband
   */
  public double getDeadband() { return getDouble(deadband); }
  
  /**
   * Set the {@code deadband} property.
   * @see #deadband
   */
  public void setDeadband(double v) { setDouble(deadband, v, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BMaxDeviationAlgorithm.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

    /**
     * A BMaxDeviationAlgorithm's grandparent must be a
     * BRollingAvg or BRollingAvgMaster
     */
    public boolean isGrandparentLegal(BComponent grandparent)
    {
        return (grandparent instanceof BRollingAvg || grandparent instanceof BRollingAvgMaster);
    }

    public BFacets getSlotFacets(Slot slot)
    {
        if (slot == deadband  ||
                slot == deviationLimit   )
        {
            //BRollingAvg point = (BRollingAvg) getParentPoint();
            BControlPoint point = getParentPoint();
            //if (point != null) return point.getDeviationFacets();
            if (point != null) return (BFacets)point.get("deviationFacets");
            return BFacets.DEFAULT;
        }
        return super.getSlotFacets(slot);
    }

    @Override
    protected boolean isNormal(BStatusValue bStatusValue) {
        //BRollingAvg rollingAvg = (BRollingAvg)getParentPoint();
        BControlPoint point = getParentPoint();

        double error = ((BStatusNumeric)point.get("maximumDeviation")).getValue();

        // if BRollingAvgMaster the maximumDeviation is NaN when not being calculated
        if(Double.isNaN(error)) return true;

        // If in alarm / offnormal state, must return to within (error - deadband)
        if(isCurrentOffnormal())
            return (error < (getDeviationLimit() - getDeadband()));
        else
            return (error < getDeviationLimit());
    }

    /**
     *  Write the key-value pairs defining alarm data for the
     *  alarm algorithm and state to the given Facets.
     * <p>
     *  The alarm data for an Out of Range alarm is given by
     *  BACnet table 13-3, Standard Object Property Values
     *  returned in notifications.
     *
     *
     * @param map The map.
     */
    public void writeAlarmData(BStatusValue out, java.util.Map map)
    {
        BControlPoint point = getParentPoint();
        BAlarmSourceExt ext = getParentExt();

        BStatusNumeric maximumDeviation = (BStatusNumeric)point.get("maximumDeviation");
        BFacets deviationFacets = (BFacets)point.get("deviationFacets");

        String maximumDeviationInput = "";
        boolean master = false;
        if(point instanceof BRollingAvgMaster)
        {
            master = true;
            maximumDeviationInput = ((BRollingAvgMaster)point).getMaximumDeviationInput();
        }

        if(!ext.getAlarmState().isInAlarm())
        {
            map.put(BAlarmRecord.DEADBAND, BString.make(BDouble.toString(getDeadband(), deviationFacets)));
            map.put(BAlarmRecord.ERROR_LIMIT, BString.make(BDouble.toString(getDeviationLimit(), deviationFacets)));
            map.put(BAlarmRecord.OFFNORMAL_VALUE, BString.make(maximumDeviation.valueToString(deviationFacets)));
            map.put(BAlarmRecord.ALARM_VALUE, (BIDataValue)maximumDeviation.getValueValue());
            map.put(BAlarmRecord.PRESENT_VALUE, BString.make(maximumDeviation.valueToString(deviationFacets)));
            map.put(BAlarmRecord.STATUS, BString.make(out.getStatus().toString(null)));

            if(master) map.put("deviationInput", BString.make(maximumDeviationInput));
        }
        else
        {
            map.put(BAlarmRecord.PRESENT_VALUE, BString.make(maximumDeviation.valueToString(deviationFacets)));
        }



    }
}
