package com.vykon.vykonPro.energy;

import javax.baja.status.BStatus;
import javax.baja.status.BStatusBoolean;
import javax.baja.status.BStatusNumeric;
import javax.baja.sys.Action;
import javax.baja.sys.BAbsTime;
import javax.baja.sys.BComponent;
import javax.baja.sys.BDynamicEnum;
import javax.baja.sys.BEnum;
import javax.baja.sys.BEnumRange;
import javax.baja.sys.BFacets;
import javax.baja.sys.BRelTime;
import javax.baja.sys.BTime;
import javax.baja.sys.BValue;
import javax.baja.sys.Clock;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Slot;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;
import javax.baja.units.UnitDatabase;

import com.vykon.vykonPro.enums.BPeakIntervalEnum;


public class BBlockDemandCalc
    extends BComponent
{
  /*-
   class BBlockDemandCalc
   {
     properties
     {
       status: BStatus
         -- Component status based on currentPulseCount or errors calculating kw or kwh.
         flags{ readonly, transient, defaultOnClone }
         default{[ BStatus.ok ]}
         
       lastFaultCause: String
         -- The last fault cause
         flags{ readonly, defaultOnClone }
         default{[ "" ]}
         
       lastFaultTime: BAbsTime
         -- The time of the last fault
         flags{ readonly, defaultOnClone }
         default{[ BAbsTime.NULL ]}
         
       consumptionFacets: BFacets
        -- These facets are applied against the consumption properties.
        flags{ operator }
        default {[ BFacets.makeNumeric(UnitDatabase.getUnit("kilowatt hour"), 1) ]}
       
       demandFacets: BFacets
        -- These facets are applied against the demand properties.
        flags{ operator }
        default {[ BFacets.makeNumeric(UnitDatabase.getUnit("kilowatt"), 1) ]}
        
       currentPulseCount: BStatusNumeric
        --Pulse count input
        flags{ summary, operator, defaultOnClone }
        default{[ new BStatusNumeric() ]}
        
       kwhPerPulse: double 
        --KWH value per pulse
        flags{ operator }
        default{[ 1.0 ]}
       
       kwhPerPulseFacets: BFacets
        -- These facets are applied against the kwhPerPulse property
        flags{ operator }
        default{[ BFacets.make("precision", 3) ]}
       
       checkFrequency: boolean
         -- if true validates the calculated pulse frequency is less than the maxPulseFrequency and
         -- it is a valid value (invalid values are Double.NaN, Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY).
         -- If calculated pulse frequency is not valid or greater than max pulse frequency, consumption values
         -- are not calculated.
         flags{ operator }
         default{[ true ]}
         
       maxPulseFrequency: double
         -- max allowable pulse frequency for which consumption will be calculated.
         flags{ operator }
         default{[ 10 ]}
         slotfacets{[ BFacets.makeNumeric(UnitDatabase.getUnit("hertz"), 1, 1, 100) ]}
         
       actualPulseFrequency: double
         -- the actual mesaured pulse frequency.
         flags{ readonly, transient }
         default{[ 0 ]}
         slotfacets{[ BFacets.makeNumeric(UnitDatabase.getUnit("hertz"), 1) ]}
         
       meterRollover: long
        -- maximum pulse count from device
        flags{ operator }
        default{[65535l]}    
       
       peakMode: BStatusBoolean
         -- peak mode for calculating consumption and demand
         flags{ summary, operator }
         default{[ new BStatusBoolean() ]}
         slotfacets{[ TRUE_TEXT="onPeak", FALSE_TEXT="offPeak" ]}
      
       kwhTotal: BStatusNumeric
         -- Running KWH value (onPeak and offPeak) since last reset
         flags{ readonly, summary }
         default{[ new BStatusNumeric() ]}
         
       kwhOffPeakTotal: BStatusNumeric
        -- Running KWH value during offPeak mode since last reset
        flags{ readonly, summary }
        default {[ new BStatusNumeric() ]}
       
       kwhOnPeakTotal: BStatusNumeric
        -- Running KWH value during onPeak mode since last reset
        flags{ readonly, summary }
        default {[ new BStatusNumeric() ]}
       
       monthlyKwh: BStatusNumeric
         -- Monthly KWH value accumulated (onPeak and offPeak)
         flags{ readonly }
         default{[ new BStatusNumeric() ]}
         
       monthlyKwhOffPeak: BStatusNumeric
         -- Monthly kwh value accumulated during offPeak mode
         flags{ readonly }
         default{[ new BStatusNumeric() ]}
       
       monthlyKwhOnPeak: BStatusNumeric
         -- Monthly kwh value accumulated during onPeak mode
         flags{ readonly }
         default{[ new BStatusNumeric() ]}
       
       kwh5: BStatusNumeric
         -- kwh for previous 5 minute interval
         flags{ readonly, transient }
         default{[ new BStatusNumeric() ]}
         
       kwh15: BStatusNumeric
        -- kwh for previous 15 minute interval
        flags{ readonly, transient }
        default{[ new BStatusNumeric() ]}
        
       kwh30: BStatusNumeric
        -- kwh for previous 30 minute interval
        flags{ readonly, transient }
        default{[ new BStatusNumeric() ]}

       maxKw: BStatusNumeric
         -- peak kw observed regardless of onPeak or offPeak mode
         flags{ readonly }
         default{[ new BStatusNumeric() ]}
         
       maxKwTimestamp: BAbsTime
         -- time of the observed demand peak
         flags{ readonly }
         default{[ BAbsTime.NULL ]}
         
       maxKwOffPeak: BStatusNumeric
        -- peak kw observed during offPeak mode
        flags{ readonly }
        default{[ new BStatusNumeric() ]}
        
       maxKwOffPeakTimestamp: BAbsTime
        -- time of the observed offPeak demand peak
        flags{ readonly }
        default{[ BAbsTime.NULL ]}
        
       maxKwOnPeak: BStatusNumeric
        -- peak kw observed during onPeak mode
        flags{ readonly }
        default{[ new BStatusNumeric() ]}
      
       maxKwOnPeakTimestamp: BAbsTime
        -- time of the observer onPeak demand peak
        flags{ readonly }
        default{[ BAbsTime.NULL ]}
        
       peakKwInterval: BEnum
        -- configures whether the maxKwOnPeak and maxKwOffPeak values are 15 or 30 minute block demand values
        flags{ operator }
        default{[ (BDynamicEnum.make(0, BEnumRange.make(BPeakIntervalEnum.TYPE))) ]}
       
       kw5: BStatusNumeric
         -- 5 minute block demand value
         flags{ readonly, transient }
         default{[ new BStatusNumeric() ]}
         
       kw15: BStatusNumeric
        -- 15 minute block demand value
        flags{ readonly, transient }
        default {[ new BStatusNumeric() ]}

       kw30: BStatusNumeric
        -- 30 minute block demand value
        flags{ readonly, transient }
        default {[ new BStatusNumeric() ]}
     }
     actions
     {
       presetKwh(arg: BConsumptionProperties)
         flags{ operator }
         default{[ new BConsumptionProperties() ]}
         //used to preload kwh values
       
       presetKw(arg: BDemandProperties)
         flags{ operator }
         default{[ new BDemandProperties() ]}
         
       monthlyReset()
         // resets monthly values
       
       reset()
         // resets all values
       
       timerExpired5()
         flags{ hidden }
         
       
     }
     topics
     {
     }
   }
   -*/
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.vykon.vykonPro.energy.BBlockDemandCalc(2767236146)1.0$ @*/
/* Generated Mon Nov 21 09:01:23 EST 2011 by Slot-o-Matic 2000 (c) Tridium, Inc. 2000 */

////////////////////////////////////////////////////////////////
// Property "status"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>status</code> property.
   * Component status based on currentPulseCount or errors
   * calculating kw or kwh.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getStatus
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setStatus
   */
  public static final Property status = newProperty(Flags.READONLY|Flags.TRANSIENT|Flags.DEFAULT_ON_CLONE, BStatus.ok,null);
  
  /**
   * Get the <code>status</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#status
   */
  public BStatus getStatus() { return (BStatus)get(status); }
  
  /**
   * Set the <code>status</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#status
   */
  public void setStatus(BStatus v) { set(status,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastFaultCause"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastFaultCause</code> property.
   * The last fault cause
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getLastFaultCause
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setLastFaultCause
   */
  public static final Property lastFaultCause = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, "",null);
  
  /**
   * Get the <code>lastFaultCause</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#lastFaultCause
   */
  public String getLastFaultCause() { return getString(lastFaultCause); }
  
  /**
   * Set the <code>lastFaultCause</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#lastFaultCause
   */
  public void setLastFaultCause(String v) { setString(lastFaultCause,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastFaultTime"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastFaultTime</code> property.
   * The time of the last fault
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getLastFaultTime
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setLastFaultTime
   */
  public static final Property lastFaultTime = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, BAbsTime.NULL,null);
  
  /**
   * Get the <code>lastFaultTime</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#lastFaultTime
   */
  public BAbsTime getLastFaultTime() { return (BAbsTime)get(lastFaultTime); }
  
  /**
   * Set the <code>lastFaultTime</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#lastFaultTime
   */
  public void setLastFaultTime(BAbsTime v) { set(lastFaultTime,v,null); }

////////////////////////////////////////////////////////////////
// Property "consumptionFacets"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>consumptionFacets</code> property.
   * These facets are applied against the consumption properties.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getConsumptionFacets
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setConsumptionFacets
   */
  public static final Property consumptionFacets = newProperty(Flags.OPERATOR, BFacets.makeNumeric(UnitDatabase.getUnit("kilowatt hour"), 1),null);
  
  /**
   * Get the <code>consumptionFacets</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#consumptionFacets
   */
  public BFacets getConsumptionFacets() { return (BFacets)get(consumptionFacets); }
  
  /**
   * Set the <code>consumptionFacets</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#consumptionFacets
   */
  public void setConsumptionFacets(BFacets v) { set(consumptionFacets,v,null); }

////////////////////////////////////////////////////////////////
// Property "demandFacets"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>demandFacets</code> property.
   * These facets are applied against the demand properties.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getDemandFacets
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setDemandFacets
   */
  public static final Property demandFacets = newProperty(Flags.OPERATOR, BFacets.makeNumeric(UnitDatabase.getUnit("kilowatt"), 1),null);
  
  /**
   * Get the <code>demandFacets</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#demandFacets
   */
  public BFacets getDemandFacets() { return (BFacets)get(demandFacets); }
  
  /**
   * Set the <code>demandFacets</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#demandFacets
   */
  public void setDemandFacets(BFacets v) { set(demandFacets,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentPulseCount"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentPulseCount</code> property.
   * Pulse count input
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getCurrentPulseCount
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setCurrentPulseCount
   */
  public static final Property currentPulseCount = newProperty(Flags.SUMMARY|Flags.OPERATOR|Flags.DEFAULT_ON_CLONE, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentPulseCount</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#currentPulseCount
   */
  public BStatusNumeric getCurrentPulseCount() { return (BStatusNumeric)get(currentPulseCount); }
  
  /**
   * Set the <code>currentPulseCount</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#currentPulseCount
   */
  public void setCurrentPulseCount(BStatusNumeric v) { set(currentPulseCount,v,null); }

////////////////////////////////////////////////////////////////
// Property "kwhPerPulse"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>kwhPerPulse</code> property.
   * KWH value per pulse
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getKwhPerPulse
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setKwhPerPulse
   */
  public static final Property kwhPerPulse = newProperty(Flags.OPERATOR, 1.0,null);
  
  /**
   * Get the <code>kwhPerPulse</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwhPerPulse
   */
  public double getKwhPerPulse() { return getDouble(kwhPerPulse); }
  
  /**
   * Set the <code>kwhPerPulse</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwhPerPulse
   */
  public void setKwhPerPulse(double v) { setDouble(kwhPerPulse,v,null); }

////////////////////////////////////////////////////////////////
// Property "kwhPerPulseFacets"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>kwhPerPulseFacets</code> property.
   * These facets are applied against the kwhPerPulse property
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getKwhPerPulseFacets
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setKwhPerPulseFacets
   */
  public static final Property kwhPerPulseFacets = newProperty(Flags.OPERATOR, BFacets.make("precision", 3),null);
  
  /**
   * Get the <code>kwhPerPulseFacets</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwhPerPulseFacets
   */
  public BFacets getKwhPerPulseFacets() { return (BFacets)get(kwhPerPulseFacets); }
  
  /**
   * Set the <code>kwhPerPulseFacets</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwhPerPulseFacets
   */
  public void setKwhPerPulseFacets(BFacets v) { set(kwhPerPulseFacets,v,null); }

////////////////////////////////////////////////////////////////
// Property "checkFrequency"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>checkFrequency</code> property.
   * if true validates the calculated pulse frequency is
   * less than the maxPulseFrequency and it is a valid value (invalid values are Double.NaN, Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY). If calculated pulse frequency is not valid or greater than max pulse frequency, consumption values are not calculated.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getCheckFrequency
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setCheckFrequency
   */
  public static final Property checkFrequency = newProperty(Flags.OPERATOR, true,null);
  
  /**
   * Get the <code>checkFrequency</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#checkFrequency
   */
  public boolean getCheckFrequency() { return getBoolean(checkFrequency); }
  
  /**
   * Set the <code>checkFrequency</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#checkFrequency
   */
  public void setCheckFrequency(boolean v) { setBoolean(checkFrequency,v,null); }

////////////////////////////////////////////////////////////////
// Property "maxPulseFrequency"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>maxPulseFrequency</code> property.
   * max allowable pulse frequency for which consumption
   * will be calculated.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getMaxPulseFrequency
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setMaxPulseFrequency
   */
  public static final Property maxPulseFrequency = newProperty(Flags.OPERATOR, 10,BFacets.makeNumeric(UnitDatabase.getUnit("hertz"), 1, 1, 100));
  
  /**
   * Get the <code>maxPulseFrequency</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxPulseFrequency
   */
  public double getMaxPulseFrequency() { return getDouble(maxPulseFrequency); }
  
  /**
   * Set the <code>maxPulseFrequency</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxPulseFrequency
   */
  public void setMaxPulseFrequency(double v) { setDouble(maxPulseFrequency,v,null); }

////////////////////////////////////////////////////////////////
// Property "actualPulseFrequency"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>actualPulseFrequency</code> property.
   * the actual mesaured pulse frequency.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getActualPulseFrequency
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setActualPulseFrequency
   */
  public static final Property actualPulseFrequency = newProperty(Flags.READONLY|Flags.TRANSIENT, 0,BFacets.makeNumeric(UnitDatabase.getUnit("hertz"), 1));
  
  /**
   * Get the <code>actualPulseFrequency</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#actualPulseFrequency
   */
  public double getActualPulseFrequency() { return getDouble(actualPulseFrequency); }
  
  /**
   * Set the <code>actualPulseFrequency</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#actualPulseFrequency
   */
  public void setActualPulseFrequency(double v) { setDouble(actualPulseFrequency,v,null); }

////////////////////////////////////////////////////////////////
// Property "meterRollover"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>meterRollover</code> property.
   * maximum pulse count from device
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getMeterRollover
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setMeterRollover
   */
  public static final Property meterRollover = newProperty(Flags.OPERATOR, 65535l,null);
  
  /**
   * Get the <code>meterRollover</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#meterRollover
   */
  public long getMeterRollover() { return getLong(meterRollover); }
  
  /**
   * Set the <code>meterRollover</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#meterRollover
   */
  public void setMeterRollover(long v) { setLong(meterRollover,v,null); }

////////////////////////////////////////////////////////////////
// Property "peakMode"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>peakMode</code> property.
   * peak mode for calculating consumption and demand
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getPeakMode
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setPeakMode
   */
  public static final Property peakMode = newProperty(Flags.SUMMARY|Flags.OPERATOR, new BStatusBoolean(),BFacets.make(BFacets.make(BFacets.TRUE_TEXT,"onPeak"),BFacets.make(BFacets.FALSE_TEXT,"offPeak")));
  
  /**
   * Get the <code>peakMode</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#peakMode
   */
  public BStatusBoolean getPeakMode() { return (BStatusBoolean)get(peakMode); }
  
  /**
   * Set the <code>peakMode</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#peakMode
   */
  public void setPeakMode(BStatusBoolean v) { set(peakMode,v,null); }

////////////////////////////////////////////////////////////////
// Property "kwhTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>kwhTotal</code> property.
   * Running KWH value (onPeak and offPeak) since last reset
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getKwhTotal
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setKwhTotal
   */
  public static final Property kwhTotal = newProperty(Flags.READONLY|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>kwhTotal</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwhTotal
   */
  public BStatusNumeric getKwhTotal() { return (BStatusNumeric)get(kwhTotal); }
  
  /**
   * Set the <code>kwhTotal</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwhTotal
   */
  public void setKwhTotal(BStatusNumeric v) { set(kwhTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "kwhOffPeakTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>kwhOffPeakTotal</code> property.
   * Running KWH value during offPeak mode since last reset
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getKwhOffPeakTotal
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setKwhOffPeakTotal
   */
  public static final Property kwhOffPeakTotal = newProperty(Flags.READONLY|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>kwhOffPeakTotal</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwhOffPeakTotal
   */
  public BStatusNumeric getKwhOffPeakTotal() { return (BStatusNumeric)get(kwhOffPeakTotal); }
  
  /**
   * Set the <code>kwhOffPeakTotal</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwhOffPeakTotal
   */
  public void setKwhOffPeakTotal(BStatusNumeric v) { set(kwhOffPeakTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "kwhOnPeakTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>kwhOnPeakTotal</code> property.
   * Running KWH value during onPeak mode since last reset
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getKwhOnPeakTotal
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setKwhOnPeakTotal
   */
  public static final Property kwhOnPeakTotal = newProperty(Flags.READONLY|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>kwhOnPeakTotal</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwhOnPeakTotal
   */
  public BStatusNumeric getKwhOnPeakTotal() { return (BStatusNumeric)get(kwhOnPeakTotal); }
  
  /**
   * Set the <code>kwhOnPeakTotal</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwhOnPeakTotal
   */
  public void setKwhOnPeakTotal(BStatusNumeric v) { set(kwhOnPeakTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "monthlyKwh"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>monthlyKwh</code> property.
   * Monthly KWH value accumulated (onPeak and offPeak)
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getMonthlyKwh
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setMonthlyKwh
   */
  public static final Property monthlyKwh = newProperty(Flags.READONLY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>monthlyKwh</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#monthlyKwh
   */
  public BStatusNumeric getMonthlyKwh() { return (BStatusNumeric)get(monthlyKwh); }
  
  /**
   * Set the <code>monthlyKwh</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#monthlyKwh
   */
  public void setMonthlyKwh(BStatusNumeric v) { set(monthlyKwh,v,null); }

////////////////////////////////////////////////////////////////
// Property "monthlyKwhOffPeak"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>monthlyKwhOffPeak</code> property.
   * Monthly kwh value accumulated during offPeak mode
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getMonthlyKwhOffPeak
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setMonthlyKwhOffPeak
   */
  public static final Property monthlyKwhOffPeak = newProperty(Flags.READONLY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>monthlyKwhOffPeak</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#monthlyKwhOffPeak
   */
  public BStatusNumeric getMonthlyKwhOffPeak() { return (BStatusNumeric)get(monthlyKwhOffPeak); }
  
  /**
   * Set the <code>monthlyKwhOffPeak</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#monthlyKwhOffPeak
   */
  public void setMonthlyKwhOffPeak(BStatusNumeric v) { set(monthlyKwhOffPeak,v,null); }

////////////////////////////////////////////////////////////////
// Property "monthlyKwhOnPeak"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>monthlyKwhOnPeak</code> property.
   * Monthly kwh value accumulated during onPeak mode
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getMonthlyKwhOnPeak
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setMonthlyKwhOnPeak
   */
  public static final Property monthlyKwhOnPeak = newProperty(Flags.READONLY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>monthlyKwhOnPeak</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#monthlyKwhOnPeak
   */
  public BStatusNumeric getMonthlyKwhOnPeak() { return (BStatusNumeric)get(monthlyKwhOnPeak); }
  
  /**
   * Set the <code>monthlyKwhOnPeak</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#monthlyKwhOnPeak
   */
  public void setMonthlyKwhOnPeak(BStatusNumeric v) { set(monthlyKwhOnPeak,v,null); }

////////////////////////////////////////////////////////////////
// Property "kwh5"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>kwh5</code> property.
   * kwh for previous 5 minute interval
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getKwh5
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setKwh5
   */
  public static final Property kwh5 = newProperty(Flags.READONLY|Flags.TRANSIENT, new BStatusNumeric(),null);
  
  /**
   * Get the <code>kwh5</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwh5
   */
  public BStatusNumeric getKwh5() { return (BStatusNumeric)get(kwh5); }
  
  /**
   * Set the <code>kwh5</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwh5
   */
  public void setKwh5(BStatusNumeric v) { set(kwh5,v,null); }

////////////////////////////////////////////////////////////////
// Property "kwh15"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>kwh15</code> property.
   * kwh for previous 15 minute interval
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getKwh15
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setKwh15
   */
  public static final Property kwh15 = newProperty(Flags.READONLY|Flags.TRANSIENT, new BStatusNumeric(),null);
  
  /**
   * Get the <code>kwh15</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwh15
   */
  public BStatusNumeric getKwh15() { return (BStatusNumeric)get(kwh15); }
  
  /**
   * Set the <code>kwh15</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwh15
   */
  public void setKwh15(BStatusNumeric v) { set(kwh15,v,null); }

////////////////////////////////////////////////////////////////
// Property "kwh30"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>kwh30</code> property.
   * kwh for previous 30 minute interval
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getKwh30
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setKwh30
   */
  public static final Property kwh30 = newProperty(Flags.READONLY|Flags.TRANSIENT, new BStatusNumeric(),null);
  
  /**
   * Get the <code>kwh30</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwh30
   */
  public BStatusNumeric getKwh30() { return (BStatusNumeric)get(kwh30); }
  
  /**
   * Set the <code>kwh30</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kwh30
   */
  public void setKwh30(BStatusNumeric v) { set(kwh30,v,null); }

////////////////////////////////////////////////////////////////
// Property "maxKw"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>maxKw</code> property.
   * peak kw observed regardless of onPeak or offPeak mode
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getMaxKw
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setMaxKw
   */
  public static final Property maxKw = newProperty(Flags.READONLY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>maxKw</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxKw
   */
  public BStatusNumeric getMaxKw() { return (BStatusNumeric)get(maxKw); }
  
  /**
   * Set the <code>maxKw</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxKw
   */
  public void setMaxKw(BStatusNumeric v) { set(maxKw,v,null); }

////////////////////////////////////////////////////////////////
// Property "maxKwTimestamp"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>maxKwTimestamp</code> property.
   * time of the observed demand peak
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getMaxKwTimestamp
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setMaxKwTimestamp
   */
  public static final Property maxKwTimestamp = newProperty(Flags.READONLY, BAbsTime.NULL,null);
  
  /**
   * Get the <code>maxKwTimestamp</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxKwTimestamp
   */
  public BAbsTime getMaxKwTimestamp() { return (BAbsTime)get(maxKwTimestamp); }
  
  /**
   * Set the <code>maxKwTimestamp</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxKwTimestamp
   */
  public void setMaxKwTimestamp(BAbsTime v) { set(maxKwTimestamp,v,null); }

////////////////////////////////////////////////////////////////
// Property "maxKwOffPeak"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>maxKwOffPeak</code> property.
   * peak kw observed during offPeak mode
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getMaxKwOffPeak
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setMaxKwOffPeak
   */
  public static final Property maxKwOffPeak = newProperty(Flags.READONLY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>maxKwOffPeak</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxKwOffPeak
   */
  public BStatusNumeric getMaxKwOffPeak() { return (BStatusNumeric)get(maxKwOffPeak); }
  
  /**
   * Set the <code>maxKwOffPeak</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxKwOffPeak
   */
  public void setMaxKwOffPeak(BStatusNumeric v) { set(maxKwOffPeak,v,null); }

////////////////////////////////////////////////////////////////
// Property "maxKwOffPeakTimestamp"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>maxKwOffPeakTimestamp</code> property.
   * time of the observed offPeak demand peak
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getMaxKwOffPeakTimestamp
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setMaxKwOffPeakTimestamp
   */
  public static final Property maxKwOffPeakTimestamp = newProperty(Flags.READONLY, BAbsTime.NULL,null);
  
  /**
   * Get the <code>maxKwOffPeakTimestamp</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxKwOffPeakTimestamp
   */
  public BAbsTime getMaxKwOffPeakTimestamp() { return (BAbsTime)get(maxKwOffPeakTimestamp); }
  
  /**
   * Set the <code>maxKwOffPeakTimestamp</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxKwOffPeakTimestamp
   */
  public void setMaxKwOffPeakTimestamp(BAbsTime v) { set(maxKwOffPeakTimestamp,v,null); }

////////////////////////////////////////////////////////////////
// Property "maxKwOnPeak"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>maxKwOnPeak</code> property.
   * peak kw observed during onPeak mode
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getMaxKwOnPeak
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setMaxKwOnPeak
   */
  public static final Property maxKwOnPeak = newProperty(Flags.READONLY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>maxKwOnPeak</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxKwOnPeak
   */
  public BStatusNumeric getMaxKwOnPeak() { return (BStatusNumeric)get(maxKwOnPeak); }
  
  /**
   * Set the <code>maxKwOnPeak</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxKwOnPeak
   */
  public void setMaxKwOnPeak(BStatusNumeric v) { set(maxKwOnPeak,v,null); }

////////////////////////////////////////////////////////////////
// Property "maxKwOnPeakTimestamp"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>maxKwOnPeakTimestamp</code> property.
   * time of the observer onPeak demand peak
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getMaxKwOnPeakTimestamp
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setMaxKwOnPeakTimestamp
   */
  public static final Property maxKwOnPeakTimestamp = newProperty(Flags.READONLY, BAbsTime.NULL,null);
  
  /**
   * Get the <code>maxKwOnPeakTimestamp</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxKwOnPeakTimestamp
   */
  public BAbsTime getMaxKwOnPeakTimestamp() { return (BAbsTime)get(maxKwOnPeakTimestamp); }
  
  /**
   * Set the <code>maxKwOnPeakTimestamp</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#maxKwOnPeakTimestamp
   */
  public void setMaxKwOnPeakTimestamp(BAbsTime v) { set(maxKwOnPeakTimestamp,v,null); }

////////////////////////////////////////////////////////////////
// Property "peakKwInterval"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>peakKwInterval</code> property.
   * configures whether the maxKwOnPeak and maxKwOffPeak
   * values are 15 or 30 minute block demand values
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getPeakKwInterval
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setPeakKwInterval
   */
  public static final Property peakKwInterval = newProperty(Flags.OPERATOR, (BDynamicEnum.make(0, BEnumRange.make(BPeakIntervalEnum.TYPE))),null);
  
  /**
   * Get the <code>peakKwInterval</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#peakKwInterval
   */
  public BEnum getPeakKwInterval() { return (BEnum)get(peakKwInterval); }
  
  /**
   * Set the <code>peakKwInterval</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#peakKwInterval
   */
  public void setPeakKwInterval(BEnum v) { set(peakKwInterval,v,null); }

////////////////////////////////////////////////////////////////
// Property "kw5"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>kw5</code> property.
   * 5 minute block demand value
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getKw5
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setKw5
   */
  public static final Property kw5 = newProperty(Flags.READONLY|Flags.TRANSIENT, new BStatusNumeric(),null);
  
  /**
   * Get the <code>kw5</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kw5
   */
  public BStatusNumeric getKw5() { return (BStatusNumeric)get(kw5); }
  
  /**
   * Set the <code>kw5</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kw5
   */
  public void setKw5(BStatusNumeric v) { set(kw5,v,null); }

////////////////////////////////////////////////////////////////
// Property "kw15"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>kw15</code> property.
   * 15 minute block demand value
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getKw15
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setKw15
   */
  public static final Property kw15 = newProperty(Flags.READONLY|Flags.TRANSIENT, new BStatusNumeric(),null);
  
  /**
   * Get the <code>kw15</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kw15
   */
  public BStatusNumeric getKw15() { return (BStatusNumeric)get(kw15); }
  
  /**
   * Set the <code>kw15</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kw15
   */
  public void setKw15(BStatusNumeric v) { set(kw15,v,null); }

////////////////////////////////////////////////////////////////
// Property "kw30"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>kw30</code> property.
   * 30 minute block demand value
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#getKw30
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#setKw30
   */
  public static final Property kw30 = newProperty(Flags.READONLY|Flags.TRANSIENT, new BStatusNumeric(),null);
  
  /**
   * Get the <code>kw30</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kw30
   */
  public BStatusNumeric getKw30() { return (BStatusNumeric)get(kw30); }
  
  /**
   * Set the <code>kw30</code> property.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#kw30
   */
  public void setKw30(BStatusNumeric v) { set(kw30,v,null); }

////////////////////////////////////////////////////////////////
// Action "presetKwh"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>presetKwh</code> action.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#presetKwh()
   */
  public static final Action presetKwh = newAction(Flags.OPERATOR,new BConsumptionProperties(),null);
  
  /**
   * Invoke the <code>presetKwh</code> action.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#presetKwh
   */
  public void presetKwh(BConsumptionProperties arg) { invoke(presetKwh,arg,null); }

////////////////////////////////////////////////////////////////
// Action "presetKw"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>presetKw</code> action.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#presetKw()
   */
  public static final Action presetKw = newAction(Flags.OPERATOR,new BDemandProperties(),null);
  
  /**
   * Invoke the <code>presetKw</code> action.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#presetKw
   */
  public void presetKw(BDemandProperties arg) { invoke(presetKw,arg,null); }

////////////////////////////////////////////////////////////////
// Action "monthlyReset"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>monthlyReset</code> action.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#monthlyReset()
   */
  public static final Action monthlyReset = newAction(0,null);
  
  /**
   * Invoke the <code>monthlyReset</code> action.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#monthlyReset
   */
  public void monthlyReset() { invoke(monthlyReset,null,null); }

////////////////////////////////////////////////////////////////
// Action "reset"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>reset</code> action.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#reset()
   */
  public static final Action reset = newAction(0,null);
  
  /**
   * Invoke the <code>reset</code> action.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#reset
   */
  public void reset() { invoke(reset,null,null); }

////////////////////////////////////////////////////////////////
// Action "timerExpired5"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>timerExpired5</code> action.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#timerExpired5()
   */
  public static final Action timerExpired5 = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>timerExpired5</code> action.
   * @see com.vykon.vykonPro.energy.BBlockDemandCalc#timerExpired5
   */
  public void timerExpired5() { invoke(timerExpired5,null,null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BBlockDemandCalc.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  public void started() throws Exception
  {
    if (!isRunning())
      return;
    
    initTicket5();
    
    // 9/25/11 changed from millis() to ticks()
    lastTicks = Clock.ticks();
    lastTicks5 = Clock.ticks();
    lastTicks15 = Clock.ticks();
    lastTicks30 = Clock.ticks();
    
    lastPulseCount = getCurrentPulseCount().getValue();
    lastPulseCount5 = getCurrentPulseCount().getValue();
    lastPulseCount15 = getCurrentPulseCount().getValue();
    lastPulseCount30 = getCurrentPulseCount().getValue();
    
    super.started();
  }

  public void stopped() throws Exception
  {
    if (ticket5 != null)
      ticket5.cancel();
    
    super.stopped();
  }

  public void clockChanged(BRelTime value)
  {
    initTicket5();
  }
  
  private void initTicket5()
  {
    BAbsTime now = Clock.time();
    
    if(ticket5 != null)
      ticket5.cancel();
    
    //always want to calculate demand at 5 minute intervals a few seconds before
    BAbsTime startTime = BAbsTime.make(now, BTime.make(0, 4, 55));
    while (!startTime.isAfter(now))
      startTime = startTime.add(BRelTime.makeMinutes(5));
    
    ticket5 = Clock.schedulePeriodically(this, startTime, BRelTime.makeMinutes(5), timerExpired5, null);
  }
  
  public BFacets getSlotFacets(Slot slot)
  {
    if( slot.equals(kwhOffPeakTotal)||
        slot.equals(kwhOnPeakTotal) ||
        slot.equals(kwh15)          ||
        slot.equals(kwh30)          ||
        slot.equals(monthlyKwhOnPeak)      ||
        slot.equals(monthlyKwhOffPeak)   )
      return getConsumptionFacets();
    else if( slot.equals(kw15) ||
             slot.equals(kw30) ||
             slot.equals(kw5)  ||
             slot.equals(maxKwOffPeak) ||
             slot.equals(maxKwOnPeak))
      return getDemandFacets();
    else if( slot.equals(kwhPerPulse))
      return getKwhPerPulseFacets();

    return super.getSlotFacets(slot);
  }
  
  public void changed(Property property, Context context) 
  {
   if( !Sys.atSteadyState() || !isRunning() )
     return;
   
   if(property.equals(currentPulseCount)) calculateTotalKwh();
   
   super.changed(property, context);
  }
  
  private int calculateDiffTime(long lastTicks)
  {
    //ticks are the number of milliseconds but independent of clock changes
    // 9/26/11 changed from Clock.millis() to Clock.ticks()
    long thisCountChangeTicks = Clock.ticks();
    long countChangePeriod = thisCountChangeTicks - lastTicks;
    
    if(countChangePeriod<=0)
    {
      setStatus(BStatus.fault);
      setLastFaultCause("Elapsed ticks indicate a negative change.");
      setLastFaultTime(Clock.time());
      return 0;
    }
    
    return (int)(countChangePeriod/1000);
  }
  
  private void calculateKw5()
  {
    elapsedSeconds = calculateDiffTime(lastTicks5);
    checkFreq = true;
    intervalKwh = calculateKwh(lastPulseCount5, elapsedSeconds);
    getKwh5().setValue(intervalKwh);
    
    if(validFreq)
    {
      getKw5().setValue( (intervalKwh / elapsedSeconds) * 3600 );
    }
    else
    {
      getKw5().setValue(0);
    }
    
    // 9/25/11 changed millis() to ticks()
    lastTicks5 = Clock.ticks();
    lastPulseCount5 = getCurrentPulseCount().getValue();
  }
  
  private void calculateKw15()
  {
    elapsedSeconds = calculateDiffTime(lastTicks15);
    intervalKwh = calculateKwh(lastPulseCount15, elapsedSeconds);
    getKwh15().setValue(intervalKwh);
    
    if(validFreq)
    {
      getKw15().setValue((intervalKwh / elapsedSeconds) * 3600);
        
      if (getPeakKwInterval().getOrdinal() == BPeakIntervalEnum.FIFTEEN_MINUTE)
      {
        checkPeakKw(getKw15().getValue());
      }
    }
    else
    {
      getKw15().setValue(0);
    }
    
    // 9/25/11 changed millis() to ticks()
    lastTicks15 = Clock.ticks();
    lastPulseCount15 = getCurrentPulseCount().getValue();
  }
  
  private void calculateKw30()
  {
    elapsedSeconds = calculateDiffTime(lastTicks30);
    intervalKwh = calculateKwh(lastPulseCount30, elapsedSeconds);
    getKwh30().setValue(intervalKwh);
    
    if(validFreq)
    {
      getKw30().setValue((intervalKwh / elapsedSeconds) * 3600);

      if (getPeakKwInterval().getOrdinal() == BPeakIntervalEnum.THIRTY_MINUTE)
      {
        checkPeakKw(getKw30().getValue());
      }
    }
    else
    {
      getKw30().setValue(0);
    }
    
    // 9/25/11 changed millis() to ticks()
    lastTicks30 = Clock.ticks();
    lastPulseCount30 = getCurrentPulseCount().getValue();
  }
  
  private void checkPeakKw(double demand)
  {
    BTime tstamp = BTime.make(BRelTime.make(Clock.time().getTimeOfDayMillis()+6000));
    
    if(demand > getMaxKw().getValue())
    {
      getMaxKw().setValue(demand);
      setMaxKwTimestamp(BAbsTime.make(Clock.time(), tstamp));
    }
    
    //if peakMode is true, then onPeak time
    if(getPeakMode().getValue())
    {
      if(demand > getMaxKwOnPeak().getValue())
      {
        getMaxKwOnPeak().setValue(demand);
        setMaxKwOnPeakTimestamp(BAbsTime.make(Clock.time(), tstamp));
      }
    }
    else
    {
      if(demand > getMaxKwOffPeak().getValue())
      {
        getMaxKwOffPeak().setValue(demand);
        setMaxKwOffPeakTimestamp(BAbsTime.make(Clock.time(), tstamp));
      }
    }
  }
  
  /*
   * returns the interval consumption between lastTicks and currentTicks
   */
  private double calculateKwh(double lastCount, int elapsedTime)
  {
    // don't process if current input is invalid.
    if ( ! getCurrentPulseCount().getStatus().isValid()  || 
          Double.isNaN(getCurrentPulseCount().getValue())||
          getCurrentPulseCount().getValue() < 0)
    {
      setStatus(BStatus.fault);
      setLastFaultCause("Current pulse counts are not valid.");
      setLastFaultTime(Clock.time());
      return 0d;
    }
    
    currentCount = getCurrentPulseCount().getValue();
    
    if ( currentCount >= lastCount )
      pulseCountDiff = currentCount - lastCount;
    else
      pulseCountDiff = ((double)getMeterRollover() - lastCount) + 1.0  + currentCount;  
    
    if(pulseCountDiff < 0)
    {
      setStatus(BStatus.fault);
      setLastFaultCause("Calculated a negative number of pulses. Consumption value not calculated.");
      setLastFaultTime(Clock.time());
      return 0d;
    }
    
    int maxPulseFreq = (int)(getMaxPulseFrequency());
    
    double pulseFreq = pulseCountDiff/elapsedTime;
    
    // 9/25/11 check pulseFreq
    validFreq = false;
    if(pulseFreq!=Double.NaN && pulseFreq!=Double.POSITIVE_INFINITY && pulseFreq!=Double.NEGATIVE_INFINITY && pulseFreq<=maxPulseFreq)
      validFreq = true;
    
    //update pulse freq every 5 minutes, but only for 5 minute calc
    if( checkFreq) 
    {
      if(validFreq) setActualPulseFrequency(pulseFreq);
      else setActualPulseFrequency(0);
      
      checkFreq = false;
    }
        
    //only validate frequency if configured to
    if(getCheckFrequency())
    {
      // To insure pulseCountDiff is not an invalid number due to power outage, etc. of the pulseCounter
      if (!validFreq)
      {
        setStatus(BStatus.fault);
        setLastFaultCause("Pulse frequency is " + pulseFreq + " which exceeds maxPulseFrequency of " + maxPulseFreq + ". Demand value not calculated.");
        setLastFaultTime(Clock.time());
        return 0d;
      }
    }
        
    kwhNew = pulseCountDiff * getKwhPerPulse();
    setStatus(BStatus.ok);
    return kwhNew;
  }
  
  private void calculateTotalKwh()
  {
    elapsedSeconds = calculateDiffTime(lastTicks);
    
    intervalKwh = calculateKwh(lastPulseCount, elapsedSeconds);
    
    lastPulseCount = currentCount;
    
    // 9/25/11 changed millis() to ticks()
    lastTicks = Clock.ticks();
  
    getKwhTotal().setValue(getKwhTotal().getValue() + intervalKwh);
    getMonthlyKwh().setValue(getMonthlyKwh().getValue() + intervalKwh);
    
    //if peakMode is true, then onPeak time
    if(getPeakMode().getValue())
    {
      getKwhOnPeakTotal().setValue(getKwhOnPeakTotal().getValue() + intervalKwh);
      getMonthlyKwhOnPeak().setValue(getMonthlyKwhOnPeak().getValue() + intervalKwh);
    }
    else
    {
      getKwhOffPeakTotal().setValue(getKwhOffPeakTotal().getValue() + intervalKwh);
      getMonthlyKwhOffPeak().setValue(getMonthlyKwhOffPeak().getValue() + intervalKwh);
    }
  }
  
  
  ////////////////////////////////////////////////
  // actions
  ///////////////////////////////////////////////
  
  /*
   * used to preset the kwh total for a new
   * component.
   */
  public void doPresetKwh(BConsumptionProperties arg)
  {
    getKwhTotal().setValue(arg.getKwhTotal());
    getKwhOffPeakTotal().setValue(arg.getKwhOffPeakTotal());
    getKwhOnPeakTotal().setValue(arg.getKwhOnPeakTotal());
    getMonthlyKwh().setValue(arg.getMonthlyKwh());
    getMonthlyKwhOffPeak().setValue(arg.getMonthlyKwhOffPeak());
    getMonthlyKwhOnPeak().setValue(arg.getMonthlyKwhOnPeak());
  }
  
  public void doPresetKw(BDemandProperties arg)
  {
    getMaxKw().setValue(arg.getMaxKw());
    setMaxKwTimestamp(arg.getMaxKwTimestamp());
    getMaxKwOffPeak().setValue(arg.getMaxKwOffPeak());
    setMaxKwOffPeakTimestamp(arg.getMaxKwOffPeakTimestamp());
    getMaxKwOnPeak().setValue(arg.getMaxKwOnPeak());
    setMaxKwOnPeakTimestamp(arg.getMaxKwOnPeakTimestamp());
  }
  public BValue getActionParameterDefault(Action action)
  {
     if (action == presetKwh) 
      //return getDefaultConsumptionProps();
       return new BConsumptionProperties();
     else if(action == presetKw)
       return new BDemandProperties();
     
     return super.getActionParameterDefault(action);
  }
  
  
  /*
   * used to reset all monthly totals and peaks
   */
  public void doMonthlyReset()
  {
    //reset monthly consumption totals
    getMonthlyKwh().setValue(0);
    getMonthlyKwhOffPeak().setValue(0);
    getMonthlyKwhOnPeak().setValue(0);
    
    //reset monthly demand peaks
    getMaxKw().setValue(0);
    setMaxKwTimestamp(BAbsTime.NULL);
    getMaxKwOffPeak().setValue(0);
    setMaxKwOffPeakTimestamp(BAbsTime.NULL);
    getMaxKwOnPeak().setValue(0);
    setMaxKwOnPeakTimestamp(BAbsTime.NULL);
  }
  
  /*
   * used to reset all totals and peaks
   */
  public void doReset()
  {
    //reset consumption totals
    getMonthlyKwh().setValue(0);
    getMonthlyKwhOffPeak().setValue(0);
    getMonthlyKwhOnPeak().setValue(0);
    getKwhTotal().setValue(0);
    getKwhOffPeakTotal().setValue(0);
    getKwhOnPeakTotal().setValue(0);
    
    //reset the actualPulseFrequency
    setActualPulseFrequency(0);
    
    //reset interval consumption
    getKwh5().setValue(0);
    getKwh15().setValue(0);
    getKwh30().setValue(0);
    
    //reset demand peaks
    getMaxKw().setValue(0);
    setMaxKwTimestamp(BAbsTime.NULL);
    getMaxKwOffPeak().setValue(0);
    setMaxKwOffPeakTimestamp(BAbsTime.NULL);
    getMaxKwOnPeak().setValue(0);
    setMaxKwOnPeakTimestamp(BAbsTime.NULL);
    
    //reset interval demand
    getKw5().setValue(0);
    getKw15().setValue(0);
    getKw30().setValue(0);
    
    //zero out accumulated consumption from demand calculations
    lastPulseCount5 = getCurrentPulseCount().getValue();
    lastPulseCount15 = getCurrentPulseCount().getValue();
    lastPulseCount30 = getCurrentPulseCount().getValue();
    lastTicks = Clock.ticks();
    lastTicks5 = Clock.ticks();
    lastTicks15 = Clock.ticks();
    lastTicks30 = Clock.ticks();
  }
      
  public void doTimerExpired5()
  {
    //figure out what interval it is
    int currentMinute = Clock.time().getMinute();
    
    switch(currentMinute)
    {
      case 4:
      case 9:
      case 19:
      case 24:
      case 34:
      case 39:
      case 49:
      case 54:
        //5 minute intervals so calculate 5 min kw
        calculateKw5();
        break;
        
      case 14:
      case 44:
        calculateKw15();
        calculateKw5();
        break;
        
      case 29:
      case 59:
        calculateKw30();
        calculateKw15();
        calculateKw5();
    }
  }
  
  ///////////////////////////////////////////////
  // local variables
  ///////////////////////////////////////////////
  
  Clock.Ticket ticket5 = null;
  
  //consumption variables
  double lastPulseCount;
  double currentCount;
  double pulseCountDiff;
  double kwhNew;
  double intervalKwh;
  
  //demand variables
  double lastPulseCount5;
  double lastPulseCount15;
  double lastPulseCount30;
  int elapsedSeconds = 0;
  long lastTicks = 0L;
  long lastTicks5 = 0L;
  long lastTicks15 = 0L;
  long lastTicks30 = 0L;
  
  boolean checkFreq = false;
  boolean validFreq = false;
  
  
}
