package com.vykon.vykonPro.energy;

import javax.baja.status.BStatusNumeric;
import javax.baja.sys.Action;
import javax.baja.sys.BAbsTime;
import javax.baja.sys.BComponent;
import javax.baja.sys.BDynamicEnum;
import javax.baja.sys.BEnum;
import javax.baja.sys.BEnumRange;
import javax.baja.sys.BFacets;
import javax.baja.sys.BIcon;
import javax.baja.sys.BRelTime;
import javax.baja.sys.BTime;
import javax.baja.sys.Clock;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Slot;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.vykon.vykonPro.enums.BCalculationModeEnum;

/**
 * BNumericDelta calculates the change in value since the last execution. The calculationMode
 * determines the frequency at which the object performs the calculation.  The modes are
 * interval which calculates at the execute period frequency; COV which calculates on a change
 * of value of the in property; and manual which calculates when the manualCalculate action is triggered.
 * 
 * The rollover property should be configured for the maximum expected value of the connected input.
 * When the connected input value rolls over to 0, the rollover value is used in the calculation.
 * 
 * 
 * @author
 * @creation 9 Nov 00
 * @version $Revision: 29$ $Date: 3/28/2005 12:40:33 PM$
 * @since Baja 1.0
 */

public class BNumericDelta
    extends BComponent
{
  /*-
   class BNumericDelta
   {
     properties
     {
       deltaFacets: BFacets
        -- These facets are applied against the out property.
        default {[ BFacets.makeNumeric() ]}
      
       delta: BStatusNumeric
          -- output value which reflects the delta (change in value) from the last execution.
          flags { readonly, summary }
          default {[ new BStatusNumeric() ]}
       
       in: BStatusNumeric
          -- Input value.
          flags { summary }
          default {[ new BStatusNumeric() ]}
              
       lastIn: double
         -- last in value reading.
         flags{ hidden }
         default {[ 0 ]}
         
       rollover: double
          -- Maximum value of in variable before it resets to zero.
          default {[ 0 ]}
          
       calculationMode: BEnum
         -- Either manual or interval mode.
         default {[ BDynamicEnum.make(BCalculationModeEnum.INTERVAL) ]}
         slotfacets{[ BFacets.makeEnum(BEnumRange.make(BCalculationModeEnum.TYPE)) ]}
         
       executePeriod: BRelTime
          -- The amount of time between executions.
          default {[ BRelTime.makeMinutes(15) ]}
          slotfacets {[ BFacets.make(BFacets.MIN, BRelTime.make(1)) ]}
          
       lastUpdateTime: BAbsTime
         -- The last time that the delta value was calculated
         flags{ readonly, defaultOnClone }
         default{[ BAbsTime.NULL ]}
          
     }
     actions
     {
       calculate()
        flags{ hidden }
        
       manualCalculate()
     }
     topics
     {
     }
   }
   -*/
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.vykon.vykonPro.energy.BNumericDelta(1999024408)1.0$ @*/
/* Generated Mon Dec 27 14:43:46 EST 2010 by Slot-o-Matic 2000 (c) Tridium, Inc. 2000 */

////////////////////////////////////////////////////////////////
// Property "deltaFacets"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>deltaFacets</code> property.
   * These facets are applied against the out property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#getDeltaFacets
   * @see com.vykon.vykonPro.energy.BNumericDelta#setDeltaFacets
   */
  public static final Property deltaFacets = newProperty(0, BFacets.makeNumeric(),null);
  
  /**
   * Get the <code>deltaFacets</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#deltaFacets
   */
  public BFacets getDeltaFacets() { return (BFacets)get(deltaFacets); }
  
  /**
   * Set the <code>deltaFacets</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#deltaFacets
   */
  public void setDeltaFacets(BFacets v) { set(deltaFacets,v,null); }

////////////////////////////////////////////////////////////////
// Property "delta"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>delta</code> property.
   * output value which reflects the delta (change in value)
   * from the last execution.
   * @see com.vykon.vykonPro.energy.BNumericDelta#getDelta
   * @see com.vykon.vykonPro.energy.BNumericDelta#setDelta
   */
  public static final Property delta = newProperty(Flags.READONLY|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>delta</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#delta
   */
  public BStatusNumeric getDelta() { return (BStatusNumeric)get(delta); }
  
  /**
   * Set the <code>delta</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#delta
   */
  public void setDelta(BStatusNumeric v) { set(delta,v,null); }

////////////////////////////////////////////////////////////////
// Property "in"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>in</code> property.
   * Input value.
   * @see com.vykon.vykonPro.energy.BNumericDelta#getIn
   * @see com.vykon.vykonPro.energy.BNumericDelta#setIn
   */
  public static final Property in = newProperty(Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>in</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#in
   */
  public BStatusNumeric getIn() { return (BStatusNumeric)get(in); }
  
  /**
   * Set the <code>in</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#in
   */
  public void setIn(BStatusNumeric v) { set(in,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastIn"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastIn</code> property.
   * last in value reading.
   * @see com.vykon.vykonPro.energy.BNumericDelta#getLastIn
   * @see com.vykon.vykonPro.energy.BNumericDelta#setLastIn
   */
  public static final Property lastIn = newProperty(Flags.HIDDEN, 0,null);
  
  /**
   * Get the <code>lastIn</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#lastIn
   */
  public double getLastIn() { return getDouble(lastIn); }
  
  /**
   * Set the <code>lastIn</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#lastIn
   */
  public void setLastIn(double v) { setDouble(lastIn,v,null); }

////////////////////////////////////////////////////////////////
// Property "rollover"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>rollover</code> property.
   * Maximum value of in variable before it resets to zero.
   * @see com.vykon.vykonPro.energy.BNumericDelta#getRollover
   * @see com.vykon.vykonPro.energy.BNumericDelta#setRollover
   */
  public static final Property rollover = newProperty(0, 0,null);
  
  /**
   * Get the <code>rollover</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#rollover
   */
  public double getRollover() { return getDouble(rollover); }
  
  /**
   * Set the <code>rollover</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#rollover
   */
  public void setRollover(double v) { setDouble(rollover,v,null); }

////////////////////////////////////////////////////////////////
// Property "calculationMode"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>calculationMode</code> property.
   * Either manual or interval mode.
   * @see com.vykon.vykonPro.energy.BNumericDelta#getCalculationMode
   * @see com.vykon.vykonPro.energy.BNumericDelta#setCalculationMode
   */
  public static final Property calculationMode = newProperty(0, BDynamicEnum.make(BCalculationModeEnum.INTERVAL),BFacets.makeEnum(BEnumRange.make(BCalculationModeEnum.TYPE)));
  
  /**
   * Get the <code>calculationMode</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#calculationMode
   */
  public BEnum getCalculationMode() { return (BEnum)get(calculationMode); }
  
  /**
   * Set the <code>calculationMode</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#calculationMode
   */
  public void setCalculationMode(BEnum v) { set(calculationMode,v,null); }

////////////////////////////////////////////////////////////////
// Property "executePeriod"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>executePeriod</code> property.
   * The amount of time between executions.
   * @see com.vykon.vykonPro.energy.BNumericDelta#getExecutePeriod
   * @see com.vykon.vykonPro.energy.BNumericDelta#setExecutePeriod
   */
  public static final Property executePeriod = newProperty(0, BRelTime.makeMinutes(15),BFacets.make(BFacets.MIN, BRelTime.make(1)));
  
  /**
   * Get the <code>executePeriod</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#executePeriod
   */
  public BRelTime getExecutePeriod() { return (BRelTime)get(executePeriod); }
  
  /**
   * Set the <code>executePeriod</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#executePeriod
   */
  public void setExecutePeriod(BRelTime v) { set(executePeriod,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastUpdateTime"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastUpdateTime</code> property.
   * The last time that the delta value was calculated
   * @see com.vykon.vykonPro.energy.BNumericDelta#getLastUpdateTime
   * @see com.vykon.vykonPro.energy.BNumericDelta#setLastUpdateTime
   */
  public static final Property lastUpdateTime = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, BAbsTime.NULL,null);
  
  /**
   * Get the <code>lastUpdateTime</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#lastUpdateTime
   */
  public BAbsTime getLastUpdateTime() { return (BAbsTime)get(lastUpdateTime); }
  
  /**
   * Set the <code>lastUpdateTime</code> property.
   * @see com.vykon.vykonPro.energy.BNumericDelta#lastUpdateTime
   */
  public void setLastUpdateTime(BAbsTime v) { set(lastUpdateTime,v,null); }

////////////////////////////////////////////////////////////////
// Action "calculate"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>calculate</code> action.
   * @see com.vykon.vykonPro.energy.BNumericDelta#calculate()
   */
  public static final Action calculate = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>calculate</code> action.
   * @see com.vykon.vykonPro.energy.BNumericDelta#calculate
   */
  public void calculate() { invoke(calculate,null,null); }

////////////////////////////////////////////////////////////////
// Action "manualCalculate"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>manualCalculate</code> action.
   * @see com.vykon.vykonPro.energy.BNumericDelta#manualCalculate()
   */
  public static final Action manualCalculate = newAction(0,null);
  
  /**
   * Invoke the <code>manualCalculate</code> action.
   * @see com.vykon.vykonPro.energy.BNumericDelta#manualCalculate
   */
  public void manualCalculate() { invoke(manualCalculate,null,null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BNumericDelta.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  public BFacets getSlotFacets(Slot slot)
  {
    if (slot.equals(delta))
      return getDeltaFacets();
    return super.getSlotFacets(slot);

  }

  // //////////////////////////////////////////////////////////////
  // Initialization
  // //////////////////////////////////////////////////////////////

  public void started() throws Exception
  {
    if(getCalculationMode().getOrdinal()==(BCalculationModeEnum.INTERVAL)) initClockTicket();

    super.started();

    if (!Sys.atSteadyState())
      return;
  }

  public void stopped() throws Exception
  {
    if (ticket != null)
      ticket.cancel();
    super.stopped();
  }

  public void clockChanged(BRelTime value)
  {
    if(getCalculationMode().getOrdinal()==(BCalculationModeEnum.INTERVAL)) initClockTicket();
  }

  private void initClockTicket()
  {
    if (ticket != null)
      ticket.cancel();
    
    BAbsTime now = Clock.time();
    BAbsTime startTime = BAbsTime.make(now, BTime.make(0, 0, 0));
    while (!startTime.isAfter(now))
      startTime = startTime.add(getExecutePeriod());
    
    ticket = Clock.schedulePeriodically(this, startTime, getExecutePeriod(), calculate, null);
  }

  public void changed(Property property, Context context)
  {
    super.changed(property, context);
    if (!Sys.atSteadyState() || !isRunning())
      return;

    if (property.equals(executePeriod))
    {
      if(getCalculationMode().getOrdinal()==(BCalculationModeEnum.INTERVAL)) initClockTicket();
    }
    
    if (property.equals(calculationMode))
    {
      if(getCalculationMode().getOrdinal()==(BCalculationModeEnum.INTERVAL)) initClockTicket();
      else
      {
        if (ticket != null) ticket.cancel();
      }
    }

    //check if the calculationMode is set to cov
    if(getCalculationMode().getOrdinal()==(BCalculationModeEnum.COV))
    {
      if(property.equals(in)) calculate();
    }
  }

  public void doCalculate() throws Exception
  {
    if (getIn().getStatus().isValid())
    {
      if (getIn().getValue() >= getLastIn())
      {
        getDelta().setValue(getIn().getValue() - getLastIn());
        setLastIn(getIn().getValue());
      }
      else if (getIn().getValue() < getLastIn())
      {
        getDelta().setValue(getRollover() - getLastIn() + getIn().getValue());
        setLastIn(getIn().getValue());
      }
    }
    //if(getCalculationMode().getOrdinal()==(BCalculationModeEnum.INTERVAL)) initClockTicket();
    setLastUpdateTime(Clock.time());
  }
  
   
  public void doManualCalculate()
  {
    calculate();
  }
  
  // //////////////////////////////////////////////////////////////
  // Attributes
  // //////////////////////////////////////////////////////////////

  
  Clock.Ticket ticket = null;

  // //////////////////////////////////////////////////////////////
  // Icon
  // //////////////////////////////////////////////////////////////

  /**
   * Get the icon.
   */
  public BIcon getIcon()
  {
    return icon;
  }

  private static final BIcon icon = BIcon.std("delta.png");

}
