package com.vykon.vykonPro.history;

import java.io.IOException;
import java.util.logging.Level;

import javax.baja.control.BControlPoint;
import javax.baja.control.BNumericPoint;
import javax.baja.history.BCollectionInterval;
import javax.baja.history.BHistoryConfig;
import javax.baja.history.BHistoryRecord;
import javax.baja.history.BNumericTrendRecord;
import javax.baja.history.BRolloverValue;
import javax.baja.history.ext.BHistoryExt;
import javax.baja.status.BStatus;
import javax.baja.status.BStatusNumeric;
import javax.baja.status.BStatusValue;
import javax.baja.sys.Action;
import javax.baja.sys.BAbsTime;
import javax.baja.sys.BComponent;
import javax.baja.sys.BFacets;
import javax.baja.sys.BInteger;
import javax.baja.sys.BRelTime;
import javax.baja.sys.BString;
import javax.baja.sys.Clock;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.tridium.history.BNumeric64BitTrendRecord;

/**
 * 
 *  BManualNumericHistoryExt provides an action named "Manual Collect" which causes the parent
 *  control point's out value to be written to a record in the configured history. If the external
 *  logic used to trigger the action occurs at irregular intervals, set the interval property to 
 *  irregular. Otherwise the interval value should be configured to match the frequency of the 
 *  external logic.
 *  
 *  <p>
 *  Note that when executing a "Manual Collect" that at least 1/2 of the configured interval time 
 *  must have elapsed since the last record was written to the history. This limitation is enforced 
 *  by the BHistoryExt superclass.
 *
 * @author E351261
 * @creation Jan 19, 2012
 *
 */

public class BManualNumericHistoryExt
    extends BHistoryExt
{
  /*-
   class BManualNumericHistoryExt
   {
     properties
     {
       interval: BRelTime
        -- The amount of time between records. If triggering the manualCollect action using a fixed trigger
        -- the interval should be set to the same value as the source trigger. If triggering the manualCollect
        -- action using logic with a variable execution frequency set, the interval should be set to irregular.
        default {[ BRelTime.makeMinutes(15) ]}
        slotfacets {[ BFacets.make(BFacets.MIN, BRelTime.make(1)) ]}
        
        precision: int
        -- Specifies the precision of the persistently stored numeric values.
        default {[ 32 ]}
        slotfacets {[ BFacets.make(BFacets.FIELD_EDITOR, BString.make("history:PrecisionFE")) ]}
      
       minRolloverValue: BRolloverValue
        -- Specifies the minimum value for this logged point if applicable.
        -- Used as a reference when reporting delta log values.
        default {[ new BRolloverValue() ]}
        
       maxRolloverValue: BRolloverValue
        -- Specifies the maximum value for this logged point if applicable.
        -- Used as a reference when reporting delta log values.
        default {[ new BRolloverValue() ]}
     }
     actions
     {
       manualCollect()
     }
     topics
     {
     }
   }
   -*/
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.vykon.vykonPro.history.BManualNumericHistoryExt(2403131518)1.0$ @*/
/* Generated Thu Jan 19 14:24:54 EST 2012 by Slot-o-Matic 2000 (c) Tridium, Inc. 2000 */

////////////////////////////////////////////////////////////////
// Property "interval"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>interval</code> property.
   * The amount of time between records. If triggering the manualCollect action using a fixed trigger the interval should be set to the same value as the source trigger. If triggering the manualCollect action using logic with a variable execution frequency set, the interval should be set to irregular.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#getInterval
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#setInterval
   */
  public static final Property interval = newProperty(0, BRelTime.makeMinutes(15),BFacets.make(BFacets.MIN, BRelTime.make(1)));
  
  /**
   * Get the <code>interval</code> property.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#interval
   */
  public BRelTime getInterval() { return (BRelTime)get(interval); }
  
  /**
   * Set the <code>interval</code> property.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#interval
   */
  public void setInterval(BRelTime v) { set(interval,v,null); }

////////////////////////////////////////////////////////////////
// Property "precision"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>precision</code> property.
   * Specifies the precision of the persistently stored
   * numeric values.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#getPrecision
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#setPrecision
   */
  public static final Property precision = newProperty(0, 32,BFacets.make(BFacets.FIELD_EDITOR, BString.make("history:PrecisionFE")));
  
  /**
   * Get the <code>precision</code> property.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#precision
   */
  public int getPrecision() { return getInt(precision); }
  
  /**
   * Set the <code>precision</code> property.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#precision
   */
  public void setPrecision(int v) { setInt(precision,v,null); }

////////////////////////////////////////////////////////////////
// Property "minRolloverValue"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>minRolloverValue</code> property.
   * Specifies the minimum value for this logged point if applicable. Used as a reference when reporting delta log values.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#getMinRolloverValue
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#setMinRolloverValue
   */
  public static final Property minRolloverValue = newProperty(0, new BRolloverValue(),null);
  
  /**
   * Get the <code>minRolloverValue</code> property.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#minRolloverValue
   */
  public BRolloverValue getMinRolloverValue() { return (BRolloverValue)get(minRolloverValue); }
  
  /**
   * Set the <code>minRolloverValue</code> property.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#minRolloverValue
   */
  public void setMinRolloverValue(BRolloverValue v) { set(minRolloverValue,v,null); }

////////////////////////////////////////////////////////////////
// Property "maxRolloverValue"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>maxRolloverValue</code> property.
   * Specifies the maximum value for this logged point if applicable. Used as a reference when reporting delta log values.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#getMaxRolloverValue
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#setMaxRolloverValue
   */
  public static final Property maxRolloverValue = newProperty(0, new BRolloverValue(),null);
  
  /**
   * Get the <code>maxRolloverValue</code> property.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#maxRolloverValue
   */
  public BRolloverValue getMaxRolloverValue() { return (BRolloverValue)get(maxRolloverValue); }
  
  /**
   * Set the <code>maxRolloverValue</code> property.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#maxRolloverValue
   */
  public void setMaxRolloverValue(BRolloverValue v) { set(maxRolloverValue,v,null); }

////////////////////////////////////////////////////////////////
// Action "manualCollect"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>manualCollect</code> action.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#manualCollect()
   */
  public static final Action manualCollect = newAction(0,null);
  
  /**
   * Invoke the <code>manualCollect</code> action.
   * @see com.vykon.vykonPro.history.BManualNumericHistoryExt#manualCollect
   */
  public void manualCollect() { invoke(manualCollect,null,null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BManualNumericHistoryExt.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  /**
   * Extensions may only be placed in BControlPoints.
   */
  public boolean isParentLegal(BComponent parent)
  {
    return parent instanceof BNumericPoint;
  }
  
  
  public void started() throws Exception
  {
    super.started();

    // make sure the intervals match at startup
    if (isValidInterval(getInterval()))
    {
      BCollectionInterval collectionInterval = BCollectionInterval.make(getInterval());
      if (!getHistoryConfig().getInterval().equals(collectionInterval))
        getHistoryConfig().setInterval(collectionInterval);
    }
    
    if (getPrecision() == 64)
      rec = new BNumeric64BitTrendRecord();
    else
      rec = new BNumericTrendRecord();
    syncProperties();
  }
  
  public void activated(BAbsTime startTime, BAbsTime currentTime, BStatusValue value)
                                                                                     throws IOException
  {
    if (log.isLoggable(Level.FINE))
    {
      String s =
          getSlotPath() + ": activated (start=" +
              startTime.toString(BHistoryRecord.TIMESTAMP_FACETS) + ", current=" +
              currentTime.toString(BHistoryRecord.TIMESTAMP_FACETS) + ")";
      log.fine(s);
    }

    // TODO don't need to do anything since this isn't cov or interval
  }

  public void deactivated(BAbsTime currentTime, BStatusValue value)
  {
    //TODO perhaps should write a final record, but probably not
  }
  
  
  /**
   * Handle a property change.
   */
  public void changed(Property p, Context cx)
  {
    super.changed(p, cx);
    if (cx == Context.decoding) return;
    if (!isRunning()) return;
    
    if (p.equals(interval))
    {
      if (!isValidInterval(getInterval())) return;
      getHistoryConfig().setInterval(BCollectionInterval.make(getInterval()));
    }
    else if (p.equals(minRolloverValue))
    {
      BHistoryConfig config = getHistoryConfig();
      Property prop = config.loadSlots().getProperty(minRolloverValue.getName());
      if (prop == null)
        config.add(minRolloverValue.getName(), getMinRolloverValue().newCopy(), Flags.READONLY);
      else
        config.set(prop, getMinRolloverValue().newCopy());
    }
    else if (p.equals(maxRolloverValue))
    {
      BHistoryConfig config = getHistoryConfig();
      Property prop = config.loadSlots().getProperty(maxRolloverValue.getName());
      if (prop == null)
        config.add(maxRolloverValue.getName(), getMaxRolloverValue().newCopy(), Flags.READONLY);
      else
        config.set(prop, getMaxRolloverValue().newCopy());
    }
    else if (p.equals(precision))
    {
      BHistoryConfig config = getHistoryConfig();
      Property prop = config.loadSlots().getProperty(precision.getName());
      if (prop == null)
        config.add(precision.getName(), BInteger.make(getPrecision()), Flags.READONLY, BFacets.make(BFacets.FIELD_EDITOR, BString.make("history:PrecisionFE")), null);
      else
        config.set(prop, BInteger.make(getPrecision()));
      
      if (getPrecision() == 64)
        rec = new BNumeric64BitTrendRecord();
      else
        rec = new BNumericTrendRecord();
        
      if (!config.getRecordType().equals(getRecordType().getTypeSpec()))
        config.setRecordType(getRecordType().getTypeSpec());
    }
  }
  
  /**
   * Write a record for the specified timestamp and value.
   */
  protected void writeRecord(BAbsTime timestamp, BStatusValue out)
    throws IOException
  {
    append(rec.set(timestamp,
                   ((BStatusNumeric)out).getValue(),
                   ((BStatusNumeric)out).getStatus())); 
  }
  
  /**
   * Update the status for this extension.
   */
  public void updateStatus()
  {
    super.updateStatus();
    
    // if I'm already in fault, then let it ride
    if (getStatus().isFault()) return;

    if (!isValidInterval(getInterval()))
    {
      BStatus newStatus = BStatus.makeFault(getStatus(), true);
      if (!getStatus().equals(newStatus))
      {
        setStatus(newStatus);
        setFaultCause("Invalid interval: " + getInterval().toString());
      }
    }
    else
    {
      setFaultCause("");
    }
  }
  
  /**
   * Test the validity of the current interval.
   */
  private boolean isValidInterval(BRelTime interval)
  {
    return interval.getMillis() > 0;
  }

  /**
   * Synchronize the extension's history config with the special extension
   * properties (Min/Max Rollover Values).
   */
  private void syncProperties()
  {
    BHistoryConfig config = getHistoryConfig();
    
    Property prop = config.loadSlots().getProperty(minRolloverValue.getName());
    if (prop == null)
      config.add(minRolloverValue.getName(), getMinRolloverValue().newCopy(), Flags.READONLY);
    else
    {
      BRolloverValue configMin = (BRolloverValue)config.get(prop);
      if (!configMin.equivalent(getMinRolloverValue()))
        config.set(prop, getMinRolloverValue().newCopy());
    }
    
    prop = config.loadSlots().getProperty(maxRolloverValue.getName());
    if (prop == null)
      config.add(maxRolloverValue.getName(), getMaxRolloverValue().newCopy(), Flags.READONLY);
    else
    {
      BRolloverValue configMax = (BRolloverValue)config.get(prop);
      if (!configMax.equivalent(getMaxRolloverValue()))
        config.set(prop, getMaxRolloverValue().newCopy());
    }
    
    prop = config.loadSlots().getProperty(precision.getName());
    if (prop == null)
      config.add(precision.getName(), BInteger.make(getPrecision()), Flags.READONLY, BFacets.make(BFacets.FIELD_EDITOR, BString.make("history:PrecisionFE")), null);
    else
    {
      BInteger configPrecision = (BInteger)config.get(prop);
      if (configPrecision.getInt() != getPrecision())
        config.set(prop, BInteger.make(getPrecision()));
    }
  }

  public Type getRecordType()
  {
    if (getPrecision() == 64)
      return BNumeric64BitTrendRecord.TYPE;
    else
      return BNumericTrendRecord.TYPE;
  }

  public void pointChanged(BAbsTime arg0, BStatusValue arg1) throws Exception
  {
    // TODO shouldn't need to do anything since not doing cov
  }
  
  //////////////////////////////////////////////////////////////
  // Actions
  //////////////////////////////////////////////////////////////
  
  public void doManualCollect()
  {
    BControlPoint point = getParentPoint();
    BStatusValue val = (BStatusValue) point.getOutStatusValue().newCopy(true); 
    
    try
    {
      writeRecord(Clock.time(), val);
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

////////////////////////////////////////////////////////////////
//Attributes
////////////////////////////////////////////////////////////////

 private BNumericTrendRecord rec;
 
}
