package com.vykon.vykonPro.history;

import java.io.IOException;

import javax.baja.collection.BITable;
import javax.baja.collection.Column;
import javax.baja.collection.ColumnList;
import javax.baja.collection.TableCursor;
import javax.baja.history.BCollectionInterval;
import javax.baja.history.BHistoryConfig;
import javax.baja.history.BIHistory;
import javax.baja.history.BTrendFlags;
import javax.baja.naming.BOrd;
import javax.baja.naming.SlotPath;
import javax.baja.status.BStatus;
import javax.baja.status.BStatusNumeric;
import javax.baja.sys.Action;
import javax.baja.sys.BAbsTime;
import javax.baja.sys.BComponent;
import javax.baja.sys.BDouble;
import javax.baja.sys.BDynamicEnum;
import javax.baja.sys.BEnum;
import javax.baja.sys.BEnumRange;
import javax.baja.sys.BFacets;
import javax.baja.sys.BINumeric;
import javax.baja.sys.BIcon;
import javax.baja.sys.BMonth;
import javax.baja.sys.BRelTime;
import javax.baja.sys.BTime;
import javax.baja.sys.BValue;
import javax.baja.sys.BWeekday;
import javax.baja.sys.Clock;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Slot;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;
import javax.baja.util.BFormat;
import javax.baja.util.IFuture;
import javax.baja.util.Invocation;

import com.vykon.vykonPro.enums.BDateModeEnum;
import com.vykon.vykonPro.util.BVykonProService;

 /** BNumericHistoryAnalysis is used to calculate the total,min and max for various periods for 
  * the specified numeric history.  This object posts work to the BHistoryAnalysisService
  * which must be added to the stations services container.
  * 
  * Calculations are performed at the configured executePeriod.
  * 
  * If the history is stored as delta values then set the deltaLogged property to true, else leave
  * the deltaLogged value at the default of false.
 *
 *
 * @author    
 * @creation  1 May 2008
 * @version   $Revision: $ $Date: $
 * @since     Baja 1.0
 */
 
public class BNumericHistoryAnalysis
  extends BComponent
{ 
  /*-
  
  class BNumericHistoryAnalysis
  {
    properties
    {
      status: BStatus
           -- The status of the component
           flags { readonly, defaultOnClone }
           default {[ BStatus.ok ]}
           
      faultCause: String
           -- The fault cause.
           flags { readonly, defaultOnClone }
           default {[ "" ]}
           
      historyName: BFormat
      --String which when resolved should be the historyId which will be queried.
      --%componentSpace.rootComponent.stationName% will resolve the station name.
      flags{ }
      default {[ BFormat.make("%componentSpace.rootComponent.stationName%/%parent.name%") ]}
      
      historyOrd: BOrd
        --ord which resolves to the history id which is used for the bql query
        flags{ readonly, defaultOnClone }
        default{[ BOrd.DEFAULT ]}
      
      historyInterval: BRelTime
        -- configured interval of the referenced history
        flags{ hidden, readonly, defaultOnClone }
        default{[ BRelTime.DEFAULT ]}
        
      facets: BFacets
        -- These facets are applied against the out properties.
        default {[ BFacets.makeNumeric() ]}
      
      deltaLogged: boolean
        --boolean property to control bql query.  Set to true if history is logged in a delta format.
        default{[ false ]}
      
      excludeInvalidValues: boolean
        --boolean property if true will exclude values from calculation if the status in the history
        --record returns false for the BStatus.isValid() check.  This means that the status includes
        --a bit for disabled, fault, down, stale or null.
        default{[ true ]}
        slotfacets{[BFacets.makeBoolean("exclude", "include")]}
         
      excludeHiddenValues: boolean
        --boolean property if true will exclude values from calculation if the trend flag for the
        --record has the hidden bit set.
        default{[ true ]}
        slotfacets{[BFacets.makeBoolean("exclude", "include")]}
         
      dateMode: BEnum
         -- The mode determines the dates used for the monthly and yearly queries.
         flags{ operator }
         default{[ ( BDynamicEnum.make(0, BEnumRange.make(BDateModeEnum.TYPE))) ]}
         
       billingCycleStartDay: int
         -- The day of the month which the billing cycle starts.
         flags{ operator }
         default{[ 1 ]}
         slotfacets{[ BFacets.makeInt(null, 1, 28) ]}
         
      executePeriod: BRelTime
        --time period at which the object performs calculations
        flags{  }
        default{[ BRelTime.makeHours(1) ]}
      
      calculating: boolean
         -- Readonly boolean property that indicates if the component is actively calculating values
         flags{ readonly, defaultOnClone }
         default{[ false ]}
       
      nextUpdateTime: BAbsTime
         -- The last time that the calculate method updated the calculated values
         flags{ readonly, defaultOnClone }
         default{[ BAbsTime.NULL ]}
          
       lastUpdateTime: BAbsTime
         -- The last time that the calculate method updated the calculated values
         flags{ readonly, defaultOnClone }
         default{[ BAbsTime.NULL ]}
         
      currentHourTotal: BStatusNumeric
        --total for current hour
        flags{ readonly, defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
        
      currentHourAvg: BStatusNumeric
        --average for the current hour
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentHourMin: BStatusNumeric
        --min value for current hour
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentHourMax: BStatusNumeric
        --max value for current hour
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      lastHourTotal: BStatusNumeric
        --total for last hour
        flags{ readonly, defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
        
      lastHourAvg: BStatusNumeric
        --average for the last hour
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      lastHourMin: BStatusNumeric
        --min value for last hour
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      lastHourMax: BStatusNumeric
        --max value for last hour
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      
      currentDayTotal: BStatusNumeric
        --total for current day
        flags{ readonly, defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
        
      currentDayAvg: BStatusNumeric
        --average for the current day
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentDayMin: BStatusNumeric
        --min value for current day
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentDayMax: BStatusNumeric
        --max value for current day
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      yesterdayTotal: BStatusNumeric
        --total from previous day
        flags{ readonly,defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
      
      yesterdayAvg: BStatusNumeric
        --average for yesterday
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      yesterdayMin: BStatusNumeric
        --min value from previous day
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      yesterdayMax: BStatusNumeric
        --max value from previous day
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentWeekTotal: BStatusNumeric
        --total for current week
        flags{ readonly,defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentWeekAvg: BStatusNumeric
        --average for the current week
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentWeekMin: BStatusNumeric
        --min value for current week
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentWeekMax: BStatusNumeric
        --max value for current week
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      lastWeekTotal: BStatusNumeric
        --total for previous week
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      lastWeekAvg: BStatusNumeric
        --average for last week
        flags{ readonly,defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
      
      lastWeekMin: BStatusNumeric
        --min value for previous week
        flags{ readonly,defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
      
      lastWeekMax: BStatusNumeric
        --max value for previous week
        flags{ readonly,defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentMonthTotal: BStatusNumeric
        --total for current month
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentMonthAvg: BStatusNumeric
        --average for the current month
        flags{ readonly,defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentMonthMin: BStatusNumeric
        --min value for current month
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentMonthMax: BStatusNumeric
        --max value for current month
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
        
      lastMonthTotal: BStatusNumeric
        --total for previous month
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      lastMonthAvg: BStatusNumeric
        --average for last month
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      lastMonthMin: BStatusNumeric
        --min value for previous month
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      lastMonthMax: BStatusNumeric
        --max value for previous month
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentYearTotal: BStatusNumeric
        --total for current year
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentYearAvg: BStatusNumeric
        --average for the current year
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentYearMin: BStatusNumeric
        --min value for current year
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
      
      currentYearMax: BStatusNumeric
        --max value for current year
        flags{ readonly,defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
        
      lastYearTotal: BStatusNumeric
        --total for previous month
        flags{ readonly, defaultOnClone,transient, summary }
        default{[ new BStatusNumeric() ]}
        
      lastYearAvg: BStatusNumeric
        --average for last year
        flags{ readonly,defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
        
      lastYearMin: BStatusNumeric
        --min value for previous month
        flags{ readonly,defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
        
      lastYearMax: BStatusNumeric
        --max value for previous month
        flags{ readonly,defaultOnClone, transient, summary }
        default{[ new BStatusNumeric() ]}
    }
    
    actions
    {
    
      calculate()
      reset()
      query()
        flags{ async, hidden }
      
    }
  }
  
  -*/
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.vykon.vykonPro.history.BNumericHistoryAnalysis(3353891641)1.0$ @*/
/* Generated Sat Mar 19 17:46:05 EDT 2011 by Slot-o-Matic 2000 (c) Tridium, Inc. 2000 */

////////////////////////////////////////////////////////////////
// Property "status"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>status</code> property.
   * The status of the component
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getStatus
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setStatus
   */
  public static final Property status = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, BStatus.ok,null);
  
  /**
   * Get the <code>status</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#status
   */
  public BStatus getStatus() { return (BStatus)get(status); }
  
  /**
   * Set the <code>status</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#status
   */
  public void setStatus(BStatus v) { set(status,v,null); }

////////////////////////////////////////////////////////////////
// Property "faultCause"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>faultCause</code> property.
   * The fault cause.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getFaultCause
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setFaultCause
   */
  public static final Property faultCause = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, "",null);
  
  /**
   * Get the <code>faultCause</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#faultCause
   */
  public String getFaultCause() { return getString(faultCause); }
  
  /**
   * Set the <code>faultCause</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#faultCause
   */
  public void setFaultCause(String v) { setString(faultCause,v,null); }

////////////////////////////////////////////////////////////////
// Property "historyName"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>historyName</code> property.
   * String which when resolved should be the historyId
   * which will be queried. %componentSpace.rootComponent.stationName%
   * will resolve the station name.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getHistoryName
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setHistoryName
   */
  public static final Property historyName = newProperty(0, BFormat.make("%componentSpace.rootComponent.stationName%/%parent.name%"),null);
  
  /**
   * Get the <code>historyName</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#historyName
   */
  public BFormat getHistoryName() { return (BFormat)get(historyName); }
  
  /**
   * Set the <code>historyName</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#historyName
   */
  public void setHistoryName(BFormat v) { set(historyName,v,null); }

////////////////////////////////////////////////////////////////
// Property "historyOrd"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>historyOrd</code> property.
   * ord which resolves to the history id which is used
   * for the bql query
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getHistoryOrd
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setHistoryOrd
   */
  public static final Property historyOrd = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, BOrd.DEFAULT,null);
  
  /**
   * Get the <code>historyOrd</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#historyOrd
   */
  public BOrd getHistoryOrd() { return (BOrd)get(historyOrd); }
  
  /**
   * Set the <code>historyOrd</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#historyOrd
   */
  public void setHistoryOrd(BOrd v) { set(historyOrd,v,null); }

////////////////////////////////////////////////////////////////
// Property "historyInterval"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>historyInterval</code> property.
   * configured interval of the referenced history
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getHistoryInterval
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setHistoryInterval
   */
  public static final Property historyInterval = newProperty(Flags.HIDDEN|Flags.READONLY|Flags.DEFAULT_ON_CLONE, BRelTime.DEFAULT,null);
  
  /**
   * Get the <code>historyInterval</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#historyInterval
   */
  public BRelTime getHistoryInterval() { return (BRelTime)get(historyInterval); }
  
  /**
   * Set the <code>historyInterval</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#historyInterval
   */
  public void setHistoryInterval(BRelTime v) { set(historyInterval,v,null); }

////////////////////////////////////////////////////////////////
// Property "facets"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>facets</code> property.
   * These facets are applied against the out properties.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getFacets
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setFacets
   */
  public static final Property facets = newProperty(0, BFacets.makeNumeric(),null);
  
  /**
   * Get the <code>facets</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#facets
   */
  public BFacets getFacets() { return (BFacets)get(facets); }
  
  /**
   * Set the <code>facets</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#facets
   */
  public void setFacets(BFacets v) { set(facets,v,null); }

////////////////////////////////////////////////////////////////
// Property "deltaLogged"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>deltaLogged</code> property.
   * boolean property to control bql query.  Set to true
   * if history is logged in a delta format.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getDeltaLogged
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setDeltaLogged
   */
  public static final Property deltaLogged = newProperty(0, false,null);
  
  /**
   * Get the <code>deltaLogged</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#deltaLogged
   */
  public boolean getDeltaLogged() { return getBoolean(deltaLogged); }
  
  /**
   * Set the <code>deltaLogged</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#deltaLogged
   */
  public void setDeltaLogged(boolean v) { setBoolean(deltaLogged,v,null); }

////////////////////////////////////////////////////////////////
// Property "excludeInvalidValues"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>excludeInvalidValues</code> property.
   * boolean property if true will exclude values from calculation if the status in the history record returns false for the BStatus.isValid() check.  This means that the status includes a bit for disabled, fault, down, stale or null.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getExcludeInvalidValues
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setExcludeInvalidValues
   */
  public static final Property excludeInvalidValues = newProperty(0, true,BFacets.makeBoolean("exclude", "include"));
  
  /**
   * Get the <code>excludeInvalidValues</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#excludeInvalidValues
   */
  public boolean getExcludeInvalidValues() { return getBoolean(excludeInvalidValues); }
  
  /**
   * Set the <code>excludeInvalidValues</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#excludeInvalidValues
   */
  public void setExcludeInvalidValues(boolean v) { setBoolean(excludeInvalidValues,v,null); }

////////////////////////////////////////////////////////////////
// Property "excludeHiddenValues"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>excludeHiddenValues</code> property.
   * boolean property if true will exclude values from calculation if the trend flag for the record has the hidden bit set.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getExcludeHiddenValues
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setExcludeHiddenValues
   */
  public static final Property excludeHiddenValues = newProperty(0, true,BFacets.makeBoolean("exclude", "include"));
  
  /**
   * Get the <code>excludeHiddenValues</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#excludeHiddenValues
   */
  public boolean getExcludeHiddenValues() { return getBoolean(excludeHiddenValues); }
  
  /**
   * Set the <code>excludeHiddenValues</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#excludeHiddenValues
   */
  public void setExcludeHiddenValues(boolean v) { setBoolean(excludeHiddenValues,v,null); }

////////////////////////////////////////////////////////////////
// Property "dateMode"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>dateMode</code> property.
   * The mode determines the dates used for the monthly
   * and yearly queries.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getDateMode
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setDateMode
   */
  public static final Property dateMode = newProperty(Flags.OPERATOR, ( BDynamicEnum.make(0, BEnumRange.make(BDateModeEnum.TYPE))),null);
  
  /**
   * Get the <code>dateMode</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#dateMode
   */
  public BEnum getDateMode() { return (BEnum)get(dateMode); }
  
  /**
   * Set the <code>dateMode</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#dateMode
   */
  public void setDateMode(BEnum v) { set(dateMode,v,null); }

////////////////////////////////////////////////////////////////
// Property "billingCycleStartDay"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>billingCycleStartDay</code> property.
   * The day of the month which the billing cycle starts.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getBillingCycleStartDay
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setBillingCycleStartDay
   */
  public static final Property billingCycleStartDay = newProperty(Flags.OPERATOR, 1,BFacets.makeInt(null, 1, 28));
  
  /**
   * Get the <code>billingCycleStartDay</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#billingCycleStartDay
   */
  public int getBillingCycleStartDay() { return getInt(billingCycleStartDay); }
  
  /**
   * Set the <code>billingCycleStartDay</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#billingCycleStartDay
   */
  public void setBillingCycleStartDay(int v) { setInt(billingCycleStartDay,v,null); }

////////////////////////////////////////////////////////////////
// Property "executePeriod"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>executePeriod</code> property.
   * time period at which the object performs calculations
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getExecutePeriod
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setExecutePeriod
   */
  public static final Property executePeriod = newProperty(0, BRelTime.makeHours(1),null);
  
  /**
   * Get the <code>executePeriod</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#executePeriod
   */
  public BRelTime getExecutePeriod() { return (BRelTime)get(executePeriod); }
  
  /**
   * Set the <code>executePeriod</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#executePeriod
   */
  public void setExecutePeriod(BRelTime v) { set(executePeriod,v,null); }

////////////////////////////////////////////////////////////////
// Property "calculating"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>calculating</code> property.
   * Readonly boolean property that indicates if the component
   * is actively calculating values
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCalculating
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCalculating
   */
  public static final Property calculating = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, false,null);
  
  /**
   * Get the <code>calculating</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#calculating
   */
  public boolean getCalculating() { return getBoolean(calculating); }
  
  /**
   * Set the <code>calculating</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#calculating
   */
  public void setCalculating(boolean v) { setBoolean(calculating,v,null); }

////////////////////////////////////////////////////////////////
// Property "nextUpdateTime"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>nextUpdateTime</code> property.
   * The last time that the calculate method updated the
   * calculated values
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getNextUpdateTime
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setNextUpdateTime
   */
  public static final Property nextUpdateTime = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, BAbsTime.NULL,null);
  
  /**
   * Get the <code>nextUpdateTime</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#nextUpdateTime
   */
  public BAbsTime getNextUpdateTime() { return (BAbsTime)get(nextUpdateTime); }
  
  /**
   * Set the <code>nextUpdateTime</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#nextUpdateTime
   */
  public void setNextUpdateTime(BAbsTime v) { set(nextUpdateTime,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastUpdateTime"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastUpdateTime</code> property.
   * The last time that the calculate method updated the
   * calculated values
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastUpdateTime
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastUpdateTime
   */
  public static final Property lastUpdateTime = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, BAbsTime.NULL,null);
  
  /**
   * Get the <code>lastUpdateTime</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastUpdateTime
   */
  public BAbsTime getLastUpdateTime() { return (BAbsTime)get(lastUpdateTime); }
  
  /**
   * Set the <code>lastUpdateTime</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastUpdateTime
   */
  public void setLastUpdateTime(BAbsTime v) { set(lastUpdateTime,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentHourTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentHourTotal</code> property.
   * total for current hour
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentHourTotal
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentHourTotal
   */
  public static final Property currentHourTotal = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentHourTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentHourTotal
   */
  public BStatusNumeric getCurrentHourTotal() { return (BStatusNumeric)get(currentHourTotal); }
  
  /**
   * Set the <code>currentHourTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentHourTotal
   */
  public void setCurrentHourTotal(BStatusNumeric v) { set(currentHourTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentHourAvg"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentHourAvg</code> property.
   * average for the current hour
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentHourAvg
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentHourAvg
   */
  public static final Property currentHourAvg = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentHourAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentHourAvg
   */
  public BStatusNumeric getCurrentHourAvg() { return (BStatusNumeric)get(currentHourAvg); }
  
  /**
   * Set the <code>currentHourAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentHourAvg
   */
  public void setCurrentHourAvg(BStatusNumeric v) { set(currentHourAvg,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentHourMin"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentHourMin</code> property.
   * min value for current hour
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentHourMin
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentHourMin
   */
  public static final Property currentHourMin = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentHourMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentHourMin
   */
  public BStatusNumeric getCurrentHourMin() { return (BStatusNumeric)get(currentHourMin); }
  
  /**
   * Set the <code>currentHourMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentHourMin
   */
  public void setCurrentHourMin(BStatusNumeric v) { set(currentHourMin,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentHourMax"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentHourMax</code> property.
   * max value for current hour
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentHourMax
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentHourMax
   */
  public static final Property currentHourMax = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentHourMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentHourMax
   */
  public BStatusNumeric getCurrentHourMax() { return (BStatusNumeric)get(currentHourMax); }
  
  /**
   * Set the <code>currentHourMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentHourMax
   */
  public void setCurrentHourMax(BStatusNumeric v) { set(currentHourMax,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastHourTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastHourTotal</code> property.
   * total for last hour
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastHourTotal
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastHourTotal
   */
  public static final Property lastHourTotal = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastHourTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastHourTotal
   */
  public BStatusNumeric getLastHourTotal() { return (BStatusNumeric)get(lastHourTotal); }
  
  /**
   * Set the <code>lastHourTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastHourTotal
   */
  public void setLastHourTotal(BStatusNumeric v) { set(lastHourTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastHourAvg"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastHourAvg</code> property.
   * average for the last hour
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastHourAvg
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastHourAvg
   */
  public static final Property lastHourAvg = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastHourAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastHourAvg
   */
  public BStatusNumeric getLastHourAvg() { return (BStatusNumeric)get(lastHourAvg); }
  
  /**
   * Set the <code>lastHourAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastHourAvg
   */
  public void setLastHourAvg(BStatusNumeric v) { set(lastHourAvg,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastHourMin"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastHourMin</code> property.
   * min value for last hour
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastHourMin
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastHourMin
   */
  public static final Property lastHourMin = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastHourMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastHourMin
   */
  public BStatusNumeric getLastHourMin() { return (BStatusNumeric)get(lastHourMin); }
  
  /**
   * Set the <code>lastHourMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastHourMin
   */
  public void setLastHourMin(BStatusNumeric v) { set(lastHourMin,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastHourMax"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastHourMax</code> property.
   * max value for last hour
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastHourMax
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastHourMax
   */
  public static final Property lastHourMax = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastHourMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastHourMax
   */
  public BStatusNumeric getLastHourMax() { return (BStatusNumeric)get(lastHourMax); }
  
  /**
   * Set the <code>lastHourMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastHourMax
   */
  public void setLastHourMax(BStatusNumeric v) { set(lastHourMax,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentDayTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentDayTotal</code> property.
   * total for current day
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentDayTotal
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentDayTotal
   */
  public static final Property currentDayTotal = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentDayTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentDayTotal
   */
  public BStatusNumeric getCurrentDayTotal() { return (BStatusNumeric)get(currentDayTotal); }
  
  /**
   * Set the <code>currentDayTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentDayTotal
   */
  public void setCurrentDayTotal(BStatusNumeric v) { set(currentDayTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentDayAvg"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentDayAvg</code> property.
   * average for the current day
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentDayAvg
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentDayAvg
   */
  public static final Property currentDayAvg = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentDayAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentDayAvg
   */
  public BStatusNumeric getCurrentDayAvg() { return (BStatusNumeric)get(currentDayAvg); }
  
  /**
   * Set the <code>currentDayAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentDayAvg
   */
  public void setCurrentDayAvg(BStatusNumeric v) { set(currentDayAvg,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentDayMin"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentDayMin</code> property.
   * min value for current day
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentDayMin
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentDayMin
   */
  public static final Property currentDayMin = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentDayMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentDayMin
   */
  public BStatusNumeric getCurrentDayMin() { return (BStatusNumeric)get(currentDayMin); }
  
  /**
   * Set the <code>currentDayMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentDayMin
   */
  public void setCurrentDayMin(BStatusNumeric v) { set(currentDayMin,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentDayMax"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentDayMax</code> property.
   * max value for current day
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentDayMax
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentDayMax
   */
  public static final Property currentDayMax = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentDayMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentDayMax
   */
  public BStatusNumeric getCurrentDayMax() { return (BStatusNumeric)get(currentDayMax); }
  
  /**
   * Set the <code>currentDayMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentDayMax
   */
  public void setCurrentDayMax(BStatusNumeric v) { set(currentDayMax,v,null); }

////////////////////////////////////////////////////////////////
// Property "yesterdayTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>yesterdayTotal</code> property.
   * total from previous day
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getYesterdayTotal
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setYesterdayTotal
   */
  public static final Property yesterdayTotal = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>yesterdayTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#yesterdayTotal
   */
  public BStatusNumeric getYesterdayTotal() { return (BStatusNumeric)get(yesterdayTotal); }
  
  /**
   * Set the <code>yesterdayTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#yesterdayTotal
   */
  public void setYesterdayTotal(BStatusNumeric v) { set(yesterdayTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "yesterdayAvg"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>yesterdayAvg</code> property.
   * average for yesterday
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getYesterdayAvg
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setYesterdayAvg
   */
  public static final Property yesterdayAvg = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>yesterdayAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#yesterdayAvg
   */
  public BStatusNumeric getYesterdayAvg() { return (BStatusNumeric)get(yesterdayAvg); }
  
  /**
   * Set the <code>yesterdayAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#yesterdayAvg
   */
  public void setYesterdayAvg(BStatusNumeric v) { set(yesterdayAvg,v,null); }

////////////////////////////////////////////////////////////////
// Property "yesterdayMin"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>yesterdayMin</code> property.
   * min value from previous day
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getYesterdayMin
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setYesterdayMin
   */
  public static final Property yesterdayMin = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>yesterdayMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#yesterdayMin
   */
  public BStatusNumeric getYesterdayMin() { return (BStatusNumeric)get(yesterdayMin); }
  
  /**
   * Set the <code>yesterdayMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#yesterdayMin
   */
  public void setYesterdayMin(BStatusNumeric v) { set(yesterdayMin,v,null); }

////////////////////////////////////////////////////////////////
// Property "yesterdayMax"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>yesterdayMax</code> property.
   * max value from previous day
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getYesterdayMax
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setYesterdayMax
   */
  public static final Property yesterdayMax = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>yesterdayMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#yesterdayMax
   */
  public BStatusNumeric getYesterdayMax() { return (BStatusNumeric)get(yesterdayMax); }
  
  /**
   * Set the <code>yesterdayMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#yesterdayMax
   */
  public void setYesterdayMax(BStatusNumeric v) { set(yesterdayMax,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentWeekTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentWeekTotal</code> property.
   * total for current week
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentWeekTotal
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentWeekTotal
   */
  public static final Property currentWeekTotal = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentWeekTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentWeekTotal
   */
  public BStatusNumeric getCurrentWeekTotal() { return (BStatusNumeric)get(currentWeekTotal); }
  
  /**
   * Set the <code>currentWeekTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentWeekTotal
   */
  public void setCurrentWeekTotal(BStatusNumeric v) { set(currentWeekTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentWeekAvg"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentWeekAvg</code> property.
   * average for the current week
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentWeekAvg
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentWeekAvg
   */
  public static final Property currentWeekAvg = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentWeekAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentWeekAvg
   */
  public BStatusNumeric getCurrentWeekAvg() { return (BStatusNumeric)get(currentWeekAvg); }
  
  /**
   * Set the <code>currentWeekAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentWeekAvg
   */
  public void setCurrentWeekAvg(BStatusNumeric v) { set(currentWeekAvg,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentWeekMin"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentWeekMin</code> property.
   * min value for current week
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentWeekMin
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentWeekMin
   */
  public static final Property currentWeekMin = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentWeekMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentWeekMin
   */
  public BStatusNumeric getCurrentWeekMin() { return (BStatusNumeric)get(currentWeekMin); }
  
  /**
   * Set the <code>currentWeekMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentWeekMin
   */
  public void setCurrentWeekMin(BStatusNumeric v) { set(currentWeekMin,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentWeekMax"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentWeekMax</code> property.
   * max value for current week
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentWeekMax
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentWeekMax
   */
  public static final Property currentWeekMax = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentWeekMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentWeekMax
   */
  public BStatusNumeric getCurrentWeekMax() { return (BStatusNumeric)get(currentWeekMax); }
  
  /**
   * Set the <code>currentWeekMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentWeekMax
   */
  public void setCurrentWeekMax(BStatusNumeric v) { set(currentWeekMax,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastWeekTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastWeekTotal</code> property.
   * total for previous week
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastWeekTotal
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastWeekTotal
   */
  public static final Property lastWeekTotal = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastWeekTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastWeekTotal
   */
  public BStatusNumeric getLastWeekTotal() { return (BStatusNumeric)get(lastWeekTotal); }
  
  /**
   * Set the <code>lastWeekTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastWeekTotal
   */
  public void setLastWeekTotal(BStatusNumeric v) { set(lastWeekTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastWeekAvg"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastWeekAvg</code> property.
   * average for last week
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastWeekAvg
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastWeekAvg
   */
  public static final Property lastWeekAvg = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastWeekAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastWeekAvg
   */
  public BStatusNumeric getLastWeekAvg() { return (BStatusNumeric)get(lastWeekAvg); }
  
  /**
   * Set the <code>lastWeekAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastWeekAvg
   */
  public void setLastWeekAvg(BStatusNumeric v) { set(lastWeekAvg,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastWeekMin"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastWeekMin</code> property.
   * min value for previous week
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastWeekMin
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastWeekMin
   */
  public static final Property lastWeekMin = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastWeekMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastWeekMin
   */
  public BStatusNumeric getLastWeekMin() { return (BStatusNumeric)get(lastWeekMin); }
  
  /**
   * Set the <code>lastWeekMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastWeekMin
   */
  public void setLastWeekMin(BStatusNumeric v) { set(lastWeekMin,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastWeekMax"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastWeekMax</code> property.
   * max value for previous week
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastWeekMax
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastWeekMax
   */
  public static final Property lastWeekMax = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastWeekMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastWeekMax
   */
  public BStatusNumeric getLastWeekMax() { return (BStatusNumeric)get(lastWeekMax); }
  
  /**
   * Set the <code>lastWeekMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastWeekMax
   */
  public void setLastWeekMax(BStatusNumeric v) { set(lastWeekMax,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentMonthTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentMonthTotal</code> property.
   * total for current month
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentMonthTotal
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentMonthTotal
   */
  public static final Property currentMonthTotal = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentMonthTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentMonthTotal
   */
  public BStatusNumeric getCurrentMonthTotal() { return (BStatusNumeric)get(currentMonthTotal); }
  
  /**
   * Set the <code>currentMonthTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentMonthTotal
   */
  public void setCurrentMonthTotal(BStatusNumeric v) { set(currentMonthTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentMonthAvg"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentMonthAvg</code> property.
   * average for the current month
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentMonthAvg
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentMonthAvg
   */
  public static final Property currentMonthAvg = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentMonthAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentMonthAvg
   */
  public BStatusNumeric getCurrentMonthAvg() { return (BStatusNumeric)get(currentMonthAvg); }
  
  /**
   * Set the <code>currentMonthAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentMonthAvg
   */
  public void setCurrentMonthAvg(BStatusNumeric v) { set(currentMonthAvg,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentMonthMin"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentMonthMin</code> property.
   * min value for current month
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentMonthMin
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentMonthMin
   */
  public static final Property currentMonthMin = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentMonthMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentMonthMin
   */
  public BStatusNumeric getCurrentMonthMin() { return (BStatusNumeric)get(currentMonthMin); }
  
  /**
   * Set the <code>currentMonthMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentMonthMin
   */
  public void setCurrentMonthMin(BStatusNumeric v) { set(currentMonthMin,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentMonthMax"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentMonthMax</code> property.
   * max value for current month
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentMonthMax
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentMonthMax
   */
  public static final Property currentMonthMax = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentMonthMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentMonthMax
   */
  public BStatusNumeric getCurrentMonthMax() { return (BStatusNumeric)get(currentMonthMax); }
  
  /**
   * Set the <code>currentMonthMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentMonthMax
   */
  public void setCurrentMonthMax(BStatusNumeric v) { set(currentMonthMax,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastMonthTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastMonthTotal</code> property.
   * total for previous month
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastMonthTotal
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastMonthTotal
   */
  public static final Property lastMonthTotal = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastMonthTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastMonthTotal
   */
  public BStatusNumeric getLastMonthTotal() { return (BStatusNumeric)get(lastMonthTotal); }
  
  /**
   * Set the <code>lastMonthTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastMonthTotal
   */
  public void setLastMonthTotal(BStatusNumeric v) { set(lastMonthTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastMonthAvg"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastMonthAvg</code> property.
   * average for last month
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastMonthAvg
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastMonthAvg
   */
  public static final Property lastMonthAvg = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastMonthAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastMonthAvg
   */
  public BStatusNumeric getLastMonthAvg() { return (BStatusNumeric)get(lastMonthAvg); }
  
  /**
   * Set the <code>lastMonthAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastMonthAvg
   */
  public void setLastMonthAvg(BStatusNumeric v) { set(lastMonthAvg,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastMonthMin"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastMonthMin</code> property.
   * min value for previous month
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastMonthMin
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastMonthMin
   */
  public static final Property lastMonthMin = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastMonthMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastMonthMin
   */
  public BStatusNumeric getLastMonthMin() { return (BStatusNumeric)get(lastMonthMin); }
  
  /**
   * Set the <code>lastMonthMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastMonthMin
   */
  public void setLastMonthMin(BStatusNumeric v) { set(lastMonthMin,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastMonthMax"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastMonthMax</code> property.
   * max value for previous month
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastMonthMax
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastMonthMax
   */
  public static final Property lastMonthMax = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastMonthMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastMonthMax
   */
  public BStatusNumeric getLastMonthMax() { return (BStatusNumeric)get(lastMonthMax); }
  
  /**
   * Set the <code>lastMonthMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastMonthMax
   */
  public void setLastMonthMax(BStatusNumeric v) { set(lastMonthMax,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentYearTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentYearTotal</code> property.
   * total for current year
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentYearTotal
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentYearTotal
   */
  public static final Property currentYearTotal = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentYearTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentYearTotal
   */
  public BStatusNumeric getCurrentYearTotal() { return (BStatusNumeric)get(currentYearTotal); }
  
  /**
   * Set the <code>currentYearTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentYearTotal
   */
  public void setCurrentYearTotal(BStatusNumeric v) { set(currentYearTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentYearAvg"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentYearAvg</code> property.
   * average for the current year
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentYearAvg
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentYearAvg
   */
  public static final Property currentYearAvg = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentYearAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentYearAvg
   */
  public BStatusNumeric getCurrentYearAvg() { return (BStatusNumeric)get(currentYearAvg); }
  
  /**
   * Set the <code>currentYearAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentYearAvg
   */
  public void setCurrentYearAvg(BStatusNumeric v) { set(currentYearAvg,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentYearMin"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentYearMin</code> property.
   * min value for current year
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentYearMin
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentYearMin
   */
  public static final Property currentYearMin = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentYearMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentYearMin
   */
  public BStatusNumeric getCurrentYearMin() { return (BStatusNumeric)get(currentYearMin); }
  
  /**
   * Set the <code>currentYearMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentYearMin
   */
  public void setCurrentYearMin(BStatusNumeric v) { set(currentYearMin,v,null); }

////////////////////////////////////////////////////////////////
// Property "currentYearMax"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>currentYearMax</code> property.
   * max value for current year
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getCurrentYearMax
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setCurrentYearMax
   */
  public static final Property currentYearMax = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>currentYearMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentYearMax
   */
  public BStatusNumeric getCurrentYearMax() { return (BStatusNumeric)get(currentYearMax); }
  
  /**
   * Set the <code>currentYearMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#currentYearMax
   */
  public void setCurrentYearMax(BStatusNumeric v) { set(currentYearMax,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastYearTotal"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastYearTotal</code> property.
   * total for previous month
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastYearTotal
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastYearTotal
   */
  public static final Property lastYearTotal = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastYearTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastYearTotal
   */
  public BStatusNumeric getLastYearTotal() { return (BStatusNumeric)get(lastYearTotal); }
  
  /**
   * Set the <code>lastYearTotal</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastYearTotal
   */
  public void setLastYearTotal(BStatusNumeric v) { set(lastYearTotal,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastYearAvg"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastYearAvg</code> property.
   * average for last year
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastYearAvg
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastYearAvg
   */
  public static final Property lastYearAvg = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastYearAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastYearAvg
   */
  public BStatusNumeric getLastYearAvg() { return (BStatusNumeric)get(lastYearAvg); }
  
  /**
   * Set the <code>lastYearAvg</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastYearAvg
   */
  public void setLastYearAvg(BStatusNumeric v) { set(lastYearAvg,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastYearMin"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastYearMin</code> property.
   * min value for previous month
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastYearMin
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastYearMin
   */
  public static final Property lastYearMin = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastYearMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastYearMin
   */
  public BStatusNumeric getLastYearMin() { return (BStatusNumeric)get(lastYearMin); }
  
  /**
   * Set the <code>lastYearMin</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastYearMin
   */
  public void setLastYearMin(BStatusNumeric v) { set(lastYearMin,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastYearMax"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastYearMax</code> property.
   * max value for previous month
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#getLastYearMax
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#setLastYearMax
   */
  public static final Property lastYearMax = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE|Flags.TRANSIENT|Flags.SUMMARY, new BStatusNumeric(),null);
  
  /**
   * Get the <code>lastYearMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastYearMax
   */
  public BStatusNumeric getLastYearMax() { return (BStatusNumeric)get(lastYearMax); }
  
  /**
   * Set the <code>lastYearMax</code> property.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#lastYearMax
   */
  public void setLastYearMax(BStatusNumeric v) { set(lastYearMax,v,null); }

////////////////////////////////////////////////////////////////
// Action "calculate"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>calculate</code> action.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#calculate()
   */
  public static final Action calculate = newAction(0,null);
  
  /**
   * Invoke the <code>calculate</code> action.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#calculate
   */
  public void calculate() { invoke(calculate,null,null); }

////////////////////////////////////////////////////////////////
// Action "reset"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>reset</code> action.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#reset()
   */
  public static final Action reset = newAction(0,null);
  
  /**
   * Invoke the <code>reset</code> action.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#reset
   */
  public void reset() { invoke(reset,null,null); }

////////////////////////////////////////////////////////////////
// Action "query"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>query</code> action.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#query()
   */
  public static final Action query = newAction(Flags.ASYNC|Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>query</code> action.
   * @see com.vykon.vykonPro.history.BNumericHistoryAnalysis#query
   */
  public void query() { invoke(query,null,null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BNumericHistoryAnalysis.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

////////////////////////////////////////////////////////////////
//  Initialization  /  Cleanup
////////////////////////////////////////////////////////////////

  public void started()
    throws Exception
  {
    billDay = getBillingCycleStartDay();
    lastOrd = getHistoryOrd();
    
    if(!isRunning()) return;
    
    checkForService();
    updateHistory();
    
    //setup ticket to re-execute object
    initClockTicket(); 
  }

  public void stopped()
    throws Exception
  {
    if(ticket != null)
      ticket.cancel();
    super.stopped();
  }

  public void atSteadyState()
  {
    calculate();
  }
  
  private final void checkForService()
  {
    serviceFound = false;
    
    if(BVykonProService.getInstance()==null)
    {
      serviceFault = ("This component requires the VykonPro service in the station");
    }
    else if(BVykonProService.getInstance().isDisabled())
    {
      serviceFault = ("The vykonPro service is disabled");
    }
    else
    {
      serviceFound = true;
      serviceFault = "";
    }
    
    updateStatus();
  }
  
  public BFacets getSlotFacets(Slot slot)
  {
    if(slot.getName().startsWith("current") || slot.getName().startsWith("last") ||
        slot.getName().startsWith("yesterday"))
      return getFacets();
    return super.getSlotFacets(slot);

  }
  
  public void clockChanged(BRelTime value)
  {
    initClockTicket();
  }

  private void initClockTicket()
  {
    if(ticket != null)
      ticket.cancel();
    
    BAbsTime now = Clock.time();
    BAbsTime startTime = BAbsTime.make(now, BTime.make(0, 0, 0));
    while (!startTime.isAfter(now))
      startTime = startTime.add(getExecutePeriod());
    
    ticket = Clock.schedulePeriodically(this, startTime, getExecutePeriod(), calculate, null);
    
    setNextUpdateTime(startTime.add(getExecutePeriod()));
  }
  
  public void changed(Property property, Context context) 
  {
   super.changed(property, context);
    if( !Sys.atSteadyState() || !isRunning() )
     return;
   
   if(property.equals(executePeriod)) calculate();
   else if(property.equals(historyName)) updateHistory();
   else if(property.equals(billingCycleStartDay)) billDay = getBillingCycleStartDay(); 
  }
  
  private String resolveHistoryName()
  {
    BFormat pattern = getHistoryName();
    BFormat station;
    BFormat history;
    try
    {
      String p = pattern.encodeToString();
      if(p.indexOf("/")>-1)
      {
        int firstSlash = p.indexOf("/");
        station = BFormat.make(p.substring(0, firstSlash));
        history = BFormat.make(p.substring(firstSlash+1));
        
        String s = SlotPath.escape(SlotPath.unescape(station.format(this)));
        String h = SlotPath.escape(SlotPath.unescape(history.format(this)));
        
        return s + "/" + h;
      }
      else
      {
        return "";
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
      return "couldn't resolve the historyName";
    }
  }
  
  private void updateHistory()
  {
    ordOkay = false;
    String hname = resolveHistoryName();
    
    if (hname.length() > 0)
    {
      BOrd hord = BOrd.make("history:/" + hname);

      //check if the history exists
      try
      {
        if (hord.resolve(this).get() != null)
        {
          //succeeded so set status okay  
          ordOkay = true;
          ordFault = "";
          
          setHistoryOrd(hord);
          
          if (!getHistoryOrd().equals(lastOrd))
          {
            //historyOrd has changed so clear runtime values
            lastOrd = (BOrd.make(getHistoryOrd().toString()));
            reset();
          }
          
          BHistoryConfig config = ((BIHistory)hord.resolve(this).get()).getConfig();
          BCollectionInterval interval = config.getInterval();
          if(interval.isIrregular()) setHistoryInterval(BRelTime.DEFAULT);
          else setHistoryInterval(interval.getInterval());
        }
        else
        {
          ordFault = ("history:/" + hname + " | History Ord does not resolve to an actual history");
        }
      }
      catch (Exception e)
      {
        System.out.println(this.getSlotPath() + " - exception resolving history");
        System.out.println(e);
        
        ordFault = ("history:/" + hname + " | History Ord does not resolve to an actual history");
      }
    }
    else
    {
      ordFault = ("History Name was blank");
    }
    
    updateStatus();
  }
  
  private void updateStatus()
  {
    if(serviceFound && ordOkay)
    {
      setStatus(BStatus.ok);
      setFaultCause("");
    }
    else 
    {
      setStatus(BStatus.fault);
      
      if(!serviceFound) setFaultCause(serviceFault);
      else if(!ordOkay) setFaultCause(ordFault);
      else setFaultCause("unknown fault");
    }
  }
  
  public IFuture post(Action action, BValue argument, Context cx)
  {
    if(action == query)
      return BVykonProService.getInstance().getWorker().post(new Invocation(this, action, argument, cx));
    else
      return super.post(action, argument, cx);
  }
  
  public void doCalculate() throws Exception
  {
    // set ticket to re-execute later
    //initClockTicket();
    setNextUpdateTime(Clock.time().add(getExecutePeriod()));
    
    if(getHistoryOrd().equals(BOrd.NULL))
    {
      updateHistory();
    }
    else if(!BOrd.make("history:/" + resolveHistoryName()).equals(getHistoryOrd()))
    {
      updateHistory();
    }
    
    if(!getStatus().isOk())
    {
      checkForService();
      if(!getStatus().isOk())
      {
        updateHistory();
        if(!getStatus().isOk()) return;
      }
      
    }
    
    //call the query action so it posts to worker thread
    query();
  }
  
  
  public void doQuery() throws Exception
  {
    if(!getStatus().isOk()) return;
    
    setCalculating(true);
    
    now = Clock.time();
    currentHour = now.getHour();
    day = now.getDayOfYear();
    currentDay = now.getDay();
    month = now.getMonth().getMonthOfYear();
    year = now.getYear();
    
    if(getDateMode().getOrdinal()==BDateModeEnum.BILLING_CYCLE)
    {
      firstOfMonth = BAbsTime.make(year, now.getMonth(), getBillingCycleStartDay(), 0, 0, 0, 0);
      firstOfYear = BAbsTime.make(year, BMonth.make(0), getBillingCycleStartDay(), 0,0,0,0);
      
      if(currentDay<billDay)
      {
        //hasn't reach billing start day for month
        firstOfMonth = firstOfMonth.prevMonth();
      }
    }
    
    //determine syntax to plug into queries based on deltaLogged property
    //if value is logged in delta format then delta=false because we just need to get each value as logged
    //if value is not logged in delta format then we need to get the delta value for each period
    if(getDeltaLogged()) deltaSyntax=";delta=false";
    else deltaSyntax=";delta=true";
    
    currentHourQuery();
    currentDayQuery();
    currentWeekQuery();
    currentMonthQuery();
    currentYearQuery();
    
    if(currentHour != lastHour)
    {
      lastHourQuery();
      lastHour = currentHour;
    }
    
    if (day != lastDay)
    {
      yesterdayQuery();
      lastDay = day;
    }
    
    if (resetLatch == true || (now.getWeekday().equals(resetDay) && latch == false))
    {
      lastWeekQuery();
      latch = true;
      resetLatch = false;
    }
    
    if (!now.getWeekday().equals(resetDay)) latch = false;
    
    if (now.getMonth().getMonthOfYear() != previousMonth)
    {
      lastMonthQuery();
      previousMonth = month;
    }
    
    if (now.getYear() != previousYear)
    {
      lastYearQuery();
      previousYear = year;
    }
    
    setCalculating(false);
    setLastUpdateTime(Clock.time());
  }
  
  
  public void doReset() throws Exception
  {
    
    getCurrentHourTotal().setValue(0);
    getCurrentHourAvg().setValue(0);
    getCurrentHourMin().setValue(0);
    getCurrentHourMax().setValue(0);
    
    getLastHourTotal().setValue(0);
    getLastHourAvg().setValue(0);
    getLastHourMin().setValue(0);
    getLastHourMax().setValue(0);
    lastHour = 0;
    
    getCurrentDayTotal().setValue(0);
    getCurrentDayAvg().setValue(0);
    getCurrentDayMin().setValue(0);
    getCurrentDayMax().setValue(0);
    getYesterdayTotal().setValue(0);
    getYesterdayAvg().setValue(0);
    getYesterdayMin().setValue(0);
    getYesterdayMax().setValue(0);
    lastDay=0;
    getCurrentWeekTotal().setValue(0);
    getCurrentWeekAvg().setValue(0);
    getCurrentWeekMin().setValue(0);
    getCurrentWeekMax().setValue(0);
    getLastWeekTotal().setValue(0);
    getLastWeekAvg().setValue(0);
    getLastWeekMin().setValue(0);
    getLastWeekMax().setValue(0);
    getCurrentMonthTotal().setValue(0);
    getCurrentMonthAvg().setValue(0);
    getCurrentMonthMin().setValue(0);
    getCurrentMonthMax().setValue(0);
    getLastMonthTotal().setValue(0);
    getLastMonthAvg().setValue(0);
    getLastMonthMin().setValue(0);
    getLastMonthMax().setValue(0);
    previousMonth=0;
    getCurrentYearTotal().setValue(0);
    getCurrentYearAvg().setValue(0);
    getCurrentYearMin().setValue(0);
    getCurrentYearMax().setValue(0);
    getLastYearTotal().setValue(0);
    getLastYearAvg().setValue(0);
    getLastYearMin().setValue(0);
    getLastYearMax().setValue(0);
    previousYear=0;
    resetLatch = true;
    calculate();
  }
   

  public void currentHourQuery() throws Exception
  {
    
    //TODO calculate time
    StringBuffer sb = new StringBuffer();
    sb.append("?period=timeRange;start=");
    BAbsTime current = Clock.time();
    BAbsTime start = null;
    start = BAbsTime.make(current, BTime.make(current.getHour(),0,0));
    
    if(!getDeltaLogged())
    {
      //performing delta calc so need to get one interval before the hour
      if(getHistoryInterval().getMinutes()<=60)
      {
        start = start.subtract(getHistoryInterval());
        ch=true;
      }
      
    }
    
    sb.append(start.encodeToString());
    sb.append(";end=");
    sb.append(current.encodeToString());
    runQuery(sb.toString());
    
    getCurrentHourTotal().setValue(total);
    getCurrentHourMin().setValue(min);
    getCurrentHourMax().setValue(max);
    getCurrentHourAvg().setValue(avg);
  }

  public void lastHourQuery() throws Exception
  {
    //TODO calculate time
    StringBuffer sb = new StringBuffer();
    sb.append("?period=timeRange;start=");
    BAbsTime current = Clock.time();
    BAbsTime start = null;
    BAbsTime stop = null;
    //TODO need to handle case where interval > 1 hr
    stop = BAbsTime.make(current, BTime.make(current.getHour(),0,0));
    start = stop.subtract(BRelTime.makeHours(1));
    //shift stop time so it doesn't include the top of the current hour
    //7/14/11 changed from getHistoryInterval() to BRelTime.makeSeconds(1)
    stop = stop.subtract(BRelTime.makeSeconds(1));
    
    if(!getDeltaLogged())
    {
      //performing delta calc so need to get one interval before the hour
      //7/4/11 added if to check for interval < 1 hr
      if(getHistoryInterval().getMinutes()<=60)
      {
        start = start.subtract(getHistoryInterval());
        ch = true;
      }
    }
    
    sb.append(start.encodeToString());
    sb.append(";end=");
    sb.append(stop.encodeToString());
    
    runQuery(sb.toString());
    
    getLastHourTotal().setValue(total);
    getLastHourMin().setValue(min);
    getLastHourMax().setValue(max);
    getLastHourAvg().setValue(avg);
  }

  public void currentDayQuery() throws Exception
  {
    runQuery("?period=today");
    getCurrentDayTotal().setValue(total);
    getCurrentDayMin().setValue(min);
    getCurrentDayMax().setValue(max);
    getCurrentDayAvg().setValue(avg);
  }

  
  public void yesterdayQuery() throws Exception
  {
    runQuery("?period=yesterday");
    getYesterdayTotal().setValue(total);
    getYesterdayMin().setValue(min);
    getYesterdayMax().setValue(max);
    getYesterdayAvg().setValue(avg);
  }
  
  
  public void currentWeekQuery() throws Exception
  {
    runQuery("?period=weekToDate");
    getCurrentWeekTotal().setValue(total);
    getCurrentWeekMin().setValue(min);
    getCurrentWeekMax().setValue(max);
    getCurrentWeekAvg().setValue(avg);
  }


  public void lastWeekQuery() throws Exception
  {
    runQuery("?period=lastWeek");
    getLastWeekTotal().setValue(total);
    getLastWeekMin().setValue(min);
    getLastWeekMax().setValue(max);
    getLastWeekAvg().setValue(avg);
  }


  public void currentMonthQuery() throws Exception
  {
    if(getDateMode().getOrdinal()==BDateModeEnum.FIRST_OF_MONTH)
    {
      runQuery("?period=monthToDate");
    }
    else
    {
      String p = "?period=timeRange;start=" + firstOfMonth.encodeToString();
      runQuery(p);
    }
    
    getCurrentMonthTotal().setValue(total);
    getCurrentMonthMin().setValue(min);
    getCurrentMonthMax().setValue(max);
    getCurrentMonthAvg().setValue(avg);
  }


  public void lastMonthQuery() throws Exception
  {
    if(getDateMode().getOrdinal()==BDateModeEnum.FIRST_OF_MONTH)
    {
      runQuery("?period=lastMonth");
    }
    else
    {
      StringBuffer sb = new StringBuffer();
      sb.append("?period=timeRange;start=");
      sb.append(firstOfMonth.prevMonth().encodeToString());
      sb.append(";end=");
      sb.append(firstOfMonth.encodeToString());
      
      runQuery(sb.toString());
    }
    
    getLastMonthTotal().setValue(total);
    getLastMonthMin().setValue(min);
    getLastMonthMax().setValue(max);
    getLastMonthAvg().setValue(avg);
  }

  public void currentYearQuery() throws Exception
  {
    if(getDateMode().getOrdinal()==BDateModeEnum.FIRST_OF_MONTH)
    {
      runQuery("?period=yearToDate");
    }
    else
    {
      StringBuffer sb = new StringBuffer();
      sb.append("?period=timeRange;start=");
      sb.append(firstOfYear.encodeToString());
      
      runQuery(sb.toString());
    }
    
    getCurrentYearTotal().setValue(total);
    getCurrentYearMin().setValue(min);
    getCurrentYearMax().setValue(max);
    getCurrentYearAvg().setValue(avg);
  }

  public void lastYearQuery() throws Exception
  {
    if(getDateMode().getOrdinal()==BDateModeEnum.FIRST_OF_MONTH)
    {
      runQuery("?period=lastYear");
    }
    else
    {
      StringBuffer sb = new StringBuffer();
      sb.append("?period=timeRange;start=");
      sb.append(firstOfYear.prevYear().encodeToString());
      sb.append(";end=");
      sb.append(firstOfYear.encodeToString());
      
      runQuery(sb.toString());
    }
    
    getLastYearTotal().setValue(total);
    getLastYearMin().setValue(min);
    getLastYearMax().setValue(max);
    getLastYearAvg().setValue(avg);
  }

  private void runQuery(String period)
  {
    BOrd qord = BOrd.make(getHistoryOrd().toString() + period + deltaSyntax + "|bql:select timestamp, trendFlags, status, value");
    BITable result = (BITable)qord.resolve().get();
    ColumnList columns = result.getColumns();
    Column valueColumn = columns.get(3);
    Column statusColumn = columns.get(2);
    Column trendColumn = columns.get(1);
    
    total = 0d;
    min = 0d;
    max = 0d;
    records = 0d;
    avg = 0d;
    boolean first = true;
    boolean excInvalid = getExcludeInvalidValues();
    boolean excHidden = getExcludeHiddenValues();
    double value = 0d;
    BStatus status = BStatus.nullStatus;
    BTrendFlags flags = null;

    try(TableCursor c = result.cursor())
    {
      while (c.next())
      {
        value = (((BINumeric)c.cell(valueColumn)).getNumeric());
        status = (((BStatus)c.cell(statusColumn)).getStatus());
        flags = ((BTrendFlags)c.cell(trendColumn));

        if(excInvalid & !status.isValid()) continue;
        if(excHidden & flags.getBit("hidden")) continue;
        if(BDouble.make(value)==BDouble.NaN) continue;

        //increment records to count the number in the set
        if(ch==false) records = records + 1;

        if(first)
        {
          min = value;
          max = value;
          total = value;
          if (ch==false) first = false;
          else ch = false;
        }
        else
        {
          if(value<min) min = value;
          if(value>max) max = value;
          total += value;
        }
      }
    }

    
    //calculate the avg as total/records. make sure no divide by zero error.
    if(records > 0) avg = total/records;
  }

  ////////////////////////////////////////////////////////////////
  // Icon
  ////////////////////////////////////////////////////////////////
  
  /**
   * Get the icon.
   */
  public BIcon getIcon() { return icon; }
  private static final BIcon icon = BIcon.std("historyDatabase.png");
  
  
////////////////////////////////////////////////////////////////

  private boolean serviceFound = false;
  private String serviceFault="vykonPro service fault";
  private boolean ordOkay = false;
  private String ordFault = "historyOrd fault";
  Clock.Ticket ticket = null;
  BAbsTime now = null;
  BAbsTime firstOfMonth = null;
  BAbsTime firstOfYear = null;
  BOrd lastOrd = null;
  double min = 0;
  double max = 0;
  double total = 0;
  double records = 0;
  double avg = 0;
  int day=0;
  int currentHour = 0;
  int lastHour = 0;
  int currentDay=0;
  int lastDay=0;
  int month=0;
  int previousMonth=0;
  int year=0;
  int previousYear=0;
  int billDay = 0;
  BWeekday resetDay = BWeekday.make("sunday");
  String deviceName;
  boolean latch = false;
  boolean resetLatch = true;
  boolean serviceExists = false;
  String deltaSyntax = "";
  
  boolean ch = false;
  
  
}

