package com.vykon.vykonPro.hvac;

import javax.baja.status.BStatus;
import javax.baja.status.BStatusBoolean;
import javax.baja.status.BStatusEnum;
import javax.baja.sys.Action;
import javax.baja.sys.BAbsTime;
import javax.baja.sys.BComponent;
import javax.baja.sys.BDynamicEnum;
import javax.baja.sys.BEnum;
import javax.baja.sys.BEnumRange;
import javax.baja.sys.BFacets;
import javax.baja.sys.BIcon;
import javax.baja.sys.BInteger;
import javax.baja.sys.BRelTime;
import javax.baja.sys.Clock;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Slot;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;
import javax.baja.nre.util.TextUtil;

import com.vykon.vykonPro.enums.BSequencerModeEnum;
import com.vykon.vykonPro.enums.BSequencerOutputEnum;
import com.vykon.vykonPro.util.BDisabledLoadsBitString;

/**
 * BRotationalLoadSequencer is a component that can provide control of 2 - 10 loads 
 * to designate a lead load, which can be rotated automatically.  The loads are
 * started in sequence beginning with the load that is designated as the lead load.  
 *
 * @author    James Johnson
 * @creation  05 April 2004
 * @version   $Revision: 7$ $Date: 3/3/2004 8:48:19 AM$
 * @since     Baja 1.0
 */

public class BRotationalLoadSequencerV1
    extends BComponent
{
  /*-
   class BRotationalLoadSequencerV1
   {
     properties
     {
       status: BStatus
        -- Status of the component
        flags{ readonly, defaultOnClone }
        default{[ BStatus.ok ]}
        
      faultCause: String
        -- The fault cause.
        flags{ readonly, defaultOnClone }
        default{[ "" ]}
        
      facets: BFacets
        -- These facets are applied against the out property.
        flags{ operator }
        default {[ BFacets.makeBoolean() ]}
      
      actionFacets: BFacets
        -- These facets are applied against the enableOutput and disableOutput actions
        flags{ operator }
        default{[ BFacets.makeEnum(BEnumRange.make(BSequencerOutputEnum.TYPE)) ]}
        
      schedule: BStatusBoolean
        -- When schedule input is false, all loads will be de-engergized.
        flags { operator, summary }
        default {[ new BStatusBoolean() ]}
      
      loadFailed: BStatusBoolean
        -- If any load fails to run when commanded, the loadFailed output will be set to true.
        flags { summary, readonly }
        default {[ new BStatusBoolean() ]}
        
      stopFailedLoad: BStatusBoolean
        -- When property is true, if the load fails to start the outX slot will be set to false. When 
        -- property is false, if the load fails to start the outX slot will remain set to true.
        flags{ operator }
        default {[ new BStatusBoolean(false) ]}
        
      numberOutputs: int
        -- The number of loads to be controlled.
        flags{ operator }
        default {[ 2 ]}
        slotfacets {[ BFacets.makeInt(null, 2, 10) ]}
        
      requiredLoads: int
        -- The required number of loads that need to be energized, such as with parallel pumping.
        flags{ operator }
        default {[ 1 ]}
        slotfacets {[ BFacets.makeInt(null, 0, 10) ]}
      
      rotationOrder: BRotationOrder
        -- The configured order in which the loads are started
        flags{ operator, readonly }
        default{[ new BRotationOrder() ]}
       
      rotateMode: BEnum
        -- rotateMode determines how lead load is rotated.  Manual - rotation occurs only when manually commanded.
        -- Interval - rotation occurs after station elapsed time exceeds the rotateInterval value, regardless of load
        --runtime. Rotate - rotation occurs when the lead loads accumulated runtime (since being designated as the lead load)
        --exceeds the rotateInterval value.
        flags{ operator }
        default {[ ( BDynamicEnum.make(1, BEnumRange.make(BSequencerModeEnum.TYPE))) ]}
        
      rotateInterval: BRelTime
        -- The amount of time that must elapse based on rotateMode before the lead load is rotated to the next load.
        flags{ operator }
        default {[ BRelTime.makeHours(168) ]}
      
      feedbackDelay: BRelTime
        -- The amount of time after commanding a load on after which if the feedback has not registered that the load
        -- will be considered failed.
        flags{ operator }
        default {[ BRelTime.make(30000) ]}

      lastRotateTime: BAbsTime
        -- The timestamp of the last rotation of the lead load
        flags{ readonly, defaultOnClone }
        default{[ BAbsTime.NULL ]}
      
      runtimeRemainingUntilShift: BRelTime
        -- The amount of runtime left until the lead load will be rotated, if the rotateMode is runtime.
        flags{ readonly }
        default{[ BRelTime.makeSeconds(0) ]}
              
      leadLoad: BEnum
        -- The load output which is designated as lead.
        flags{ readonly, hidden }
        default{[ BDynamicEnum.make(1, BEnumRange.make(BSequencerOutputEnum.TYPE)) ]}
      
      leadLoadEnum: BStatusEnum
        -- The load output which is designated as lead.
        flags{ readonly }
        default{[ new BStatusEnum(BDynamicEnum.make(1, BEnumRange.make(BSequencerOutputEnum.TYPE))) ]}
      
      leadRuntimeAtStart: BRelTime
        -- The current accumulated runtime of the lead output recorded when the load is designated as lead
        flags{ readonly }
        default{[ BRelTime.makeSeconds(0) ]}
      
      nextLeadLoad: BStatusEnum
        -- The load output which is designated as lead.
        flags{ readonly }
        default{[ new BStatusEnum(BDynamicEnum.make(2, BEnumRange.make(BSequencerOutputEnum.TYPE))) ]}
        
      outA: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}

      outB: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}

      outC: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outD: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outE: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outF: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outG: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outH: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outI: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outJ: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      feedbackA: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackB: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackC: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackD: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackE: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackF: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackG: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackH: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackI: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackJ: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      runtimeA: BRelTime
        flags { transient, summary }
        default {[ BRelTime.make(0) ]}

      runtimeB: BRelTime
        flags { transient, summary }
        default {[ BRelTime.make(0) ]}

      runtimeC: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeD: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeE: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeF: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeG: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeH: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeI: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeJ: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}
      
      disabledLoads: BDisabledLoadsBitString
        flags { readonly, hidden, defaultOnClone }
        default {[ new BDisabledLoadsBitString(0) ]}
      
      debug: boolean
        flags{ hidden }
        default {[ false ]}
    }

    actions
    {
      startupTimerExpired()
        flags { hidden }
        
      rotateTimerExpired()
        flags { hidden }
      
      feedbackTimerExpiredA()
        flags { hidden }
      
      feedbackTimerExpiredB()
        flags { hidden }
      
      feedbackTimerExpiredC()
        flags { hidden }
      
      feedbackTimerExpiredD()
        flags { hidden }
      
      feedbackTimerExpiredE()
        flags { hidden }
      
      feedbackTimerExpiredF()
        flags { hidden }
      
      feedbackTimerExpiredG()
        flags { hidden }
      
      feedbackTimerExpiredH()
        flags { hidden }
      
      feedbackTimerExpiredI()
        flags { hidden }
      
      feedbackTimerExpiredJ()
        flags { hidden }
                
      runtimeTimerExpired()
        flags { hidden }
        
      countdownTimerExpired()
        flags { hidden }
        
      checkNext()
        flags { hidden }
        
      disableOutput(output:BEnum)
         default {[ ( BDynamicEnum.make(1, BEnumRange.make( BSequencerOutputEnum.TYPE))) ]}
         
      enableOutput(output:BEnum)
         default {[ ( BDynamicEnum.make(1, BEnumRange.make( BSequencerOutputEnum.TYPE))) ]}
      
      manualCmd(manCmd:BManualSequencerCmd)
        default {[ new BManualSequencerCmd() ]}
        
      manualRotate()
    
      resetFailedLoad(output:BEnum)
        default {[ ( BDynamicEnum.make(1, BEnumRange.make( BSequencerOutputEnum.TYPE))) ]}
    }
     
   }
   -*/
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1(3798460467)1.0$ @*/
/* Generated Thu Feb 06 22:19:36 EST 2014 by Slot-o-Matic 2000 (c) Tridium, Inc. 2000 */

////////////////////////////////////////////////////////////////
// Property "status"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>status</code> property.
   * Status of the component
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getStatus
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setStatus
   */
  public static final Property status = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, BStatus.ok,null);
  
  /**
   * Get the <code>status</code> property.
   * Status of the component
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#status
   */
  public BStatus getStatus() { return (BStatus)get(status); }
  
  /**
   * Set the <code>status</code> property.
   * Status of the component
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#status
   */
  public void setStatus(BStatus v) { set(status,v,null); }

////////////////////////////////////////////////////////////////
// Property "faultCause"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>faultCause</code> property.
   * The fault cause.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFaultCause
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFaultCause
   */
  public static final Property faultCause = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, "",null);
  
  /**
   * Get the <code>faultCause</code> property.
   * The fault cause.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#faultCause
   */
  public String getFaultCause() { return getString(faultCause); }
  
  /**
   * Set the <code>faultCause</code> property.
   * The fault cause.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#faultCause
   */
  public void setFaultCause(String v) { setString(faultCause,v,null); }

////////////////////////////////////////////////////////////////
// Property "facets"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>facets</code> property.
   * These facets are applied against the out property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFacets
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFacets
   */
  public static final Property facets = newProperty(Flags.OPERATOR, BFacets.makeBoolean(),null);
  
  /**
   * Get the <code>facets</code> property.
   * These facets are applied against the out property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#facets
   */
  public BFacets getFacets() { return (BFacets)get(facets); }
  
  /**
   * Set the <code>facets</code> property.
   * These facets are applied against the out property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#facets
   */
  public void setFacets(BFacets v) { set(facets,v,null); }

////////////////////////////////////////////////////////////////
// Property "actionFacets"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>actionFacets</code> property.
   * These facets are applied against the enableOutput and disableOutput actions
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getActionFacets
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setActionFacets
   */
  public static final Property actionFacets = newProperty(Flags.OPERATOR, BFacets.makeEnum(BEnumRange.make(BSequencerOutputEnum.TYPE)),null);
  
  /**
   * Get the <code>actionFacets</code> property.
   * These facets are applied against the enableOutput and disableOutput actions
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#actionFacets
   */
  public BFacets getActionFacets() { return (BFacets)get(actionFacets); }
  
  /**
   * Set the <code>actionFacets</code> property.
   * These facets are applied against the enableOutput and disableOutput actions
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#actionFacets
   */
  public void setActionFacets(BFacets v) { set(actionFacets,v,null); }

////////////////////////////////////////////////////////////////
// Property "schedule"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>schedule</code> property.
   * When schedule input is false, all loads will be de-engergized.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getSchedule
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setSchedule
   */
  public static final Property schedule = newProperty(Flags.OPERATOR|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>schedule</code> property.
   * When schedule input is false, all loads will be de-engergized.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#schedule
   */
  public BStatusBoolean getSchedule() { return (BStatusBoolean)get(schedule); }
  
  /**
   * Set the <code>schedule</code> property.
   * When schedule input is false, all loads will be de-engergized.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#schedule
   */
  public void setSchedule(BStatusBoolean v) { set(schedule,v,null); }

////////////////////////////////////////////////////////////////
// Property "loadFailed"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>loadFailed</code> property.
   * If any load fails to run when commanded, the loadFailed
   * output will be set to true.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getLoadFailed
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setLoadFailed
   */
  public static final Property loadFailed = newProperty(Flags.SUMMARY|Flags.READONLY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>loadFailed</code> property.
   * If any load fails to run when commanded, the loadFailed
   * output will be set to true.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#loadFailed
   */
  public BStatusBoolean getLoadFailed() { return (BStatusBoolean)get(loadFailed); }
  
  /**
   * Set the <code>loadFailed</code> property.
   * If any load fails to run when commanded, the loadFailed
   * output will be set to true.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#loadFailed
   */
  public void setLoadFailed(BStatusBoolean v) { set(loadFailed,v,null); }

////////////////////////////////////////////////////////////////
// Property "stopFailedLoad"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>stopFailedLoad</code> property.
   * When property is true, if the load fails to start the outX slot will be set to false. When property is false, if the load fails to start the outX slot will remain set to true.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getStopFailedLoad
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setStopFailedLoad
   */
  public static final Property stopFailedLoad = newProperty(Flags.OPERATOR, new BStatusBoolean(false),null);
  
  /**
   * Get the <code>stopFailedLoad</code> property.
   * When property is true, if the load fails to start the outX slot will be set to false. When property is false, if the load fails to start the outX slot will remain set to true.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#stopFailedLoad
   */
  public BStatusBoolean getStopFailedLoad() { return (BStatusBoolean)get(stopFailedLoad); }
  
  /**
   * Set the <code>stopFailedLoad</code> property.
   * When property is true, if the load fails to start the outX slot will be set to false. When property is false, if the load fails to start the outX slot will remain set to true.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#stopFailedLoad
   */
  public void setStopFailedLoad(BStatusBoolean v) { set(stopFailedLoad,v,null); }

////////////////////////////////////////////////////////////////
// Property "numberOutputs"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>numberOutputs</code> property.
   * The number of loads to be controlled.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getNumberOutputs
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setNumberOutputs
   */
  public static final Property numberOutputs = newProperty(Flags.OPERATOR, 2,BFacets.makeInt(null, 2, 10));
  
  /**
   * Get the <code>numberOutputs</code> property.
   * The number of loads to be controlled.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#numberOutputs
   */
  public int getNumberOutputs() { return getInt(numberOutputs); }
  
  /**
   * Set the <code>numberOutputs</code> property.
   * The number of loads to be controlled.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#numberOutputs
   */
  public void setNumberOutputs(int v) { setInt(numberOutputs,v,null); }

////////////////////////////////////////////////////////////////
// Property "requiredLoads"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>requiredLoads</code> property.
   * The required number of loads that need to be energized,
   * such as with parallel pumping.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRequiredLoads
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRequiredLoads
   */
  public static final Property requiredLoads = newProperty(Flags.OPERATOR, 1,BFacets.makeInt(null, 0, 10));
  
  /**
   * Get the <code>requiredLoads</code> property.
   * The required number of loads that need to be energized,
   * such as with parallel pumping.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#requiredLoads
   */
  public int getRequiredLoads() { return getInt(requiredLoads); }
  
  /**
   * Set the <code>requiredLoads</code> property.
   * The required number of loads that need to be energized,
   * such as with parallel pumping.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#requiredLoads
   */
  public void setRequiredLoads(int v) { setInt(requiredLoads,v,null); }

////////////////////////////////////////////////////////////////
// Property "rotationOrder"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>rotationOrder</code> property.
   * The configured order in which the loads are started
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRotationOrder
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRotationOrder
   */
  public static final Property rotationOrder = newProperty(Flags.OPERATOR|Flags.READONLY, new BRotationOrder(),null);
  
  /**
   * Get the <code>rotationOrder</code> property.
   * The configured order in which the loads are started
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#rotationOrder
   */
  public BRotationOrder getRotationOrder() { return (BRotationOrder)get(rotationOrder); }
  
  /**
   * Set the <code>rotationOrder</code> property.
   * The configured order in which the loads are started
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#rotationOrder
   */
  public void setRotationOrder(BRotationOrder v) { set(rotationOrder,v,null); }

////////////////////////////////////////////////////////////////
// Property "rotateMode"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>rotateMode</code> property.
   * rotateMode determines how lead load is rotated.  Manual
   * - rotation occurs only when manually commanded. Interval
   * - rotation occurs after station elapsed time exceeds
   * the rotateInterval value, regardless of load runtime.
   * Rotate - rotation occurs when the lead loads accumulated
   * runtime (since being designated as the lead load) exceeds the rotateInterval value.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRotateMode
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRotateMode
   */
  public static final Property rotateMode = newProperty(Flags.OPERATOR, ( BDynamicEnum.make(1, BEnumRange.make(BSequencerModeEnum.TYPE))),null);
  
  /**
   * Get the <code>rotateMode</code> property.
   * rotateMode determines how lead load is rotated.  Manual
   * - rotation occurs only when manually commanded. Interval
   * - rotation occurs after station elapsed time exceeds
   * the rotateInterval value, regardless of load runtime.
   * Rotate - rotation occurs when the lead loads accumulated
   * runtime (since being designated as the lead load) exceeds the rotateInterval value.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#rotateMode
   */
  public BEnum getRotateMode() { return (BEnum)get(rotateMode); }
  
  /**
   * Set the <code>rotateMode</code> property.
   * rotateMode determines how lead load is rotated.  Manual
   * - rotation occurs only when manually commanded. Interval
   * - rotation occurs after station elapsed time exceeds
   * the rotateInterval value, regardless of load runtime.
   * Rotate - rotation occurs when the lead loads accumulated
   * runtime (since being designated as the lead load) exceeds the rotateInterval value.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#rotateMode
   */
  public void setRotateMode(BEnum v) { set(rotateMode,v,null); }

////////////////////////////////////////////////////////////////
// Property "rotateInterval"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>rotateInterval</code> property.
   * The amount of time that must elapse based on rotateMode
   * before the lead load is rotated to the next load.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRotateInterval
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRotateInterval
   */
  public static final Property rotateInterval = newProperty(Flags.OPERATOR, BRelTime.makeHours(168),null);
  
  /**
   * Get the <code>rotateInterval</code> property.
   * The amount of time that must elapse based on rotateMode
   * before the lead load is rotated to the next load.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#rotateInterval
   */
  public BRelTime getRotateInterval() { return (BRelTime)get(rotateInterval); }
  
  /**
   * Set the <code>rotateInterval</code> property.
   * The amount of time that must elapse based on rotateMode
   * before the lead load is rotated to the next load.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#rotateInterval
   */
  public void setRotateInterval(BRelTime v) { set(rotateInterval,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackDelay"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackDelay</code> property.
   * The amount of time after commanding a load on after
   * which if the feedback has not registered that the load will be considered failed.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFeedbackDelay
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFeedbackDelay
   */
  public static final Property feedbackDelay = newProperty(Flags.OPERATOR, BRelTime.make(30000),null);
  
  /**
   * Get the <code>feedbackDelay</code> property.
   * The amount of time after commanding a load on after
   * which if the feedback has not registered that the load will be considered failed.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackDelay
   */
  public BRelTime getFeedbackDelay() { return (BRelTime)get(feedbackDelay); }
  
  /**
   * Set the <code>feedbackDelay</code> property.
   * The amount of time after commanding a load on after
   * which if the feedback has not registered that the load will be considered failed.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackDelay
   */
  public void setFeedbackDelay(BRelTime v) { set(feedbackDelay,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastRotateTime"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastRotateTime</code> property.
   * The timestamp of the last rotation of the lead load
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getLastRotateTime
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setLastRotateTime
   */
  public static final Property lastRotateTime = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, BAbsTime.NULL,null);
  
  /**
   * Get the <code>lastRotateTime</code> property.
   * The timestamp of the last rotation of the lead load
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#lastRotateTime
   */
  public BAbsTime getLastRotateTime() { return (BAbsTime)get(lastRotateTime); }
  
  /**
   * Set the <code>lastRotateTime</code> property.
   * The timestamp of the last rotation of the lead load
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#lastRotateTime
   */
  public void setLastRotateTime(BAbsTime v) { set(lastRotateTime,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeRemainingUntilShift"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeRemainingUntilShift</code> property.
   * The amount of runtime left until the lead load will
   * be rotated, if the rotateMode is runtime.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRuntimeRemainingUntilShift
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRuntimeRemainingUntilShift
   */
  public static final Property runtimeRemainingUntilShift = newProperty(Flags.READONLY, BRelTime.makeSeconds(0),null);
  
  /**
   * Get the <code>runtimeRemainingUntilShift</code> property.
   * The amount of runtime left until the lead load will
   * be rotated, if the rotateMode is runtime.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeRemainingUntilShift
   */
  public BRelTime getRuntimeRemainingUntilShift() { return (BRelTime)get(runtimeRemainingUntilShift); }
  
  /**
   * Set the <code>runtimeRemainingUntilShift</code> property.
   * The amount of runtime left until the lead load will
   * be rotated, if the rotateMode is runtime.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeRemainingUntilShift
   */
  public void setRuntimeRemainingUntilShift(BRelTime v) { set(runtimeRemainingUntilShift,v,null); }

////////////////////////////////////////////////////////////////
// Property "leadLoad"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>leadLoad</code> property.
   * The load output which is designated as lead.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getLeadLoad
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setLeadLoad
   */
  public static final Property leadLoad = newProperty(Flags.READONLY|Flags.HIDDEN, BDynamicEnum.make(1, BEnumRange.make(BSequencerOutputEnum.TYPE)),null);
  
  /**
   * Get the <code>leadLoad</code> property.
   * The load output which is designated as lead.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#leadLoad
   */
  public BEnum getLeadLoad() { return (BEnum)get(leadLoad); }
  
  /**
   * Set the <code>leadLoad</code> property.
   * The load output which is designated as lead.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#leadLoad
   */
  public void setLeadLoad(BEnum v) { set(leadLoad,v,null); }

////////////////////////////////////////////////////////////////
// Property "leadLoadEnum"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>leadLoadEnum</code> property.
   * The load output which is designated as lead.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getLeadLoadEnum
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setLeadLoadEnum
   */
  public static final Property leadLoadEnum = newProperty(Flags.READONLY, new BStatusEnum(BDynamicEnum.make(1, BEnumRange.make(BSequencerOutputEnum.TYPE))),null);
  
  /**
   * Get the <code>leadLoadEnum</code> property.
   * The load output which is designated as lead.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#leadLoadEnum
   */
  public BStatusEnum getLeadLoadEnum() { return (BStatusEnum)get(leadLoadEnum); }
  
  /**
   * Set the <code>leadLoadEnum</code> property.
   * The load output which is designated as lead.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#leadLoadEnum
   */
  public void setLeadLoadEnum(BStatusEnum v) { set(leadLoadEnum,v,null); }

////////////////////////////////////////////////////////////////
// Property "leadRuntimeAtStart"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>leadRuntimeAtStart</code> property.
   * The current accumulated runtime of the lead output
   * recorded when the load is designated as lead
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getLeadRuntimeAtStart
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setLeadRuntimeAtStart
   */
  public static final Property leadRuntimeAtStart = newProperty(Flags.READONLY, BRelTime.makeSeconds(0),null);
  
  /**
   * Get the <code>leadRuntimeAtStart</code> property.
   * The current accumulated runtime of the lead output
   * recorded when the load is designated as lead
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#leadRuntimeAtStart
   */
  public BRelTime getLeadRuntimeAtStart() { return (BRelTime)get(leadRuntimeAtStart); }
  
  /**
   * Set the <code>leadRuntimeAtStart</code> property.
   * The current accumulated runtime of the lead output
   * recorded when the load is designated as lead
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#leadRuntimeAtStart
   */
  public void setLeadRuntimeAtStart(BRelTime v) { set(leadRuntimeAtStart,v,null); }

////////////////////////////////////////////////////////////////
// Property "nextLeadLoad"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>nextLeadLoad</code> property.
   * The load output which is designated as lead.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getNextLeadLoad
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setNextLeadLoad
   */
  public static final Property nextLeadLoad = newProperty(Flags.READONLY, new BStatusEnum(BDynamicEnum.make(2, BEnumRange.make(BSequencerOutputEnum.TYPE))),null);
  
  /**
   * Get the <code>nextLeadLoad</code> property.
   * The load output which is designated as lead.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#nextLeadLoad
   */
  public BStatusEnum getNextLeadLoad() { return (BStatusEnum)get(nextLeadLoad); }
  
  /**
   * Set the <code>nextLeadLoad</code> property.
   * The load output which is designated as lead.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#nextLeadLoad
   */
  public void setNextLeadLoad(BStatusEnum v) { set(nextLeadLoad,v,null); }

////////////////////////////////////////////////////////////////
// Property "outA"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outA</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getOutA
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setOutA
   */
  public static final Property outA = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outA</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outA
   */
  public BStatusBoolean getOutA() { return (BStatusBoolean)get(outA); }
  
  /**
   * Set the <code>outA</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outA
   */
  public void setOutA(BStatusBoolean v) { set(outA,v,null); }

////////////////////////////////////////////////////////////////
// Property "outB"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outB</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getOutB
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setOutB
   */
  public static final Property outB = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outB</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outB
   */
  public BStatusBoolean getOutB() { return (BStatusBoolean)get(outB); }
  
  /**
   * Set the <code>outB</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outB
   */
  public void setOutB(BStatusBoolean v) { set(outB,v,null); }

////////////////////////////////////////////////////////////////
// Property "outC"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outC</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getOutC
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setOutC
   */
  public static final Property outC = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outC</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outC
   */
  public BStatusBoolean getOutC() { return (BStatusBoolean)get(outC); }
  
  /**
   * Set the <code>outC</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outC
   */
  public void setOutC(BStatusBoolean v) { set(outC,v,null); }

////////////////////////////////////////////////////////////////
// Property "outD"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outD</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getOutD
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setOutD
   */
  public static final Property outD = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outD</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outD
   */
  public BStatusBoolean getOutD() { return (BStatusBoolean)get(outD); }
  
  /**
   * Set the <code>outD</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outD
   */
  public void setOutD(BStatusBoolean v) { set(outD,v,null); }

////////////////////////////////////////////////////////////////
// Property "outE"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outE</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getOutE
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setOutE
   */
  public static final Property outE = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outE</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outE
   */
  public BStatusBoolean getOutE() { return (BStatusBoolean)get(outE); }
  
  /**
   * Set the <code>outE</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outE
   */
  public void setOutE(BStatusBoolean v) { set(outE,v,null); }

////////////////////////////////////////////////////////////////
// Property "outF"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outF</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getOutF
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setOutF
   */
  public static final Property outF = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outF</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outF
   */
  public BStatusBoolean getOutF() { return (BStatusBoolean)get(outF); }
  
  /**
   * Set the <code>outF</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outF
   */
  public void setOutF(BStatusBoolean v) { set(outF,v,null); }

////////////////////////////////////////////////////////////////
// Property "outG"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outG</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getOutG
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setOutG
   */
  public static final Property outG = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outG</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outG
   */
  public BStatusBoolean getOutG() { return (BStatusBoolean)get(outG); }
  
  /**
   * Set the <code>outG</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outG
   */
  public void setOutG(BStatusBoolean v) { set(outG,v,null); }

////////////////////////////////////////////////////////////////
// Property "outH"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outH</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getOutH
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setOutH
   */
  public static final Property outH = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outH</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outH
   */
  public BStatusBoolean getOutH() { return (BStatusBoolean)get(outH); }
  
  /**
   * Set the <code>outH</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outH
   */
  public void setOutH(BStatusBoolean v) { set(outH,v,null); }

////////////////////////////////////////////////////////////////
// Property "outI"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outI</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getOutI
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setOutI
   */
  public static final Property outI = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outI</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outI
   */
  public BStatusBoolean getOutI() { return (BStatusBoolean)get(outI); }
  
  /**
   * Set the <code>outI</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outI
   */
  public void setOutI(BStatusBoolean v) { set(outI,v,null); }

////////////////////////////////////////////////////////////////
// Property "outJ"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getOutJ
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setOutJ
   */
  public static final Property outJ = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outJ
   */
  public BStatusBoolean getOutJ() { return (BStatusBoolean)get(outJ); }
  
  /**
   * Set the <code>outJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#outJ
   */
  public void setOutJ(BStatusBoolean v) { set(outJ,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackA"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackA</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFeedbackA
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFeedbackA
   */
  public static final Property feedbackA = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackA</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackA
   */
  public BStatusBoolean getFeedbackA() { return (BStatusBoolean)get(feedbackA); }
  
  /**
   * Set the <code>feedbackA</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackA
   */
  public void setFeedbackA(BStatusBoolean v) { set(feedbackA,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackB"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackB</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFeedbackB
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFeedbackB
   */
  public static final Property feedbackB = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackB</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackB
   */
  public BStatusBoolean getFeedbackB() { return (BStatusBoolean)get(feedbackB); }
  
  /**
   * Set the <code>feedbackB</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackB
   */
  public void setFeedbackB(BStatusBoolean v) { set(feedbackB,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackC"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackC</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFeedbackC
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFeedbackC
   */
  public static final Property feedbackC = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackC</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackC
   */
  public BStatusBoolean getFeedbackC() { return (BStatusBoolean)get(feedbackC); }
  
  /**
   * Set the <code>feedbackC</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackC
   */
  public void setFeedbackC(BStatusBoolean v) { set(feedbackC,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackD"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackD</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFeedbackD
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFeedbackD
   */
  public static final Property feedbackD = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackD</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackD
   */
  public BStatusBoolean getFeedbackD() { return (BStatusBoolean)get(feedbackD); }
  
  /**
   * Set the <code>feedbackD</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackD
   */
  public void setFeedbackD(BStatusBoolean v) { set(feedbackD,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackE"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackE</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFeedbackE
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFeedbackE
   */
  public static final Property feedbackE = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackE</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackE
   */
  public BStatusBoolean getFeedbackE() { return (BStatusBoolean)get(feedbackE); }
  
  /**
   * Set the <code>feedbackE</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackE
   */
  public void setFeedbackE(BStatusBoolean v) { set(feedbackE,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackF"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackF</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFeedbackF
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFeedbackF
   */
  public static final Property feedbackF = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackF</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackF
   */
  public BStatusBoolean getFeedbackF() { return (BStatusBoolean)get(feedbackF); }
  
  /**
   * Set the <code>feedbackF</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackF
   */
  public void setFeedbackF(BStatusBoolean v) { set(feedbackF,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackG"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackG</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFeedbackG
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFeedbackG
   */
  public static final Property feedbackG = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackG</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackG
   */
  public BStatusBoolean getFeedbackG() { return (BStatusBoolean)get(feedbackG); }
  
  /**
   * Set the <code>feedbackG</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackG
   */
  public void setFeedbackG(BStatusBoolean v) { set(feedbackG,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackH"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackH</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFeedbackH
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFeedbackH
   */
  public static final Property feedbackH = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackH</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackH
   */
  public BStatusBoolean getFeedbackH() { return (BStatusBoolean)get(feedbackH); }
  
  /**
   * Set the <code>feedbackH</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackH
   */
  public void setFeedbackH(BStatusBoolean v) { set(feedbackH,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackI"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackI</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFeedbackI
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFeedbackI
   */
  public static final Property feedbackI = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackI</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackI
   */
  public BStatusBoolean getFeedbackI() { return (BStatusBoolean)get(feedbackI); }
  
  /**
   * Set the <code>feedbackI</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackI
   */
  public void setFeedbackI(BStatusBoolean v) { set(feedbackI,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackJ"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getFeedbackJ
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setFeedbackJ
   */
  public static final Property feedbackJ = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackJ
   */
  public BStatusBoolean getFeedbackJ() { return (BStatusBoolean)get(feedbackJ); }
  
  /**
   * Set the <code>feedbackJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackJ
   */
  public void setFeedbackJ(BStatusBoolean v) { set(feedbackJ,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeA"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeA</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRuntimeA
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRuntimeA
   */
  public static final Property runtimeA = newProperty(Flags.TRANSIENT|Flags.SUMMARY, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeA</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeA
   */
  public BRelTime getRuntimeA() { return (BRelTime)get(runtimeA); }
  
  /**
   * Set the <code>runtimeA</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeA
   */
  public void setRuntimeA(BRelTime v) { set(runtimeA,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeB"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeB</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRuntimeB
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRuntimeB
   */
  public static final Property runtimeB = newProperty(Flags.TRANSIENT|Flags.SUMMARY, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeB</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeB
   */
  public BRelTime getRuntimeB() { return (BRelTime)get(runtimeB); }
  
  /**
   * Set the <code>runtimeB</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeB
   */
  public void setRuntimeB(BRelTime v) { set(runtimeB,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeC"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeC</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRuntimeC
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRuntimeC
   */
  public static final Property runtimeC = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeC</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeC
   */
  public BRelTime getRuntimeC() { return (BRelTime)get(runtimeC); }
  
  /**
   * Set the <code>runtimeC</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeC
   */
  public void setRuntimeC(BRelTime v) { set(runtimeC,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeD"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeD</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRuntimeD
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRuntimeD
   */
  public static final Property runtimeD = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeD</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeD
   */
  public BRelTime getRuntimeD() { return (BRelTime)get(runtimeD); }
  
  /**
   * Set the <code>runtimeD</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeD
   */
  public void setRuntimeD(BRelTime v) { set(runtimeD,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeE"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeE</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRuntimeE
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRuntimeE
   */
  public static final Property runtimeE = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeE</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeE
   */
  public BRelTime getRuntimeE() { return (BRelTime)get(runtimeE); }
  
  /**
   * Set the <code>runtimeE</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeE
   */
  public void setRuntimeE(BRelTime v) { set(runtimeE,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeF"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeF</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRuntimeF
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRuntimeF
   */
  public static final Property runtimeF = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeF</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeF
   */
  public BRelTime getRuntimeF() { return (BRelTime)get(runtimeF); }
  
  /**
   * Set the <code>runtimeF</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeF
   */
  public void setRuntimeF(BRelTime v) { set(runtimeF,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeG"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeG</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRuntimeG
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRuntimeG
   */
  public static final Property runtimeG = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeG</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeG
   */
  public BRelTime getRuntimeG() { return (BRelTime)get(runtimeG); }
  
  /**
   * Set the <code>runtimeG</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeG
   */
  public void setRuntimeG(BRelTime v) { set(runtimeG,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeH"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeH</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRuntimeH
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRuntimeH
   */
  public static final Property runtimeH = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeH</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeH
   */
  public BRelTime getRuntimeH() { return (BRelTime)get(runtimeH); }
  
  /**
   * Set the <code>runtimeH</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeH
   */
  public void setRuntimeH(BRelTime v) { set(runtimeH,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeI"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeI</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRuntimeI
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRuntimeI
   */
  public static final Property runtimeI = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeI</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeI
   */
  public BRelTime getRuntimeI() { return (BRelTime)get(runtimeI); }
  
  /**
   * Set the <code>runtimeI</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeI
   */
  public void setRuntimeI(BRelTime v) { set(runtimeI,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeJ"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getRuntimeJ
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setRuntimeJ
   */
  public static final Property runtimeJ = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeJ
   */
  public BRelTime getRuntimeJ() { return (BRelTime)get(runtimeJ); }
  
  /**
   * Set the <code>runtimeJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeJ
   */
  public void setRuntimeJ(BRelTime v) { set(runtimeJ,v,null); }

////////////////////////////////////////////////////////////////
// Property "disabledLoads"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>disabledLoads</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getDisabledLoads
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setDisabledLoads
   */
  public static final Property disabledLoads = newProperty(Flags.READONLY|Flags.HIDDEN|Flags.DEFAULT_ON_CLONE, new BDisabledLoadsBitString(0),null);
  
  /**
   * Get the <code>disabledLoads</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#disabledLoads
   */
  public BDisabledLoadsBitString getDisabledLoads() { return (BDisabledLoadsBitString)get(disabledLoads); }
  
  /**
   * Set the <code>disabledLoads</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#disabledLoads
   */
  public void setDisabledLoads(BDisabledLoadsBitString v) { set(disabledLoads,v,null); }

////////////////////////////////////////////////////////////////
// Property "debug"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>debug</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#getDebug
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#setDebug
   */
  public static final Property debug = newProperty(Flags.HIDDEN, false,null);
  
  /**
   * Get the <code>debug</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#debug
   */
  public boolean getDebug() { return getBoolean(debug); }
  
  /**
   * Set the <code>debug</code> property.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#debug
   */
  public void setDebug(boolean v) { setBoolean(debug,v,null); }

////////////////////////////////////////////////////////////////
// Action "startupTimerExpired"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>startupTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#startupTimerExpired()
   */
  public static final Action startupTimerExpired = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>startupTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#startupTimerExpired
   */
  public void startupTimerExpired() { invoke(startupTimerExpired,null,null); }

////////////////////////////////////////////////////////////////
// Action "rotateTimerExpired"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>rotateTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#rotateTimerExpired()
   */
  public static final Action rotateTimerExpired = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>rotateTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#rotateTimerExpired
   */
  public void rotateTimerExpired() { invoke(rotateTimerExpired,null,null); }

////////////////////////////////////////////////////////////////
// Action "feedbackTimerExpiredA"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackTimerExpiredA</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredA()
   */
  public static final Action feedbackTimerExpiredA = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>feedbackTimerExpiredA</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredA
   */
  public void feedbackTimerExpiredA() { invoke(feedbackTimerExpiredA,null,null); }

////////////////////////////////////////////////////////////////
// Action "feedbackTimerExpiredB"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackTimerExpiredB</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredB()
   */
  public static final Action feedbackTimerExpiredB = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>feedbackTimerExpiredB</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredB
   */
  public void feedbackTimerExpiredB() { invoke(feedbackTimerExpiredB,null,null); }

////////////////////////////////////////////////////////////////
// Action "feedbackTimerExpiredC"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackTimerExpiredC</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredC()
   */
  public static final Action feedbackTimerExpiredC = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>feedbackTimerExpiredC</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredC
   */
  public void feedbackTimerExpiredC() { invoke(feedbackTimerExpiredC,null,null); }

////////////////////////////////////////////////////////////////
// Action "feedbackTimerExpiredD"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackTimerExpiredD</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredD()
   */
  public static final Action feedbackTimerExpiredD = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>feedbackTimerExpiredD</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredD
   */
  public void feedbackTimerExpiredD() { invoke(feedbackTimerExpiredD,null,null); }

////////////////////////////////////////////////////////////////
// Action "feedbackTimerExpiredE"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackTimerExpiredE</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredE()
   */
  public static final Action feedbackTimerExpiredE = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>feedbackTimerExpiredE</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredE
   */
  public void feedbackTimerExpiredE() { invoke(feedbackTimerExpiredE,null,null); }

////////////////////////////////////////////////////////////////
// Action "feedbackTimerExpiredF"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackTimerExpiredF</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredF()
   */
  public static final Action feedbackTimerExpiredF = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>feedbackTimerExpiredF</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredF
   */
  public void feedbackTimerExpiredF() { invoke(feedbackTimerExpiredF,null,null); }

////////////////////////////////////////////////////////////////
// Action "feedbackTimerExpiredG"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackTimerExpiredG</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredG()
   */
  public static final Action feedbackTimerExpiredG = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>feedbackTimerExpiredG</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredG
   */
  public void feedbackTimerExpiredG() { invoke(feedbackTimerExpiredG,null,null); }

////////////////////////////////////////////////////////////////
// Action "feedbackTimerExpiredH"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackTimerExpiredH</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredH()
   */
  public static final Action feedbackTimerExpiredH = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>feedbackTimerExpiredH</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredH
   */
  public void feedbackTimerExpiredH() { invoke(feedbackTimerExpiredH,null,null); }

////////////////////////////////////////////////////////////////
// Action "feedbackTimerExpiredI"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackTimerExpiredI</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredI()
   */
  public static final Action feedbackTimerExpiredI = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>feedbackTimerExpiredI</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredI
   */
  public void feedbackTimerExpiredI() { invoke(feedbackTimerExpiredI,null,null); }

////////////////////////////////////////////////////////////////
// Action "feedbackTimerExpiredJ"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackTimerExpiredJ</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredJ()
   */
  public static final Action feedbackTimerExpiredJ = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>feedbackTimerExpiredJ</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#feedbackTimerExpiredJ
   */
  public void feedbackTimerExpiredJ() { invoke(feedbackTimerExpiredJ,null,null); }

////////////////////////////////////////////////////////////////
// Action "runtimeTimerExpired"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeTimerExpired()
   */
  public static final Action runtimeTimerExpired = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>runtimeTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#runtimeTimerExpired
   */
  public void runtimeTimerExpired() { invoke(runtimeTimerExpired,null,null); }

////////////////////////////////////////////////////////////////
// Action "countdownTimerExpired"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>countdownTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#countdownTimerExpired()
   */
  public static final Action countdownTimerExpired = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>countdownTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#countdownTimerExpired
   */
  public void countdownTimerExpired() { invoke(countdownTimerExpired,null,null); }

////////////////////////////////////////////////////////////////
// Action "checkNext"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>checkNext</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#checkNext()
   */
  public static final Action checkNext = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>checkNext</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#checkNext
   */
  public void checkNext() { invoke(checkNext,null,null); }

////////////////////////////////////////////////////////////////
// Action "disableOutput"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>disableOutput</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#disableOutput()
   */
  public static final Action disableOutput = newAction(0,( BDynamicEnum.make(1, BEnumRange.make( BSequencerOutputEnum.TYPE))),null);
  
  /**
   * Invoke the <code>disableOutput</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#disableOutput
   */
  public void disableOutput(BEnum output) { invoke(disableOutput,output,null); }

////////////////////////////////////////////////////////////////
// Action "enableOutput"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>enableOutput</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#enableOutput()
   */
  public static final Action enableOutput = newAction(0,( BDynamicEnum.make(1, BEnumRange.make( BSequencerOutputEnum.TYPE))),null);
  
  /**
   * Invoke the <code>enableOutput</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#enableOutput
   */
  public void enableOutput(BEnum output) { invoke(enableOutput,output,null); }

////////////////////////////////////////////////////////////////
// Action "manualCmd"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>manualCmd</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#manualCmd()
   */
  public static final Action manualCmd = newAction(0,new BManualSequencerCmd(),null);
  
  /**
   * Invoke the <code>manualCmd</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#manualCmd
   */
  public void manualCmd(BManualSequencerCmd manCmd) { invoke(manualCmd,manCmd,null); }

////////////////////////////////////////////////////////////////
// Action "manualRotate"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>manualRotate</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#manualRotate()
   */
  public static final Action manualRotate = newAction(0,null);
  
  /**
   * Invoke the <code>manualRotate</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#manualRotate
   */
  public void manualRotate() { invoke(manualRotate,null,null); }

////////////////////////////////////////////////////////////////
// Action "resetFailedLoad"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>resetFailedLoad</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#resetFailedLoad()
   */
  public static final Action resetFailedLoad = newAction(0,( BDynamicEnum.make(1, BEnumRange.make( BSequencerOutputEnum.TYPE))),null);
  
  /**
   * Invoke the <code>resetFailedLoad</code> action.
   * @see com.vykon.vykonPro.hvac.BRotationalLoadSequencerV1#resetFailedLoad
   */
  public void resetFailedLoad(BEnum output) { invoke(resetFailedLoad,output,null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BRotationalLoadSequencerV1.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  public BRotationalLoadSequencerV1()
  {
  }
  
  /**
   * Init if started after steady state has been reached.
   */
  public void started()
  {
    if(startupTicket!=null) startupTicket.cancel();
    startupTicket = Clock.schedule(this, BRelTime.makeSeconds(30), startupTimerExpired, null);
    numOutputs = getNumberOutputs();
    
    initNumberOutputs();
  }

  public void stopped()
  {
    cancelFeedbackTickets();
    if (rotateTicket != null) rotateTicket.cancel();
    if (startupTicket != null) startupTicket.cancel();
    if (runtimeTicket != null) runtimeTicket.cancel();
    if (countdownTicket != null) countdownTicket.cancel();
  }
  
  private void cancelFeedbackTickets()
  {
    if (feedbackTicketA != null) feedbackTicketA.cancel();
    if (feedbackTicketB != null) feedbackTicketB.cancel();
    if (feedbackTicketC != null) feedbackTicketC.cancel();
    if (feedbackTicketD != null) feedbackTicketD.cancel();
    if (feedbackTicketE != null) feedbackTicketE.cancel();
    if (feedbackTicketF != null) feedbackTicketF.cancel();
    if (feedbackTicketG != null) feedbackTicketG.cancel();
    if (feedbackTicketH != null) feedbackTicketH.cancel();
    if (feedbackTicketI != null) feedbackTicketI.cancel();
    if (feedbackTicketJ != null) feedbackTicketJ.cancel();
  }
  
   
  
/**
   * setoutput on in change.
   */
  public void changed(Property p, Context cx)
  {
    super.changed(p, cx);
    if (!Sys.atSteadyState() || !isRunning())
      return;

    if(p == numberOutputs)
    {
      initNumberOutputs();
    }
    else if(p == schedule) 
    {
      calculate();
    }
    else if(p == requiredLoads)
    {
      if(getSchedule().getValue()) startOutput();
    }
    else if (p == feedbackA || p == feedbackB || p == feedbackC || p == feedbackD ||
        p == feedbackE || p == feedbackF || p == feedbackG || p == feedbackH || p == feedbackI ||
        p == feedbackJ)
    {
      if (!currentIn)
        return;
      checkRunningStatus();
    }
    else if(p == rotateMode)
    {
      int mode = getRotateMode().getOrdinal();
      switch(mode)
      {
        case BSequencerModeEnum.RUNTIME:
          BRelTime currentRT = BRelTime.make(getRuntime(getLeadLoad().getEnum().getOrdinal()));
          setLeadRuntimeAtStart(currentRT);
          setRuntimeRemainingUntilShift(getRotateInterval());
          startRuntimeTimer();
          if(rotateTicket!=null) rotateTicket.cancel();
          break;
          
        case BSequencerModeEnum.INTERVAL:
          startRotateTimer();
          if(runtimeTicket!=null) runtimeTicket.cancel();
          break;
          
        case BSequencerModeEnum.MANUAL:
          startCountdownTimer();
          if(rotateTicket!=null) rotateTicket.cancel();
          if(runtimeTicket!=null) runtimeTicket.cancel();
          break;
      }
    }
  }

  public void initNumberOutputs()
  {
    numOutputs = getNumberOutputs();
    
    int setFlags;
    boolean used = false;
    
    for(int i = 0; i < 10; i++)
    {
      if(i >= numOutputs)
      {
        setFlags = Flags.HIDDEN | Flags.TRANSIENT;
        used = false;
      }
      else
      {
        setFlags = Flags.SUMMARY | Flags.TRANSIENT;
        used = true;
      }
      switch(i)
      {
        //should possibly check if runtime or feedback slots are linked and delete the links
        case 0: initSlot("outA", setFlags | Flags.READONLY); initSlot("runtimeA", setFlags); initSlot("feedbackA", setFlags); initLoads("outA", used,1); break;
        case 1: initSlot("outB", setFlags | Flags.READONLY); initSlot("runtimeB", setFlags); initSlot("feedbackB", setFlags); initLoads("outB", used,2); break;
        case 2: initSlot("outC", setFlags | Flags.READONLY); initSlot("runtimeC", setFlags); initSlot("feedbackC", setFlags); initLoads("outC", used,3); break;
        case 3: initSlot("outD", setFlags | Flags.READONLY); initSlot("runtimeD", setFlags); initSlot("feedbackD", setFlags); initLoads("outD", used,4); break;
        case 4: initSlot("outE", setFlags | Flags.READONLY); initSlot("runtimeE", setFlags); initSlot("feedbackE", setFlags); initLoads("outE", used,5); break;
        case 5: initSlot("outF", setFlags | Flags.READONLY); initSlot("runtimeF", setFlags); initSlot("feedbackF", setFlags); initLoads("outF", used,6); break;
        case 6: initSlot("outG", setFlags | Flags.READONLY); initSlot("runtimeG", setFlags); initSlot("feedbackG", setFlags); initLoads("outG", used,7); break;
        case 7: initSlot("outH", setFlags | Flags.READONLY); initSlot("runtimeH", setFlags); initSlot("feedbackH", setFlags); initLoads("outH", used,8); break;
        case 8: initSlot("outI", setFlags | Flags.READONLY); initSlot("runtimeI", setFlags); initSlot("feedbackI", setFlags); initLoads("outI", used,9); break;
        case 9: initSlot("outJ", setFlags | Flags.READONLY); initSlot("runtimeJ", setFlags); initSlot("feedbackJ", setFlags); initLoads("outJ", used,10); break;
      }
      
      //check for any previously disabled loads and disable them on start
      int[] disabledLoads = getDisabledLoads().getOrdinals();
      
      for(int k=0; k<disabledLoads.length; k++)
      {
        if(getDisabledLoads().getBit(disabledLoads[k]))
        {
          //the bitString contains this bit so disable the load
          String loadName = getDisabledLoads().getTag(disabledLoads[k]);
          ((BStatusBoolean)this.get(loadName)).setStatusDisabled(true);
        }
      }
    }
  }

  void initSlot(String slot, int flags)
  {
      try { setFlags(getSlot(slot), flags, null);} catch(Exception e) {};
  }
  
  void initLoads(String name, boolean used, int value)
  {
    int setFlags = Flags.HIDDEN | Flags.READONLY;
    BRotationOrder  ro = getRotationOrder();
    
    if(used)
    {
      //slot is being used so make sure it is there
      if(ro.get(name)==null)
      {
        ro.add(name, BInteger.make(value));
        ro.setFlags(ro.getSlot(name), setFlags);
      }
    }
    else
    {
      //not using this output so need to remove the slot
      if(ro.get(name)!=null) ro.remove(name);
    }
  }


  //////////////////////////////////////////////////////////////////////
  // actions
  //////////////////////////////////////////////////////////////////////
  
  public void doStartupTimerExpired()
  {
    if(startupTicket!=null) startupTicket.cancel();
    calculate();
  }
  
  public void doCountdownTimerExpired()
  {
    BAbsTime currentTime = Clock.time();
    BAbsTime nextTime = getLastRotateTime().add(getRotateInterval());
    BRelTime timeLeft = currentTime.delta(nextTime);
    
    if(timeLeft.getMillis() > 1000L) //more than 1 second left
    {
      setRuntimeRemainingUntilShift(timeLeft);
    }
    else
    {
      setRuntimeRemainingUntilShift(BRelTime.make(0));
    }
  }
  
  public void doRotateTimerExpired()
  {
    BAbsTime currentTime = Clock.time();
    BAbsTime nextTime = getLastRotateTime().add(getRotateInterval());
    
    BRelTime timeLeft = currentTime.delta(nextTime);
    
    if(timeLeft.getMillis() > 1000L)  //more than 1 second left
    {
      //we still have time left
      setRuntimeRemainingUntilShift(timeLeft);
    }
    else
    {
      //we have reached our rotation time
      setRuntimeRemainingUntilShift(BRelTime.make(0));
      rotateLead();     //change the lead load
      startOutput();    //shift loads
    }
  }

  public void doFeedbackTimerExpiredA()
  {
    feedbackTimerActiveA=false;
    if(!currentIn) return;
    checkRunningStatus();
  }

  public void doFeedbackTimerExpiredB()
  {
    feedbackTimerActiveB=false;
    if(!currentIn) return;
    checkRunningStatus();
  }
  
  public void doFeedbackTimerExpiredC()
  {
    feedbackTimerActiveC=false;
    if(!currentIn) return;
    checkRunningStatus();
  }
  
  public void doFeedbackTimerExpiredD()
  {
    feedbackTimerActiveD=false;
    if(!currentIn) return;
    checkRunningStatus();
  }
  
  public void doFeedbackTimerExpiredE()
  {
    feedbackTimerActiveE=false;
    if(!currentIn) return;
    checkRunningStatus();
  }
  
  public void doFeedbackTimerExpiredF()
  {
    feedbackTimerActiveF=false;
    if(!currentIn) return;
    checkRunningStatus();
  }
  
  public void doFeedbackTimerExpiredG()
  {
    feedbackTimerActiveG=false;
    if(!currentIn) return;
    checkRunningStatus();
  }
  
  public void doFeedbackTimerExpiredH()
  {
    feedbackTimerActiveH=false;
    if(!currentIn) return;
    checkRunningStatus();
  }
  
  public void doFeedbackTimerExpiredI()
  {
    feedbackTimerActiveI=false;
    if(!currentIn) return;
    checkRunningStatus();
  }
  
  public void doFeedbackTimerExpiredJ()
  {
    feedbackTimerActiveJ=false;
    if(!currentIn) return;
    checkRunningStatus();
  }
  
  public void doRuntimeTimerExpired()
  {
    long currentRT = getRuntime(getLeadLoad().getEnum().getOrdinal());
    long startRT = getLeadRuntimeAtStart().getMillis();
    long actualRT = currentRT - startRT;
    
    if(actualRT > 0)
    {
      long timeLeft = getRotateInterval().getMillis() - actualRT;
      if(timeLeft>1000L)  //more than 1 second left
      {
      //we still have time left
        setRuntimeRemainingUntilShift(BRelTime.make(timeLeft));
      }
      else
      {
        //we have met our accumulated runtime to shift
        setRuntimeRemainingUntilShift(BRelTime.make(0));
        rotateLead();    //change lead load
        startOutput();  //shift loads
      }
    }
    else
    {
      //calculated runtime < 0
      //TODO put in fault possibly or alarm
    }
  }
  
  public void doManualRotate()
  {
    // 7/29/2013 change for McKenneys to allow manual rotation with schedule input false
    //if(!currentIn) return;
    rotateLead();     //change lead load
    if(currentIn) startOutput();    // shift loads
    else checkNext();               // just update the nextLeadLoad property since nothing is running
    //startOutput();    //shift loads
  }

  public void doDisableOutput(BEnum output)
  {
    int outNmbr = output.getEnum().getOrdinal();
    getStatusOutput(outNmbr).setStatusDisabled(true);
    
    //update the disabledLoads bits
    int loadBit = getLoadBit(outNmbr);
    setDisabledLoads(BDisabledLoadsBitString.make(getDisabledLoads().getBits() | loadBit));
    
    startOutput();  //shift loads if necessary
  }
  
  public void doEnableOutput(BEnum output)
  {
    int outNmbr = output.getEnum().getOrdinal();
    getStatusOutput(outNmbr).setStatusDisabled(false);
    
    //update the disabledLoads bits
    int loadBit = getLoadBit(outNmbr);
    setDisabledLoads(BDisabledLoadsBitString.make(getDisabledLoads().getBits() & ~loadBit));
    
    startOutput();  //normalize loads
  }
  
  public void doResetFailedLoad(BEnum output)
  {
    int outNmbr = output.getEnum().getOrdinal();
    getStatusOutput(outNmbr).setStatusInAlarm(false);
    
    if(checkFailed()==0) getLoadFailed().setValue(false);
    
    startOutput(); //normalize loads
  }
  
  
  public void doManualCmd(BManualSequencerCmd manCmd)
  {
    
    int outNmbr = manCmd.getOutput().getOrdinal();
    
    if(manCmd.getCommand())
    {
      //command value is true so disable output
      getStatusOutput(outNmbr).setStatusDisabled(true);
      
      //update the disabledLoads bits
      int loadBit = getLoadBit(outNmbr);
      setDisabledLoads(BDisabledLoadsBitString.make(getDisabledLoads().getBits() | loadBit));
    }
    else
    {
      //command value is false so enable output
      getStatusOutput(outNmbr).setStatusDisabled(false);
      
      //update the disabledLoads bits
      int loadBit = getLoadBit(outNmbr);
      setDisabledLoads(BDisabledLoadsBitString.make(getDisabledLoads().getBits() & ~loadBit));
    }
	
	startOutput();  //shift loads if necessary
  }
  
  public void doCheckNext()
  {
    checkNextLoad();
  }
  
  ////////////////////////////////////////////////////////
  // methods
  ////////////////////////////////////////////////////////
  
  void calculate()
  { 
    currentIn = getSchedule().getValue();
    
    if(currentIn)  // if controlIn is active
    {
      if(!wasOn)   // and was not active on last execute
      {
        startOutput();          // start the required loads
        
        if(getRotateMode().getOrdinal()==BSequencerModeEnum.INTERVAL) startRotateTimer();
        else if(getRotateMode().getOrdinal()==BSequencerModeEnum.RUNTIME) startRuntimeTimer();
        else startCountdownTimer();
        
        wasOn = true;           // set was on flag.
      }
    }
    else  // control in is not active
    {
      if(wasOn)  // if was active on last execute
      {
        if(rotateTicket != null) rotateTicket.cancel();
        if(countdownTicket != null) countdownTicket.cancel();
        if(runtimeTicket != null) runtimeTicket.cancel();
        cancelFeedbackTickets();
        wasOn = false;             // clear was on flag.
      }
      
      stopOutput();              // stop all outputs
      
    }
  }

  private void rotateLead()
  {
    BRotationOrder ro = getRotationOrder();
    String lead1 = getLeadLoad().getTag();
    lead1 = lead1.substring(lead1.length()-1);
    String current = ro.getOrder();
    String[] loads = TextUtil.splitAndTrim(current, ',');
    boolean shifted = false;
    boolean next = false;
    
    for(int b=0; b<loads.length; b++)
    {
      if(next)
      {
        int p = ((BInteger)ro.get("out"+loads[b])).getInt();
        if(getOutStatus(p))
        {
          //the next load in sequence is available
          setLeadLoad(BDynamicEnum.make(p, BEnumRange.make(BSequencerOutputEnum.TYPE)));
          getLeadLoadEnum().setValue(BDynamicEnum.make(p, BEnumRange.make(BSequencerOutputEnum.TYPE)));
          shifted = true;
          break;
        }
      }
      if(lead1.equals(loads[b]))
      {
        next=true;
      }
    }
    
    if(!shifted && next)
    {
      //either lead was last load or no loads available after lead, so check from beginning again
      for(int c=0; c<loads.length; c++)
      {
        if(lead1.equals(loads[c]))
        {
          //we got back to lead load and didn't find another available so just break
          break;
        }
        else
        {
          int p = ((BInteger)ro.get("out"+loads[c])).getInt();
          if(getOutStatus(p))
          {
            //the next load in sequence is available
            setLeadLoad(BDynamicEnum.make(p, BEnumRange.make(BSequencerOutputEnum.TYPE)));
            getLeadLoadEnum().setValue(BDynamicEnum.make(p, BEnumRange.make(BSequencerOutputEnum.TYPE)));
            shifted = true;
            break;
          }
        }
      }
    }
    
    //if(leadShifted)
    if(shifted)
    {
      startOutput();
      setLastRotateTime(Clock.time());
      
      BRelTime currentRT = BRelTime.make(getRuntime(getLeadLoad().getEnum().getOrdinal()));
      setLeadRuntimeAtStart(currentRT);
      setRuntimeRemainingUntilShift(getRotateInterval());
      
      if(getStatus().isFault())
      {
        setStatus(BStatus.make(getStatus(), BStatus.FAULT, false));
        setFaultCause("");
      }
    }
    else
    {
      setStatus(BStatus.make(getStatus(), BStatus.FAULT, true));
      setFaultCause("There are no loads available to rotate the lead load at - " + Clock.time());
    }
  }
  
  private void checkNextLoad()
  {
    boolean nextAvail = false;
    boolean next = false;
    
    BRotationOrder ro = getRotationOrder();
    String lead1 = getLeadLoad().getTag();
    lead1 = lead1.substring(lead1.length()-1);
    String current = ro.getOrder();
    String[] loads = TextUtil.splitAndTrim(current, ',');
        
    for(int b=0; b<loads.length; b++)
    {
      if(next)
      {
        int p = ((BInteger)ro.get("out"+loads[b])).getInt();
        if(getOutStatus(p))
        {
          //the next load in sequence is available
          getNextLeadLoad().setValue(BDynamicEnum.make(p, BEnumRange.make(BSequencerOutputEnum.TYPE)));
          nextAvail = true;
          break;
        }
      }
      if(lead1.equals(loads[b]))
      {
        next=true;
      }
    }
    
    if(!nextAvail && next)
    {
      //either lead was last load or no loads available after lead, so check from beginning again
      for(int c=0; c<loads.length; c++)
      {
        if(lead1.equals(loads[c]))
        {
          //we got back to lead load and didn't find another available so just break
          break;
        }
        else
        {
          int p = ((BInteger)ro.get("out"+loads[c])).getInt();
          if(getOutStatus(p))
          {
            //the next load in sequence is available
            getNextLeadLoad().setValue(BDynamicEnum.make(p, BEnumRange.make(BSequencerOutputEnum.TYPE)));
            nextAvail = true;
            break;
          }
        }
      }
    }
  }
  
  long getRuntime(int index)
  {
    switch(index)
    {
    case 1: return getRuntimeA().getMillis();
    case 2: return getRuntimeB().getMillis();
    case 3: return getRuntimeC().getMillis();
    case 4: return getRuntimeD().getMillis();
    case 5: return getRuntimeE().getMillis();
    case 6: return getRuntimeF().getMillis();
    case 7: return getRuntimeG().getMillis();
    case 8: return getRuntimeH().getMillis();
    case 9: return getRuntimeI().getMillis();
    case 10: return getRuntimeJ().getMillis();
    }
    return Long.MAX_VALUE; 
  }

  boolean getOutValue(int index)
  {
    switch(index)
    {
    case 1: return getOutA().getValue();
    case 2: return getOutB().getValue();
    case 3: return getOutC().getValue();
    case 4: return getOutD().getValue();
    case 5: return getOutE().getValue();
    case 6: return getOutF().getValue();
    case 7: return getOutG().getValue();
    case 8: return getOutH().getValue();
    case 9: return getOutI().getValue();
    case 10: return getOutJ().getValue();
    }
    return false; 
  }
  
  boolean getOutStatus(int index)
  {
    switch(index)
    {
      //changed from isValid to isOk
      case 1: return getOutA().getStatus().isOk();
      case 2: return getOutB().getStatus().isOk();
      case 3: return getOutC().getStatus().isOk();
      case 4: return getOutD().getStatus().isOk();
      case 5: return getOutE().getStatus().isOk();
      case 6: return getOutF().getStatus().isOk();
      case 7: return getOutG().getStatus().isOk();
      case 8: return getOutH().getStatus().isOk();
      case 9: return getOutI().getStatus().isOk();
      case 10: return getOutJ().getStatus().isOk();
      
    }
    return false;
  }
  
  boolean getFeedbackValue(int index)
  {
    switch(index)
    {
    case 1: return getFeedbackA().getValue();
    case 2: return getFeedbackB().getValue();
    case 3: return getFeedbackC().getValue();
    case 4: return getFeedbackD().getValue();
    case 5: return getFeedbackE().getValue();
    case 6: return getFeedbackF().getValue();
    case 7: return getFeedbackG().getValue();
    case 8: return getFeedbackH().getValue();
    case 9: return getFeedbackI().getValue();
    case 10: return getFeedbackJ().getValue();
    }
    return false; 
  }
  
  // This method will start the required outputs beginning with the 
  // lead load.
  void startOutput()
  {
    if(getDebug()) System.out.println(this.getSlotPath() + " - running start output");
    
    if(!getSchedule().getValue())
    {
      //stopOutput();
      return;
    }
    
    int lead = getLeadLoad().getEnum().getOrdinal();
    int req = getRequiredLoads();
    int started = 0;
    //boolean outputChanged = false;
    
    BRotationOrder ro = getRotationOrder();
    String current = ro.getOrder();
    String[] loads = TextUtil.splitAndTrim(current, ',');
    boolean leadStarted = false;
    
    if(getDebug()) System.out.println("lead load is " + lead + " , required loads " + req );
    
    //iterate through loads in configured order, don't start a load until the lead load is found
    //then continue starting loads until the end of the for loop
    for(int k=0; k<loads.length; k++)
    {
      
      if(ro.get("out"+loads[k])==null) continue;
      
      int id = ((BInteger)ro.get("out"+loads[k])).getInt();
      
      if(id==lead)
      {
        leadStarted = true; //could be just checked
        if(started < req && getStatusOutput(lead).getStatus().isOk())
        {
          //this is the lead output and ok to run
          if(getStatusOutput(lead).getValue()==false)
          {
            getStatusOutput(lead).setValue(true);
            startFeedbackTimer(lead);
            //outputChanged=true;
          }
          started = started + 1;
          if(getDebug()) System.out.println("checked lead load, started " + started);
        }
        else
        {
          //req must be zero so make sure to stop this load
          //getStatusOutput(lead).setValue(false);
          if(getRequiredLoads()==0) getStatusOutput(lead).setValue(false);
          
          if(getDebug()) System.out.println("checking " + loads[k] + " , enough loads already started");
        }
      }
      else if(id!=lead && leadStarted && started<req)
      {
        //already tried to start lead so try next load
        if(getStatusOutput(id).getStatus().isOk())
        {
          if(getStatusOutput(id).getValue()==false)
          {
            getStatusOutput(id).setValue(true);
            startFeedbackTimer(id);
            //outputChanged=true;
          }
          started = started + 1;
          if(getDebug()) System.out.println("checked lead load, started " + started);
        }
      }
      else if(started==req)
      {
        //we have enough loads running so turn off any remaining loads
        getStatusOutput(id).setValue(false);
        if(getDebug()) System.out.println("checking " + loads[k] + " , enough loads already started");
      }
    }
    
    //go through the loads again and finish turning off loads or starting loads if still need to
    for(int k=0; k<loads.length; k++)
    {
      if(ro.get("out"+loads[k])==null) continue;
      
      int id = ((BInteger)ro.get("out"+loads[k])).getInt();
      if(id==lead)
      {
        //we got back around to lead load so just break
        break;
      }
      else if(started<req)
      {
        if(getStatusOutput(id).getStatus().isOk())
        {
          if(getStatusOutput(id).getValue()==false)
          {
            getStatusOutput(id).setValue(true);
            startFeedbackTimer(id);
            //outputChanged=true;
          }
          started = started + 1;
          if(getDebug()) System.out.println("checked lead load, started " + started);
        }
      }
      else if(started==req)
      {
        getStatusOutput(id).setValue(false);
        if(getDebug()) System.out.println("checking " + loads[k] + " , enough loads already started");
      }
    }  
            
    if(started < req)
    {
      if(getDebug()) System.out.println("didn't start the required number of loads");
      
      setStatus(BStatus.make(getStatus(), BStatus.ALARM, true));
      setFaultCause("Not enough loads available to start the required number!");
    }
    else
    {
      if(getDebug()) System.out.println("started the required number of loads");
      
      setStatus(BStatus.make(getStatus(), BStatus.ALARM, false));
      if(!getStatus().isFault()) setFaultCause("");
    }
    
    //if(outputChanged) startFeedbackTimer();
    checkNextLoad();
  }
    
  BStatusBoolean getStatusOutput(int index)
  {
    switch(index)
    {
    case 1: return getOutA();
    case 2: return getOutB();
    case 3: return getOutC();
    case 4: return getOutD();
    case 5: return getOutE();
    case 6: return getOutF();
    case 7: return getOutG();
    case 8: return getOutH();
    case 9: return getOutI();
    case 10: return getOutJ();
    }
    if(index < 1) return getOutA();
    return getOutJ();
  }
  
  private int checkFailed()
  {
    int failed = 0;
    for(int i=1;i<=getNumberOutputs();i++)
    {
      BStatusBoolean out = getStatusOutput(i);
      if(out.getStatus().isAlarm()) failed = failed + 1;
    }
    return failed;
  }

  
  // Turn off all outputs
  void stopOutput()
  {
    if(getDebug()) System.out.println(this.getSlotPath() + " stopping outputs");
	
	for(int i = 1; i <= numOutputs; i++)
    {
      getStatusOutput(i).setValue(false);
	}
    checkRunningStatus();
  }
  
  void startCountdownTimer()
  {
    if(countdownTicket != null) countdownTicket.cancel();
    countdownTicket = Clock.schedulePeriodically(this, BRelTime.makeMinutes(1), countdownTimerExpired, null);
  }
  // start rotate timer. It will call back execute when it expires.
  void startRotateTimer()
  {            
    if (rotateTicket != null) rotateTicket.cancel();
    rotateTicket = Clock.schedulePeriodically(this, BRelTime.makeMinutes(1), rotateTimerExpired, null);
  }    
  
  // start feedback timer. It will call back execute when it expires.
  void startFeedbackTimer(int timer)
  {            
    switch(timer)
    {
      case 1:
        if (feedbackTicketA != null) feedbackTicketA.cancel();
        feedbackTimerActiveA = true;
        feedbackTicketA = Clock.schedule(this, getFeedbackDelay(), feedbackTimerExpiredA, null);
        
      case 2:
        if (feedbackTicketB != null) feedbackTicketB.cancel();
        feedbackTimerActiveB = true;
        feedbackTicketB = Clock.schedule(this, getFeedbackDelay(), feedbackTimerExpiredB, null);
        
      case 3:
        if (feedbackTicketC != null) feedbackTicketC.cancel();
        feedbackTimerActiveC = true;
        feedbackTicketC = Clock.schedule(this, getFeedbackDelay(), feedbackTimerExpiredC, null);
        
      case 4:
        if (feedbackTicketD != null) feedbackTicketD.cancel();
        feedbackTimerActiveD = true;
        feedbackTicketD = Clock.schedule(this, getFeedbackDelay(), feedbackTimerExpiredD, null);
        
      case 5:
        if (feedbackTicketE != null) feedbackTicketE.cancel();
        feedbackTimerActiveE = true;
        feedbackTicketE = Clock.schedule(this, getFeedbackDelay(), feedbackTimerExpiredE, null);
        
      case 6:
        if (feedbackTicketF != null) feedbackTicketF.cancel();
        feedbackTimerActiveF = true;
        feedbackTicketF = Clock.schedule(this, getFeedbackDelay(), feedbackTimerExpiredF, null);
        
      case 7:
        if (feedbackTicketG != null) feedbackTicketG.cancel();
        feedbackTimerActiveG = true;
        feedbackTicketG = Clock.schedule(this, getFeedbackDelay(), feedbackTimerExpiredG, null);
        
      case 8:
        if (feedbackTicketH != null) feedbackTicketH.cancel();
        feedbackTimerActiveH = true;
        feedbackTicketH = Clock.schedule(this, getFeedbackDelay(), feedbackTimerExpiredH, null);
        
      case 9:
        if (feedbackTicketI != null) feedbackTicketI.cancel();
        feedbackTimerActiveI = true;
        feedbackTicketI = Clock.schedule(this, getFeedbackDelay(), feedbackTimerExpiredI, null);
        
      case 10:
        if (feedbackTicketJ != null) feedbackTicketJ.cancel();
        feedbackTimerActiveJ = true;
        feedbackTicketJ = Clock.schedule(this, getFeedbackDelay(), feedbackTimerExpiredJ, null);
    }
  }    
  
  void startRuntimeTimer()
  {
    if(runtimeTicket != null) runtimeTicket.cancel();
    runtimeTicket = Clock.schedulePeriodically(this, BRelTime.makeMinutes(1), runtimeTimerExpired, null);
  }
  
  void checkRunningStatus()
  {
    if(getDebug()) System.out.println(this.getSlotPath() + "checking running status");
    
    startedLoads = 0;
    //runningLoads = 0;
    
    BStatusBoolean out;
    boolean feedback;
    boolean isFailed=false;
    
    for(int i=1; i <= numOutputs; i++)
    {
      
      out = getStatusOutput(i);
      feedback = getFeedbackValue(i);
      
      if(getDebug()) System.out.println("load " + i + " out - " + out.getValue() + " | feedback - " + feedback);
      
      // short cirucit check if feedbackTimerActive for this load
      if(getFeedbackTimer(i)) continue;
      
      if(out.getStatus().isAlarm())
      {
        // short circuit because load is already failed and we may have turned the value off
      }
      else if(out.getValue() && !feedback)
      {
        out.setStatusInAlarm(true);
        isFailed = true;
        startedLoads = startedLoads + 1;
        
        if(getStopFailedLoad().getValue()) out.setValue(false);
      }
      else if(out.getValue() && feedback)
      {
        //out.setStatusInAlarm(false);
        startedLoads = startedLoads + 1;
        //runningLoads = runningLoads + 1;
      }
    }
    
    if(isFailed) getLoadFailed().setValue(true);
    
    if(isFailed || startedLoads > getRequiredLoads()) 
    {  
      if(getDebug()) System.out.println(startedLoads + " loads started, " + getRequiredLoads() + " loads required, or a load is failed");
      startOutput();
    }
  }

  private boolean getFeedbackTimer(int ordinal)
  {
    switch(ordinal)
    {
      case 1: return feedbackTimerActiveA;
      case 2: return feedbackTimerActiveB;
      case 3: return feedbackTimerActiveC;
      case 4: return feedbackTimerActiveD;
      case 5: return feedbackTimerActiveE;
      case 6: return feedbackTimerActiveF;
      case 7: return feedbackTimerActiveG;
      case 8: return feedbackTimerActiveH;
      case 9: return feedbackTimerActiveI;
      case 10: return feedbackTimerActiveJ;
    }
    return false;
  }
  
  private int getLoadBit(int ordinal)
  {
    //using the enum ordinal get the load bit value
    int loadBit = 0;
    switch(ordinal)
    {
      case 1:
        loadBit = BDisabledLoadsBitString.OUT_A;
        break;
        
      case 2:
        loadBit = BDisabledLoadsBitString.OUT_B;
        break;
        
      case 3:
        loadBit = BDisabledLoadsBitString.OUT_C;
        break;
        
      case 4:
        loadBit = BDisabledLoadsBitString.OUT_D;
        break;
        
      case 5:
        loadBit = BDisabledLoadsBitString.OUT_E;
        break;
        
      case 6:
        loadBit = BDisabledLoadsBitString.OUT_F;
        break;
        
      case 7:
        loadBit = BDisabledLoadsBitString.OUT_G;
        break;
        
      case 8:
        loadBit = BDisabledLoadsBitString.OUT_H;
        break;
        
      case 9:
        loadBit = BDisabledLoadsBitString.OUT_I;
        break;
        
      case 10:
        loadBit = BDisabledLoadsBitString.OUT_J;
        break;
    }
    
    return loadBit;
  }
  
  /**
   * Apply the "facets" property to the "out" property.
   */
  public BFacets getSlotFacets(Slot slot)
  {
    if (slot.getName().startsWith("out")) return getFacets();
    else if (slot.getName()=="enableOutput" || slot.getName()=="disableOutput" || slot.getName()=="resetFailedLoad")
      return getActionFacets();
    
    return super.getSlotFacets(slot);
  }


  public BIcon getIcon() { return icon; }
  private static final BIcon icon = BIcon.std("reorder.png");

////////////////////////////////////////////////////////////////
// Attributes
////////////////////////////////////////////////////////////////
  int numOutputs;
  int startedLoads;
  //int runningLoads;
  
  boolean feedbackTimerActiveA = false;
  boolean feedbackTimerActiveB = false;
  boolean feedbackTimerActiveC = false;
  boolean feedbackTimerActiveD = false;
  boolean feedbackTimerActiveE = false;
  boolean feedbackTimerActiveF = false;
  boolean feedbackTimerActiveG = false;
  boolean feedbackTimerActiveH = false;
  boolean feedbackTimerActiveI = false;
  boolean feedbackTimerActiveJ = false;
  
  boolean currentIn;
  boolean wasOn = false;
  boolean isFirst = false;
  
  Clock.Ticket feedbackTicketA;    // Used to manage the on delay timer
  Clock.Ticket feedbackTicketB;
  Clock.Ticket feedbackTicketC;
  Clock.Ticket feedbackTicketD;
  Clock.Ticket feedbackTicketE;
  Clock.Ticket feedbackTicketF;
  Clock.Ticket feedbackTicketG;
  Clock.Ticket feedbackTicketH;
  Clock.Ticket feedbackTicketI;
  Clock.Ticket feedbackTicketJ;
  
  Clock.Ticket rotateTicket;    // Used to manage the load rotation
  Clock.Ticket startupTicket;   // delays execution on station start
  Clock.Ticket runtimeTicket;   
  Clock.Ticket countdownTicket;


}
