/*
 * Copyright 2003, Tridium, Inc. All Rights Reserved.
 */

package com.vykon.vykonPro.hvac;

import java.util.Arrays;

import javax.baja.status.BStatusBoolean;
import javax.baja.status.BStatusEnum;
import javax.baja.sys.Action;
import javax.baja.sys.BAbsTime;
import javax.baja.sys.BComponent;
import javax.baja.sys.BDynamicEnum;
import javax.baja.sys.BEnum;
import javax.baja.sys.BEnumRange;
import javax.baja.sys.BFacets;
import javax.baja.sys.BIcon;
import javax.baja.sys.BLink;
import javax.baja.sys.BRelTime;
import javax.baja.sys.Clock;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Slot;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.vykon.vykonPro.enums.BRuntimeBalancerOutputEnum;
import com.vykon.vykonPro.util.BDisabledLoadsBitString;
import com.vykon.vykonPro.util.BRuntimeItem;
import com.vykon.vykonPro.util.RuntimeItemComparator;
import com.vykon.vykonPro.util.RuntimeItemComparatorDesc;


/**
 * BRuntimeBalancer is a component that can provide control of 2 - 10 loads 
 * to balance runtime.  
 *
 * @author    James Johnson
 * @creation  05 April 2004
 * @version   $Revision: 7$ $Date: 3/3/2004 8:48:19 AM$
 * @since     Baja 1.0
 */
public class BRuntimeBalancer
  extends BComponent
{
  /*-
  
  class BRuntimeBalancer
  {
    properties
    {
      facets: BFacets
        -- These facets are applied against the out property.
        flags{ operator }
        default {[ BFacets.makeBoolean() ]}
      
      actionFacets: BFacets
        -- These facets are applied against the enableOutput and disableOutput action slots.
        flags{ operator }
        default{[ BFacets.makeEnum(BEnumRange.make(BRuntimeBalancerOutputEnum.TYPE)) ]}
        
      schedule: BStatusBoolean  //in
        flags { summary, operator }
        default {[ new BStatusBoolean() ]}
      
      loadFailed: BStatusBoolean
        flags { summary, readonly }
        default {[ new BStatusBoolean() ]}
        
      numberOutputs: int
        flags{ operator }
        default {[ 2 ]}
        slotfacets {[ BFacets.makeInt(null, 2, 10) ]}
        
      requiredLoads: int
        flags{ operator }
        default {[ 1 ]}
        slotfacets {[ BFacets.makeInt(null, 0, 10) ]}
      
      lastRotateTime: BAbsTime
        -- The timestamp of the last rotation of the lead load
        flags{ readonly, defaultOnClone }
        default{[ BAbsTime.NULL ]}
      
      runtimeRemainingUntilShift: BRelTime
        -- The amount of runtime left until the lead load will be rotated, if the rotateMode is runtime.
        flags{ readonly }
        default{[ BRelTime.makeSeconds(0) ]}
      
      leadLoad: BStatusEnum
        -- The load output which is designated as lead.
        flags{ readonly }
        default{[ new BStatusEnum(BDynamicEnum.make(1, BEnumRange.make(BRuntimeBalancerOutputEnum.TYPE))) ]}
      
      leadRuntimeAtStart: BRelTime
        -- The current accumulated runtime of the lead output recorded when the load is designated as lead
        flags{ readonly }
        default{[ BRelTime.makeSeconds(0) ]}
      
      nextLoad: String
        -- The load output which will be started next if another load is required.
        flags{ readonly }
        default{[ "" ]}
        
      nextLeadLoad: BStatusEnum
        -- The load output which should be the lead load after rotateInterval has elapsed.
        flags{ readonly }
        default{[ new BStatusEnum(BDynamicEnum.make(2, BEnumRange.make(BRuntimeBalancerOutputEnum.TYPE))) ]}
        
      rotateMode: boolean
        flags{ operator }
        default{[ false ]}
        slotfacets {[ BFacets.makeBoolean("manual", "interval") ]}
        
      rotateInterval: BRelTime  //maxRuntime
        flags{ operator }
        default {[ BRelTime.makeHours(168) ]}
      
      feedbackDelay: BRelTime
        flags{ operator }
        default {[ BRelTime.make(30000) ]}

      outA: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}

      outB: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}

      outC: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outD: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outE: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outF: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outG: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outH: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outI: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      outJ: BStatusBoolean
        flags { transient }
        default {[ new BStatusBoolean() ]}
    
      feedbackA: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackB: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackC: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackD: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackE: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackF: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackG: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackH: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackI: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      feedbackJ: BStatusBoolean
        flags { transient, summary }
        default {[ new BStatusBoolean() ]}
      
      runtimeA: BRelTime
        flags { transient, summary }
        default {[ BRelTime.make(0) ]}

      runtimeB: BRelTime
        flags { transient, summary }
        default {[ BRelTime.make(0) ]}

      runtimeC: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeD: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeE: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeF: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeG: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeH: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeI: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      runtimeJ: BRelTime
        flags { transient }
        default {[ BRelTime.make(0) ]}

      disabledLoads: BDisabledLoadsBitString
        flags { readonly, hidden, defaultOnClone }
        default {[ new BDisabledLoadsBitString(0) ]}
    }

    actions
    {
      startupTimerExpired()
        flags { hidden }
        
      rotateTimerExpired()
        flags { hidden }
        
      countdownTimerExpired()
        flags { hidden }
      
      feedbackTimerExpired()
        flags { hidden }
        
      disableOutput(output:BEnum)
         default {[ ( BDynamicEnum.make(0, BEnumRange.make( BRuntimeBalancerOutputEnum.TYPE))) ]}
         
      enableOutput(output:BEnum)
         default {[ ( BDynamicEnum.make(0, BEnumRange.make( BRuntimeBalancerOutputEnum.TYPE))) ]}
      
      manualCmd(manCmd:BManualBalancerCmd)
        default {[ new BManualBalancerCmd() ]}
        
      manualRotate()
      
    }
  }
  
  -*/
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.vykon.vykonPro.hvac.BRuntimeBalancer(701597561)1.0$ @*/
/* Generated Thu Feb 07 10:24:59 EST 2013 by Slot-o-Matic 2000 (c) Tridium, Inc. 2000 */

////////////////////////////////////////////////////////////////
// Property "facets"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>facets</code> property.
   * These facets are applied against the out property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getFacets
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setFacets
   */
  public static final Property facets = newProperty(Flags.OPERATOR, BFacets.makeBoolean(),null);
  
  /**
   * Get the <code>facets</code> property.
   * These facets are applied against the out property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#facets
   */
  public BFacets getFacets() { return (BFacets)get(facets); }
  
  /**
   * Set the <code>facets</code> property.
   * These facets are applied against the out property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#facets
   */
  public void setFacets(BFacets v) { set(facets,v,null); }

////////////////////////////////////////////////////////////////
// Property "actionFacets"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>actionFacets</code> property.
   * These facets are applied against the enableOutput and disableOutput action slots.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getActionFacets
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setActionFacets
   */
  public static final Property actionFacets = newProperty(Flags.OPERATOR, BFacets.makeEnum(BEnumRange.make(BRuntimeBalancerOutputEnum.TYPE)),null);
  
  /**
   * Get the <code>actionFacets</code> property.
   * These facets are applied against the enableOutput and disableOutput action slots.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#actionFacets
   */
  public BFacets getActionFacets() { return (BFacets)get(actionFacets); }
  
  /**
   * Set the <code>actionFacets</code> property.
   * These facets are applied against the enableOutput and disableOutput action slots.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#actionFacets
   */
  public void setActionFacets(BFacets v) { set(actionFacets,v,null); }

////////////////////////////////////////////////////////////////
// Property "schedule"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>schedule</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getSchedule
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setSchedule
   */
  public static final Property schedule = newProperty(Flags.SUMMARY|Flags.OPERATOR, new BStatusBoolean(),null);
  
  /**
   * Get the <code>schedule</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#schedule
   */
  public BStatusBoolean getSchedule() { return (BStatusBoolean)get(schedule); }
  
  /**
   * Set the <code>schedule</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#schedule
   */
  public void setSchedule(BStatusBoolean v) { set(schedule,v,null); }

////////////////////////////////////////////////////////////////
// Property "loadFailed"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>loadFailed</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getLoadFailed
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setLoadFailed
   */
  public static final Property loadFailed = newProperty(Flags.SUMMARY|Flags.READONLY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>loadFailed</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#loadFailed
   */
  public BStatusBoolean getLoadFailed() { return (BStatusBoolean)get(loadFailed); }
  
  /**
   * Set the <code>loadFailed</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#loadFailed
   */
  public void setLoadFailed(BStatusBoolean v) { set(loadFailed,v,null); }

////////////////////////////////////////////////////////////////
// Property "numberOutputs"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>numberOutputs</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getNumberOutputs
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setNumberOutputs
   */
  public static final Property numberOutputs = newProperty(Flags.OPERATOR, 2,BFacets.makeInt(null, 2, 10));
  
  /**
   * Get the <code>numberOutputs</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#numberOutputs
   */
  public int getNumberOutputs() { return getInt(numberOutputs); }
  
  /**
   * Set the <code>numberOutputs</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#numberOutputs
   */
  public void setNumberOutputs(int v) { setInt(numberOutputs,v,null); }

////////////////////////////////////////////////////////////////
// Property "requiredLoads"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>requiredLoads</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRequiredLoads
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRequiredLoads
   */
  public static final Property requiredLoads = newProperty(Flags.OPERATOR, 1,BFacets.makeInt(null, 0, 10));
  
  /**
   * Get the <code>requiredLoads</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#requiredLoads
   */
  public int getRequiredLoads() { return getInt(requiredLoads); }
  
  /**
   * Set the <code>requiredLoads</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#requiredLoads
   */
  public void setRequiredLoads(int v) { setInt(requiredLoads,v,null); }

////////////////////////////////////////////////////////////////
// Property "lastRotateTime"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>lastRotateTime</code> property.
   * The timestamp of the last rotation of the lead load
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getLastRotateTime
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setLastRotateTime
   */
  public static final Property lastRotateTime = newProperty(Flags.READONLY|Flags.DEFAULT_ON_CLONE, BAbsTime.NULL,null);
  
  /**
   * Get the <code>lastRotateTime</code> property.
   * The timestamp of the last rotation of the lead load
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#lastRotateTime
   */
  public BAbsTime getLastRotateTime() { return (BAbsTime)get(lastRotateTime); }
  
  /**
   * Set the <code>lastRotateTime</code> property.
   * The timestamp of the last rotation of the lead load
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#lastRotateTime
   */
  public void setLastRotateTime(BAbsTime v) { set(lastRotateTime,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeRemainingUntilShift"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeRemainingUntilShift</code> property.
   * The amount of runtime left until the lead load will
   * be rotated, if the rotateMode is runtime.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRuntimeRemainingUntilShift
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRuntimeRemainingUntilShift
   */
  public static final Property runtimeRemainingUntilShift = newProperty(Flags.READONLY, BRelTime.makeSeconds(0),null);
  
  /**
   * Get the <code>runtimeRemainingUntilShift</code> property.
   * The amount of runtime left until the lead load will
   * be rotated, if the rotateMode is runtime.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeRemainingUntilShift
   */
  public BRelTime getRuntimeRemainingUntilShift() { return (BRelTime)get(runtimeRemainingUntilShift); }
  
  /**
   * Set the <code>runtimeRemainingUntilShift</code> property.
   * The amount of runtime left until the lead load will
   * be rotated, if the rotateMode is runtime.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeRemainingUntilShift
   */
  public void setRuntimeRemainingUntilShift(BRelTime v) { set(runtimeRemainingUntilShift,v,null); }

////////////////////////////////////////////////////////////////
// Property "leadLoad"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>leadLoad</code> property.
   * The load output which is designated as lead.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getLeadLoad
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setLeadLoad
   */
  public static final Property leadLoad = newProperty(Flags.READONLY, new BStatusEnum(BDynamicEnum.make(1, BEnumRange.make(BRuntimeBalancerOutputEnum.TYPE))),null);
  
  /**
   * Get the <code>leadLoad</code> property.
   * The load output which is designated as lead.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#leadLoad
   */
  public BStatusEnum getLeadLoad() { return (BStatusEnum)get(leadLoad); }
  
  /**
   * Set the <code>leadLoad</code> property.
   * The load output which is designated as lead.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#leadLoad
   */
  public void setLeadLoad(BStatusEnum v) { set(leadLoad,v,null); }

////////////////////////////////////////////////////////////////
// Property "leadRuntimeAtStart"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>leadRuntimeAtStart</code> property.
   * The current accumulated runtime of the lead output
   * recorded when the load is designated as lead
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getLeadRuntimeAtStart
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setLeadRuntimeAtStart
   */
  public static final Property leadRuntimeAtStart = newProperty(Flags.READONLY, BRelTime.makeSeconds(0),null);
  
  /**
   * Get the <code>leadRuntimeAtStart</code> property.
   * The current accumulated runtime of the lead output
   * recorded when the load is designated as lead
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#leadRuntimeAtStart
   */
  public BRelTime getLeadRuntimeAtStart() { return (BRelTime)get(leadRuntimeAtStart); }
  
  /**
   * Set the <code>leadRuntimeAtStart</code> property.
   * The current accumulated runtime of the lead output
   * recorded when the load is designated as lead
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#leadRuntimeAtStart
   */
  public void setLeadRuntimeAtStart(BRelTime v) { set(leadRuntimeAtStart,v,null); }

////////////////////////////////////////////////////////////////
// Property "nextLoad"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>nextLoad</code> property.
   * The load output which will be started next if another
   * load is required.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getNextLoad
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setNextLoad
   */
  public static final Property nextLoad = newProperty(Flags.READONLY, "",null);
  
  /**
   * Get the <code>nextLoad</code> property.
   * The load output which will be started next if another
   * load is required.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#nextLoad
   */
  public String getNextLoad() { return getString(nextLoad); }
  
  /**
   * Set the <code>nextLoad</code> property.
   * The load output which will be started next if another
   * load is required.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#nextLoad
   */
  public void setNextLoad(String v) { setString(nextLoad,v,null); }

////////////////////////////////////////////////////////////////
// Property "nextLeadLoad"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>nextLeadLoad</code> property.
   * The load output which should be the lead load after
   * rotateInterval has elapsed.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getNextLeadLoad
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setNextLeadLoad
   */
  public static final Property nextLeadLoad = newProperty(Flags.READONLY, new BStatusEnum(BDynamicEnum.make(2, BEnumRange.make(BRuntimeBalancerOutputEnum.TYPE))),null);
  
  /**
   * Get the <code>nextLeadLoad</code> property.
   * The load output which should be the lead load after
   * rotateInterval has elapsed.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#nextLeadLoad
   */
  public BStatusEnum getNextLeadLoad() { return (BStatusEnum)get(nextLeadLoad); }
  
  /**
   * Set the <code>nextLeadLoad</code> property.
   * The load output which should be the lead load after
   * rotateInterval has elapsed.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#nextLeadLoad
   */
  public void setNextLeadLoad(BStatusEnum v) { set(nextLeadLoad,v,null); }

////////////////////////////////////////////////////////////////
// Property "rotateMode"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>rotateMode</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRotateMode
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRotateMode
   */
  public static final Property rotateMode = newProperty(Flags.OPERATOR, false,BFacets.makeBoolean("manual", "interval"));
  
  /**
   * Get the <code>rotateMode</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#rotateMode
   */
  public boolean getRotateMode() { return getBoolean(rotateMode); }
  
  /**
   * Set the <code>rotateMode</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#rotateMode
   */
  public void setRotateMode(boolean v) { setBoolean(rotateMode,v,null); }

////////////////////////////////////////////////////////////////
// Property "rotateInterval"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>rotateInterval</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRotateInterval
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRotateInterval
   */
  public static final Property rotateInterval = newProperty(Flags.OPERATOR, BRelTime.makeHours(168),null);
  
  /**
   * Get the <code>rotateInterval</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#rotateInterval
   */
  public BRelTime getRotateInterval() { return (BRelTime)get(rotateInterval); }
  
  /**
   * Set the <code>rotateInterval</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#rotateInterval
   */
  public void setRotateInterval(BRelTime v) { set(rotateInterval,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackDelay"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackDelay</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getFeedbackDelay
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setFeedbackDelay
   */
  public static final Property feedbackDelay = newProperty(Flags.OPERATOR, BRelTime.make(30000),null);
  
  /**
   * Get the <code>feedbackDelay</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackDelay
   */
  public BRelTime getFeedbackDelay() { return (BRelTime)get(feedbackDelay); }
  
  /**
   * Set the <code>feedbackDelay</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackDelay
   */
  public void setFeedbackDelay(BRelTime v) { set(feedbackDelay,v,null); }

////////////////////////////////////////////////////////////////
// Property "outA"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outA</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getOutA
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setOutA
   */
  public static final Property outA = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outA</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outA
   */
  public BStatusBoolean getOutA() { return (BStatusBoolean)get(outA); }
  
  /**
   * Set the <code>outA</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outA
   */
  public void setOutA(BStatusBoolean v) { set(outA,v,null); }

////////////////////////////////////////////////////////////////
// Property "outB"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outB</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getOutB
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setOutB
   */
  public static final Property outB = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outB</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outB
   */
  public BStatusBoolean getOutB() { return (BStatusBoolean)get(outB); }
  
  /**
   * Set the <code>outB</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outB
   */
  public void setOutB(BStatusBoolean v) { set(outB,v,null); }

////////////////////////////////////////////////////////////////
// Property "outC"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outC</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getOutC
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setOutC
   */
  public static final Property outC = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outC</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outC
   */
  public BStatusBoolean getOutC() { return (BStatusBoolean)get(outC); }
  
  /**
   * Set the <code>outC</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outC
   */
  public void setOutC(BStatusBoolean v) { set(outC,v,null); }

////////////////////////////////////////////////////////////////
// Property "outD"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outD</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getOutD
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setOutD
   */
  public static final Property outD = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outD</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outD
   */
  public BStatusBoolean getOutD() { return (BStatusBoolean)get(outD); }
  
  /**
   * Set the <code>outD</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outD
   */
  public void setOutD(BStatusBoolean v) { set(outD,v,null); }

////////////////////////////////////////////////////////////////
// Property "outE"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outE</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getOutE
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setOutE
   */
  public static final Property outE = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outE</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outE
   */
  public BStatusBoolean getOutE() { return (BStatusBoolean)get(outE); }
  
  /**
   * Set the <code>outE</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outE
   */
  public void setOutE(BStatusBoolean v) { set(outE,v,null); }

////////////////////////////////////////////////////////////////
// Property "outF"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outF</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getOutF
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setOutF
   */
  public static final Property outF = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outF</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outF
   */
  public BStatusBoolean getOutF() { return (BStatusBoolean)get(outF); }
  
  /**
   * Set the <code>outF</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outF
   */
  public void setOutF(BStatusBoolean v) { set(outF,v,null); }

////////////////////////////////////////////////////////////////
// Property "outG"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outG</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getOutG
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setOutG
   */
  public static final Property outG = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outG</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outG
   */
  public BStatusBoolean getOutG() { return (BStatusBoolean)get(outG); }
  
  /**
   * Set the <code>outG</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outG
   */
  public void setOutG(BStatusBoolean v) { set(outG,v,null); }

////////////////////////////////////////////////////////////////
// Property "outH"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outH</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getOutH
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setOutH
   */
  public static final Property outH = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outH</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outH
   */
  public BStatusBoolean getOutH() { return (BStatusBoolean)get(outH); }
  
  /**
   * Set the <code>outH</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outH
   */
  public void setOutH(BStatusBoolean v) { set(outH,v,null); }

////////////////////////////////////////////////////////////////
// Property "outI"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outI</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getOutI
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setOutI
   */
  public static final Property outI = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outI</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outI
   */
  public BStatusBoolean getOutI() { return (BStatusBoolean)get(outI); }
  
  /**
   * Set the <code>outI</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outI
   */
  public void setOutI(BStatusBoolean v) { set(outI,v,null); }

////////////////////////////////////////////////////////////////
// Property "outJ"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>outJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getOutJ
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setOutJ
   */
  public static final Property outJ = newProperty(Flags.TRANSIENT, new BStatusBoolean(),null);
  
  /**
   * Get the <code>outJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outJ
   */
  public BStatusBoolean getOutJ() { return (BStatusBoolean)get(outJ); }
  
  /**
   * Set the <code>outJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#outJ
   */
  public void setOutJ(BStatusBoolean v) { set(outJ,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackA"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackA</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getFeedbackA
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setFeedbackA
   */
  public static final Property feedbackA = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackA</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackA
   */
  public BStatusBoolean getFeedbackA() { return (BStatusBoolean)get(feedbackA); }
  
  /**
   * Set the <code>feedbackA</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackA
   */
  public void setFeedbackA(BStatusBoolean v) { set(feedbackA,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackB"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackB</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getFeedbackB
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setFeedbackB
   */
  public static final Property feedbackB = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackB</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackB
   */
  public BStatusBoolean getFeedbackB() { return (BStatusBoolean)get(feedbackB); }
  
  /**
   * Set the <code>feedbackB</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackB
   */
  public void setFeedbackB(BStatusBoolean v) { set(feedbackB,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackC"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackC</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getFeedbackC
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setFeedbackC
   */
  public static final Property feedbackC = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackC</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackC
   */
  public BStatusBoolean getFeedbackC() { return (BStatusBoolean)get(feedbackC); }
  
  /**
   * Set the <code>feedbackC</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackC
   */
  public void setFeedbackC(BStatusBoolean v) { set(feedbackC,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackD"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackD</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getFeedbackD
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setFeedbackD
   */
  public static final Property feedbackD = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackD</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackD
   */
  public BStatusBoolean getFeedbackD() { return (BStatusBoolean)get(feedbackD); }
  
  /**
   * Set the <code>feedbackD</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackD
   */
  public void setFeedbackD(BStatusBoolean v) { set(feedbackD,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackE"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackE</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getFeedbackE
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setFeedbackE
   */
  public static final Property feedbackE = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackE</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackE
   */
  public BStatusBoolean getFeedbackE() { return (BStatusBoolean)get(feedbackE); }
  
  /**
   * Set the <code>feedbackE</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackE
   */
  public void setFeedbackE(BStatusBoolean v) { set(feedbackE,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackF"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackF</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getFeedbackF
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setFeedbackF
   */
  public static final Property feedbackF = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackF</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackF
   */
  public BStatusBoolean getFeedbackF() { return (BStatusBoolean)get(feedbackF); }
  
  /**
   * Set the <code>feedbackF</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackF
   */
  public void setFeedbackF(BStatusBoolean v) { set(feedbackF,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackG"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackG</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getFeedbackG
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setFeedbackG
   */
  public static final Property feedbackG = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackG</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackG
   */
  public BStatusBoolean getFeedbackG() { return (BStatusBoolean)get(feedbackG); }
  
  /**
   * Set the <code>feedbackG</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackG
   */
  public void setFeedbackG(BStatusBoolean v) { set(feedbackG,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackH"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackH</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getFeedbackH
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setFeedbackH
   */
  public static final Property feedbackH = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackH</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackH
   */
  public BStatusBoolean getFeedbackH() { return (BStatusBoolean)get(feedbackH); }
  
  /**
   * Set the <code>feedbackH</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackH
   */
  public void setFeedbackH(BStatusBoolean v) { set(feedbackH,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackI"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackI</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getFeedbackI
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setFeedbackI
   */
  public static final Property feedbackI = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackI</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackI
   */
  public BStatusBoolean getFeedbackI() { return (BStatusBoolean)get(feedbackI); }
  
  /**
   * Set the <code>feedbackI</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackI
   */
  public void setFeedbackI(BStatusBoolean v) { set(feedbackI,v,null); }

////////////////////////////////////////////////////////////////
// Property "feedbackJ"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getFeedbackJ
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setFeedbackJ
   */
  public static final Property feedbackJ = newProperty(Flags.TRANSIENT|Flags.SUMMARY, new BStatusBoolean(),null);
  
  /**
   * Get the <code>feedbackJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackJ
   */
  public BStatusBoolean getFeedbackJ() { return (BStatusBoolean)get(feedbackJ); }
  
  /**
   * Set the <code>feedbackJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackJ
   */
  public void setFeedbackJ(BStatusBoolean v) { set(feedbackJ,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeA"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeA</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRuntimeA
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRuntimeA
   */
  public static final Property runtimeA = newProperty(Flags.TRANSIENT|Flags.SUMMARY, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeA</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeA
   */
  public BRelTime getRuntimeA() { return (BRelTime)get(runtimeA); }
  
  /**
   * Set the <code>runtimeA</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeA
   */
  public void setRuntimeA(BRelTime v) { set(runtimeA,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeB"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeB</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRuntimeB
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRuntimeB
   */
  public static final Property runtimeB = newProperty(Flags.TRANSIENT|Flags.SUMMARY, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeB</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeB
   */
  public BRelTime getRuntimeB() { return (BRelTime)get(runtimeB); }
  
  /**
   * Set the <code>runtimeB</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeB
   */
  public void setRuntimeB(BRelTime v) { set(runtimeB,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeC"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeC</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRuntimeC
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRuntimeC
   */
  public static final Property runtimeC = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeC</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeC
   */
  public BRelTime getRuntimeC() { return (BRelTime)get(runtimeC); }
  
  /**
   * Set the <code>runtimeC</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeC
   */
  public void setRuntimeC(BRelTime v) { set(runtimeC,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeD"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeD</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRuntimeD
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRuntimeD
   */
  public static final Property runtimeD = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeD</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeD
   */
  public BRelTime getRuntimeD() { return (BRelTime)get(runtimeD); }
  
  /**
   * Set the <code>runtimeD</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeD
   */
  public void setRuntimeD(BRelTime v) { set(runtimeD,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeE"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeE</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRuntimeE
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRuntimeE
   */
  public static final Property runtimeE = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeE</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeE
   */
  public BRelTime getRuntimeE() { return (BRelTime)get(runtimeE); }
  
  /**
   * Set the <code>runtimeE</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeE
   */
  public void setRuntimeE(BRelTime v) { set(runtimeE,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeF"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeF</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRuntimeF
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRuntimeF
   */
  public static final Property runtimeF = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeF</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeF
   */
  public BRelTime getRuntimeF() { return (BRelTime)get(runtimeF); }
  
  /**
   * Set the <code>runtimeF</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeF
   */
  public void setRuntimeF(BRelTime v) { set(runtimeF,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeG"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeG</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRuntimeG
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRuntimeG
   */
  public static final Property runtimeG = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeG</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeG
   */
  public BRelTime getRuntimeG() { return (BRelTime)get(runtimeG); }
  
  /**
   * Set the <code>runtimeG</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeG
   */
  public void setRuntimeG(BRelTime v) { set(runtimeG,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeH"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeH</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRuntimeH
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRuntimeH
   */
  public static final Property runtimeH = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeH</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeH
   */
  public BRelTime getRuntimeH() { return (BRelTime)get(runtimeH); }
  
  /**
   * Set the <code>runtimeH</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeH
   */
  public void setRuntimeH(BRelTime v) { set(runtimeH,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeI"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeI</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRuntimeI
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRuntimeI
   */
  public static final Property runtimeI = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeI</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeI
   */
  public BRelTime getRuntimeI() { return (BRelTime)get(runtimeI); }
  
  /**
   * Set the <code>runtimeI</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeI
   */
  public void setRuntimeI(BRelTime v) { set(runtimeI,v,null); }

////////////////////////////////////////////////////////////////
// Property "runtimeJ"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>runtimeJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getRuntimeJ
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setRuntimeJ
   */
  public static final Property runtimeJ = newProperty(Flags.TRANSIENT, BRelTime.make(0),null);
  
  /**
   * Get the <code>runtimeJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeJ
   */
  public BRelTime getRuntimeJ() { return (BRelTime)get(runtimeJ); }
  
  /**
   * Set the <code>runtimeJ</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#runtimeJ
   */
  public void setRuntimeJ(BRelTime v) { set(runtimeJ,v,null); }

////////////////////////////////////////////////////////////////
// Property "disabledLoads"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>disabledLoads</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#getDisabledLoads
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#setDisabledLoads
   */
  public static final Property disabledLoads = newProperty(Flags.READONLY|Flags.HIDDEN|Flags.DEFAULT_ON_CLONE, new BDisabledLoadsBitString(0),null);
  
  /**
   * Get the <code>disabledLoads</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#disabledLoads
   */
  public BDisabledLoadsBitString getDisabledLoads() { return (BDisabledLoadsBitString)get(disabledLoads); }
  
  /**
   * Set the <code>disabledLoads</code> property.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#disabledLoads
   */
  public void setDisabledLoads(BDisabledLoadsBitString v) { set(disabledLoads,v,null); }

////////////////////////////////////////////////////////////////
// Action "startupTimerExpired"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>startupTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#startupTimerExpired()
   */
  public static final Action startupTimerExpired = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>startupTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#startupTimerExpired
   */
  public void startupTimerExpired() { invoke(startupTimerExpired,null,null); }

////////////////////////////////////////////////////////////////
// Action "rotateTimerExpired"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>rotateTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#rotateTimerExpired()
   */
  public static final Action rotateTimerExpired = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>rotateTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#rotateTimerExpired
   */
  public void rotateTimerExpired() { invoke(rotateTimerExpired,null,null); }

////////////////////////////////////////////////////////////////
// Action "countdownTimerExpired"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>countdownTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#countdownTimerExpired()
   */
  public static final Action countdownTimerExpired = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>countdownTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#countdownTimerExpired
   */
  public void countdownTimerExpired() { invoke(countdownTimerExpired,null,null); }

////////////////////////////////////////////////////////////////
// Action "feedbackTimerExpired"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>feedbackTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackTimerExpired()
   */
  public static final Action feedbackTimerExpired = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>feedbackTimerExpired</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#feedbackTimerExpired
   */
  public void feedbackTimerExpired() { invoke(feedbackTimerExpired,null,null); }

////////////////////////////////////////////////////////////////
// Action "disableOutput"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>disableOutput</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#disableOutput()
   */
  public static final Action disableOutput = newAction(0,( BDynamicEnum.make(0, BEnumRange.make( BRuntimeBalancerOutputEnum.TYPE))),null);
  
  /**
   * Invoke the <code>disableOutput</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#disableOutput
   */
  public void disableOutput(BEnum output) { invoke(disableOutput,output,null); }

////////////////////////////////////////////////////////////////
// Action "enableOutput"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>enableOutput</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#enableOutput()
   */
  public static final Action enableOutput = newAction(0,( BDynamicEnum.make(0, BEnumRange.make( BRuntimeBalancerOutputEnum.TYPE))),null);
  
  /**
   * Invoke the <code>enableOutput</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#enableOutput
   */
  public void enableOutput(BEnum output) { invoke(enableOutput,output,null); }

////////////////////////////////////////////////////////////////
// Action "manualCmd"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>manualCmd</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#manualCmd()
   */
  public static final Action manualCmd = newAction(0,new BManualBalancerCmd(),null);
  
  /**
   * Invoke the <code>manualCmd</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#manualCmd
   */
  public void manualCmd(BManualBalancerCmd manCmd) { invoke(manualCmd,manCmd,null); }

////////////////////////////////////////////////////////////////
// Action "manualRotate"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>manualRotate</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#manualRotate()
   */
  public static final Action manualRotate = newAction(0,null);
  
  /**
   * Invoke the <code>manualRotate</code> action.
   * @see com.vykon.vykonPro.hvac.BRuntimeBalancer#manualRotate
   */
  public void manualRotate() { invoke(manualRotate,null,null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BRuntimeBalancer.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  public BRuntimeBalancer()
  {
  }
  
  /**
   * Init if started after steady state has been reached.
   */
  public void started()
  {
    initNumberOutputs();
    checkNextLoad();
    startupTicket = Clock.schedule(this, BRelTime.makeSeconds(30), startupTimerExpired, null);
  }

  public void stopped()
  {
    if (feedbackTicket != null) feedbackTicket.cancel();
    if (rotateTicket != null) rotateTicket.cancel();
    if (startupTicket != null) startupTicket.cancel();
    if (countdownTicket != null) countdownTicket.cancel();
  }
  
/**
   * setoutput on in change.
   */
  public void changed(Property p, Context cx)
  {
    super.changed(p, cx);
    if (!Sys.atSteadyState() || !isRunning())
      return;

    if(p == numberOutputs)
    {
      initNumberOutputs();
    }
    else if(p == schedule) 
    {
      calculate();
    }
    else if(p == requiredLoads)
    {
      if(getSchedule().getValue()) updateRequiredLoads();
    }
    else if(     p == feedbackA ||
            p == feedbackB ||
            p == feedbackC ||
            p == feedbackD ||
            p == feedbackE ||
            p == feedbackF ||
            p == feedbackG ||
            p == feedbackH ||
            p == feedbackI ||
            p == feedbackJ)
    {
      if(!currentIn) return;
      if(feedbackTimerActive) return;
      checkRunningStatus();
    }
    else if(p == rotateMode)
    {
      if(getRotateMode())
      {
        //it was in interval mode so cancel ticket
        if(rotateTicket != null) rotateTicket.cancel();
      }
      else
      {
        //it was in manual mode so start rotate timer
        startRotateTimer();
      }
    }
    else if(p == rotateInterval)
    {
      if(!getRotateMode()) startRotateTimer();
    }
  }

  public void initNumberOutputs()
  {
    numOutputs = getNumberOutputs();
    
    rt = new BRuntimeItem[numOutputs];
    
    int setFlags;
    for(int i = 0; i < 10; i++)
    {
      if(i >= numOutputs)
        setFlags = Flags.HIDDEN | Flags.TRANSIENT;
      else
        setFlags = Flags.SUMMARY | Flags.TRANSIENT;
      switch(i)
      {
        //should possibly check if runtime or feedback slots are linked and delete the links
      case 0: initSlot("outA", setFlags | Flags.READONLY); initSlot("runtimeA", setFlags); initSlot("feedbackA", setFlags); break;
      case 1: initSlot("outB", setFlags | Flags.READONLY); initSlot("runtimeB", setFlags); initSlot("feedbackB", setFlags); break;
      case 2: initSlot("outC", setFlags | Flags.READONLY); initSlot("runtimeC", setFlags); initSlot("feedbackC", setFlags); break;
      case 3: initSlot("outD", setFlags | Flags.READONLY); initSlot("runtimeD", setFlags); initSlot("feedbackD", setFlags); break;
      case 4: initSlot("outE", setFlags | Flags.READONLY); initSlot("runtimeE", setFlags); initSlot("feedbackE", setFlags); break;
      case 5: initSlot("outF", setFlags | Flags.READONLY); initSlot("runtimeF", setFlags); initSlot("feedbackF", setFlags); break;
      case 6: initSlot("outG", setFlags | Flags.READONLY); initSlot("runtimeG", setFlags); initSlot("feedbackG", setFlags); break;
      case 7: initSlot("outH", setFlags | Flags.READONLY); initSlot("runtimeH", setFlags); initSlot("feedbackH", setFlags); break;
      case 8: initSlot("outI", setFlags | Flags.READONLY); initSlot("runtimeI", setFlags); initSlot("feedbackI", setFlags); break;
      case 9: initSlot("outJ", setFlags | Flags.READONLY); initSlot("runtimeJ", setFlags); initSlot("feedbackJ", setFlags); break;
      }
      
      //check for any previously disabled loads and disable them on start
      int[] disabledLoads = getDisabledLoads().getOrdinals();
      
      for(int k=0; k<disabledLoads.length; k++)
      {
        if(getDisabledLoads().getBit(disabledLoads[k]))
        {
          //the bitString contains this bit so disable the load
          String loadName = getDisabledLoads().getTag(disabledLoads[k]);
          ((BStatusBoolean)this.get(loadName)).setStatusDisabled(true);
        }
      }
    }
  }

  void initSlot(String slot, int flags)
  {
      try { setFlags(getSlot(slot), flags, null);} catch(Exception e) {};
  }


  public void doStartupTimerExpired()
  {
    calculate();
  }
  
  
  public void doRotateTimerExpired()
  {
    if(!currentIn) return;
    getRuntimes();  // load current runtimes.
    startOutput();
    startRotateTimer();     // start rotate timer
  }

  public void doCountdownTimerExpired()
  {
    //TODO update time remaining
    BAbsTime currentTime = Clock.time();
    BAbsTime nextTime = getLastRotateTime().add(getRotateInterval());
    BRelTime timeLeft = currentTime.delta(nextTime);

    if (timeLeft.getMillis() > 1000L) //more than 1 second left
    {
      setRuntimeRemainingUntilShift(timeLeft);
    }
    else
    {
      setRuntimeRemainingUntilShift(BRelTime.make(0));
    }

  }
  
  public void doFeedbackTimerExpired()
  {
    feedbackTimerActive = false;
    if(!currentIn) return;
    checkRunningStatus();
  }

  public void doManualRotate()
  {
    if(!currentIn) return;
    getRuntimes();
    startOutput();
  }

  void calculate()
  { 
    lastRequiredLoads = getRequiredLoads();
    currentIn = getSchedule().getValue();
    getRuntimes();  // load current cycle counts.
    
    if(currentIn)  // if controlIn is active
    {
      if(!wasOn)   // and was not active on last execute
      {
        startOutput();          // start lowest runtime output
        
        //if rotateMode is false then start rotateTimer
        if(!getRotateMode()) startRotateTimer();     // start rotate timer
        else startCountdownTimer();
        
        wasOn = true;           // set was on flag.
      }
    }
    else  // control in is not active
    {
      if(wasOn)  // if was active on last execute
      {
        if(rotateTicket != null) rotateTicket.cancel();
        if(feedbackTicket != null) feedbackTicket.cancel();
        if(countdownTicket != null) countdownTicket.cancel();
        
        wasOn = false;             // clear was on flag.
      }
      stopOutput();              // stop all outputs
      
    }
  }

  // read linked runtimes into array
  // set unlinked runtimes to -1
  void getRuntimes()
  {
    for(int i = 0; i < numOutputs; i++)
    {
      //check if runtime slot is linked and status is not disabled
      if( isRuntimeLinked(i) && !getStatusOutput(i).getStatus().isDisabled())
        rt[i] = new BRuntimeItem(i, getRuntime(i));
      else
        rt[i] = new BRuntimeItem(i, Long.MAX_VALUE);
    }
  }

  
  //get array of running load runtimes
  void getOnRuntimes()
  {
    for(int i=0; i < numOutputs; i++)
    {
      if( isRuntimeLinked(i) && getOutValue(i) && !getStatusOutput(i).getStatus().isAlarm())
        rt[i] = new BRuntimeItem(i, getRuntime(i));
      //else if(getStatusOutput(i).getStatus().isAlarm())
      //  rt[i] = new BRuntimeItem(i, Long.MAX_VALUE);
      else
        rt[i] = new BRuntimeItem(i, Long.MIN_VALUE);
    }
  }
  
  
  //get array of non-running load runtimes
  void getOffRuntimes()
  {
    for(int i=0; i < numOutputs; i++)
    {
      if( isRuntimeLinked(i) && !getOutValue(i) && !getStatusOutput(i).getStatus().isDisabled())
        rt[i] = new BRuntimeItem(i, getRuntime(i));
      else
        rt[i] = new BRuntimeItem(i, Long.MAX_VALUE);
    }
  }
  
  long getRuntime(int index)
  {
    switch(index)
    {
    case 0: return getRuntimeA().getMillis();
    case 1: return getRuntimeB().getMillis();
    case 2: return getRuntimeC().getMillis();
    case 3: return getRuntimeD().getMillis();
    case 4: return getRuntimeE().getMillis();
    case 5: return getRuntimeF().getMillis();
    case 6: return getRuntimeG().getMillis();
    case 7: return getRuntimeH().getMillis();
    case 8: return getRuntimeI().getMillis();
    case 9: return getRuntimeJ().getMillis();
    }
    return Long.MAX_VALUE; 
  }

  boolean getOutValue(int index)
  {
    switch(index)
    {
    case 0: return getOutA().getValue();
    case 1: return getOutB().getValue();
    case 2: return getOutC().getValue();
    case 3: return getOutD().getValue();
    case 4: return getOutE().getValue();
    case 5: return getOutF().getValue();
    case 6: return getOutG().getValue();
    case 7: return getOutH().getValue();
    case 8: return getOutI().getValue();
    case 9: return getOutJ().getValue();
    }
    return false; 
  }
  
  
  boolean getFeedbackValue(int index)
  {
    switch(index)
    {
    case 0: return getFeedbackA().getValue();
    case 1: return getFeedbackB().getValue();
    case 2: return getFeedbackC().getValue();
    case 3: return getFeedbackD().getValue();
    case 4: return getFeedbackE().getValue();
    case 5: return getFeedbackF().getValue();
    case 6: return getFeedbackG().getValue();
    case 7: return getFeedbackH().getValue();
    case 8: return getFeedbackI().getValue();
    case 9: return getFeedbackJ().getValue();
    }
    return false; 
  }
  
  // Test to see if runtime slot is linked
  boolean isRuntimeLinked(int index)
  {
    BLink[] links = null;
    switch(index)
    {
    case 0: links = getLinks(runtimeA); break;
    case 1: links = getLinks(runtimeB); break;
    case 2: links = getLinks(runtimeC); break;
    case 3: links = getLinks(runtimeD); break;
    case 4: links = getLinks(runtimeE); break;
    case 5: links = getLinks(runtimeF); break;
    case 6: links = getLinks(runtimeG); break;
    case 7: links = getLinks(runtimeH); break;
    case 8: links = getLinks(runtimeI); break;
    case 9: links = getLinks(runtimeJ); break;
    }
    if(links == null)
      return false;
    return (links.length != 0);
  }
  
  // This method will start the required outputs with the lowest runtime.
  // It will turn off all other outputs.
  void startOutput()
  {
    Arrays.sort(rt, new RuntimeItemComparator());
    boolean first = false;
    
    for( int i = 0; i < numOutputs; i++)
    {
      // get the int from the sorted array
      int j = rt[i].getOutSlot();
      if(i < getRequiredLoads())
      {
        getStatusOutput(j).setValue(true);
        if(first==false)
        {
          getLeadLoad().setValue(BDynamicEnum.make(j, BEnumRange.make(BRuntimeBalancerOutputEnum.TYPE)));
          first=true;
          //TODO update times
          setLastRotateTime(BAbsTime.now());
          setRuntimeRemainingUntilShift(getRotateInterval());
          setLeadRuntimeAtStart(BRelTime.make(getRuntime(j)));
          startCountdownTimer();
        }
      }
      else 
      {
        getStatusOutput(j).setValue(false);
      }
    }
    
    startFeedbackTimer();
    checkNextLoad();
  }


  //this method will start the required number of outputs to makeup for
  //failed loads.
  void startLag(int loads)
  {
    Arrays.sort(rt, new RuntimeItemComparator());
    
    for( int i=0; i < loads; i++)
    {
      //get the int from the sorted array for the output
      int j = rt[i].getOutSlot();
      getStatusOutput(j).setValue(true);
    }
    
    startFeedbackTimer();
    checkNextLoad();
  }
  
  //this method will stop the required number of outputs to restore
  //from starting aditional loads
  void stopLag(int loads)
  {
    Arrays.sort(rt, new RuntimeItemComparatorDesc());
    
    for( int i=0; i < loads; i++)
    {
      //get the int from the sorted array for the output
      int j = rt[i].getOutSlot();
      getStatusOutput(j).setValue(false);
    }
    
    checkNextLoad();
  }
  
  //this method will start or stop required number of outputs
  //to match the currently required loads
  void updateRequiredLoads()
  {
    if(getRequiredLoads() > lastRequiredLoads)
    {
      //need to start additional loads
      getOffRuntimes();
      startLag(getRequiredLoads()-lastRequiredLoads);
    }
    else if(getRequiredLoads() < lastRequiredLoads)
    {
      //need to stop some loads
      getOnRuntimes();
      stopLag(lastRequiredLoads-getRequiredLoads());
    }
    lastRequiredLoads = getRequiredLoads();
  }
  
  BStatusBoolean getStatusOutput(int index)
  {
    switch(index)
    {
    case 0: return getOutA();
    case 1: return getOutB();
    case 2: return getOutC();
    case 3: return getOutD();
    case 4: return getOutE();
    case 5: return getOutF();
    case 6: return getOutG();
    case 7: return getOutH();
    case 8: return getOutI();
    case 9: return getOutJ();
    }
    if(index < 0) return getOutA();
    return getOutJ();
  }

  
  // Turn off all outputs
  void stopOutput()
  {
    for(int i = 0; i < numOutputs; i++)
      getStatusOutput(i).setValue(false);
    
    checkNextLoad();
    if(countdownTicket != null) countdownTicket.cancel();
    setRuntimeRemainingUntilShift(BRelTime.make(0));
  }
  
  // Check the non-running loads to determine nextLoad
  void checkNextLoad()
  {
    getOffRuntimes();
    Arrays.sort(rt, new RuntimeItemComparator());
    int next = rt[0].getOutSlot();
    setNextLoad(getStatusOutput(next).getDisplayName(null));
    
    //TODO calculate next lead load
    getRuntimes();
    Arrays.sort(rt, new RuntimeItemComparator());
    for(int i=0; i<rt.length; i++)
    {
      int outputNum = rt[i].getOutSlot();
      if(outputNum==getLeadLoad().getEnum().getOrdinal())
      {
        //the load with the least runtime is currently the lead load
        long calcRuntime = getRuntime(outputNum) + getRuntimeRemainingUntilShift().getMillis();
        if(calcRuntime < getRuntime(rt[i+1].getOutSlot()))
        {
          //lead load will still have the least runtime at shift time
          getNextLeadLoad().setValue(BDynamicEnum.make(outputNum, BEnumRange.make(BRuntimeBalancerOutputEnum.TYPE)));
          break;
        }
      }
      getNextLeadLoad().setValue(BDynamicEnum.make(next, BEnumRange.make(BRuntimeBalancerOutputEnum.TYPE)));
      break;
    }
  }
  
  // start rotate timer. It will call back execute when it expires.
  void startRotateTimer()
  {            
    if (rotateTicket != null) rotateTicket.cancel();
    rotateTicket = Clock.schedule(this, getRotateInterval(), rotateTimerExpired, null);
  }    
  
  // start feedback timer. It will call back execute when it expires.
  void startFeedbackTimer()
  {            
    if (feedbackTicket != null) feedbackTicket.cancel();
    feedbackTimerActive = true;
    feedbackTicket = Clock.schedule(this, getFeedbackDelay(), feedbackTimerExpired, null);
  }
  
  void startCountdownTimer()
  {
    if(countdownTicket != null) countdownTicket.cancel();
    countdownTicket = Clock.schedulePeriodically(this, BRelTime.makeMinutes(1), countdownTimerExpired, null);
  }
  
  
  void checkRunningStatus()
  {
    startedLoads = 0;
    runningLoads = 0;
    
    BStatusBoolean out;
    boolean feedback;
    boolean isFailed=false;
    
    for(int i=0; i < numOutputs; i++)
    {
      out = getStatusOutput(i);
      feedback = getFeedbackValue(i);
      
      if(out.getValue() && !feedback)
      {
        out.setStatusInAlarm(true);
        isFailed = true;
        startedLoads = startedLoads + 1;
      }
      else if(out.getValue() && feedback)
      {
        out.setStatusInAlarm(false);
        startedLoads = startedLoads + 1;
        runningLoads = runningLoads + 1;
      }
      else out.setStatusInAlarm(false);
    }
    
    if(isFailed) getLoadFailed().setValue(true);
    else getLoadFailed().setValue(false);
   
    if(runningLoads < getRequiredLoads())
    {
      getOffRuntimes();
      startLag(getRequiredLoads()-runningLoads);
    }
    else if(runningLoads > getRequiredLoads())
    {
      getOnRuntimes();
      stopLag(runningLoads-getRequiredLoads());
    }
    
  }

  
  public void doDisableOutput(BEnum output)
  {
    BStatusBoolean currentOut;
    int outNmbr = output.getEnum().getOrdinal();
    currentOut = getStatusOutput(outNmbr);
    currentOut.setStatusDisabled(true);
    
    //update the disabledLoads bits
    int loadBit = getLoadBit(outNmbr);
    setDisabledLoads(BDisabledLoadsBitString.make(getDisabledLoads().getBits() | loadBit));
  }
  
  public void doEnableOutput(BEnum output)
  {
    BStatusBoolean currentOut;
    int outNmbr = output.getEnum().getOrdinal();
    currentOut = getStatusOutput(outNmbr);
    currentOut.setStatusDisabled(false);
    
    //update the disabledLoads bits
    int loadBit = getLoadBit(outNmbr);
    setDisabledLoads(BDisabledLoadsBitString.make(getDisabledLoads().getBits() & ~loadBit));
  }
  
  
  public void doManualCmd(BManualBalancerCmd manCmd)
  {
    
    BStatusBoolean currentOut;
    int outNmbr = manCmd.getOutput().getOrdinal();
    currentOut = getStatusOutput(outNmbr);
    
    if(manCmd.getCommand())
    {
      //command value is true so disable output
      currentOut.setStatusDisabled(true);
      
      //update the disabledLoads bits
      int loadBit = getLoadBit(outNmbr);
      setDisabledLoads(BDisabledLoadsBitString.make(getDisabledLoads().getBits() | loadBit));
    }
    else
    {
      //command value is false so enable output
      currentOut.setStatusDisabled(false);
      
      //update the disabledLoads bits
      int loadBit = getLoadBit(outNmbr);
      setDisabledLoads(BDisabledLoadsBitString.make(getDisabledLoads().getBits() & ~loadBit));
    }
  }
  
  private int getLoadBit(int ordinal)
  {
    //using the enum ordinal get the load bit value
    int loadBit = -1;
    switch(ordinal)
    {
      case 0:
        loadBit = BDisabledLoadsBitString.OUT_A;
        break;
        
      case 1:
        loadBit = BDisabledLoadsBitString.OUT_B;
        break;
        
      case 2:
        loadBit = BDisabledLoadsBitString.OUT_C;
        break;
        
      case 3:
        loadBit = BDisabledLoadsBitString.OUT_D;
        break;
        
      case 4:
        loadBit = BDisabledLoadsBitString.OUT_E;
        break;
        
      case 5:
        loadBit = BDisabledLoadsBitString.OUT_F;
        break;
        
      case 6:
        loadBit = BDisabledLoadsBitString.OUT_G;
        break;
        
      case 7:
        loadBit = BDisabledLoadsBitString.OUT_H;
        break;
        
      case 8:
        loadBit = BDisabledLoadsBitString.OUT_I;
        break;
        
      case 9:
        loadBit = BDisabledLoadsBitString.OUT_J;
        break;
    }
    
    return loadBit;
  }
  
  
  /**
   * Apply the "facets" property to the "out" property.
   */
  public BFacets getSlotFacets(Slot slot)
  {
    if (slot.getName().startsWith("out")) return getFacets();
    else if(slot.getName()=="enableOutput" || slot.getName()=="disableOutput")
      return getActionFacets();
    
    return super.getSlotFacets(slot);
  }


  public BIcon getIcon() { return icon; }
  private static final BIcon icon = BIcon.std("scales.png");

////////////////////////////////////////////////////////////////
// Attributes
////////////////////////////////////////////////////////////////
  int numOutputs;
  int lastNumberOutputs;
  int lastRequiredLoads;
  int startedLoads;
  int runningLoads;
  boolean feedbackTimerActive = false;
  boolean currentIn;
  boolean wasOn = false;
  BRuntimeItem rt[]; //= new BRuntimeItem[10];
  
  // rotate timer ticket.
  Clock.Ticket rotateTicket;    // Used to manage the on delay timer
  Clock.Ticket feedbackTicket;    // Used to manage the on delay timer
  Clock.Ticket startupTicket;
  Clock.Ticket countdownTicket;
}
