/**
 * Copyright 2010 - All Rights Reserved.
 */
package com.vykon.vykonPro.report;

import javax.baja.report.BReport;
import javax.baja.sys.Action;
import javax.baja.sys.BAbsTime;
import javax.baja.sys.BComponent;
import javax.baja.sys.BFacets;
import javax.baja.sys.BRelTime;
import javax.baja.sys.Clock;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Topic;
import javax.baja.sys.Type;

/**
 *  The reportDelay component is designed to be linked between an Export Source and a
 *  report recipient component, and to delay the processing of the report by the recipient. 
 *  This allows a report to be generated at a specific time, and the report is held in memory 
 *  by the report delay component until the delay time has expired. 
 *
 * @author E351261
 * @creation Oct 28, 2010
 *
 */
public class BReportDelay
    extends BComponent
{
  /*-
   class BReportDelay
   {
     properties
     {
       reportDelay:BRelTime
         -- amount of time to delay firing topic to route report to a recipient
         flags{ operator }
         default{[ BRelTime.makeMinutes(1) ]}
         slotfacets{[ BFacets.make(BFacets.MIN, BRelTime.makeSeconds(1)) ]}
         
       delayActive: boolean
         -- boolean property which indicates if the component is actively delaying a report
         flags{ readonly }
         default{[ false ]}
         
       nextTriggerTime: BAbsTime
         -- The time at which the component will forward the currently delayed report to the recipient.
         flags{ readonly }
         default{[ BAbsTime.NULL ]}
     }
     actions
     {
       route(report : BReport)
         -- action slot which is linked to a BReportSource component
         flags{ summary }
         default{[ new BReport() ]}
         
       delayExpired()
         flags{ hidden }
         
     }
     topics
     {
       out: BReport
         -- topic slot which is linked to a report recipient to route the report
         flags{ readonly, summary }
     }
   }
   -*/
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.vykon.vykonPro.report.BReportDelay(3066444365)1.0$ @*/
/* Generated Mon Nov 08 16:40:42 EST 2010 by Slot-o-Matic 2000 (c) Tridium, Inc. 2000 */

////////////////////////////////////////////////////////////////
// Property "reportDelay"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>reportDelay</code> property.
   * amount of time to delay firing topic to route report
   * to a recipient
   * @see com.vykon.vykonPro.report.BReportDelay#getReportDelay
   * @see com.vykon.vykonPro.report.BReportDelay#setReportDelay
   */
  public static final Property reportDelay = newProperty(Flags.OPERATOR, BRelTime.makeMinutes(1),BFacets.make(BFacets.MIN, BRelTime.makeSeconds(1)));
  
  /**
   * Get the <code>reportDelay</code> property.
   * @see com.vykon.vykonPro.report.BReportDelay#reportDelay
   */
  public BRelTime getReportDelay() { return (BRelTime)get(reportDelay); }
  
  /**
   * Set the <code>reportDelay</code> property.
   * @see com.vykon.vykonPro.report.BReportDelay#reportDelay
   */
  public void setReportDelay(BRelTime v) { set(reportDelay,v,null); }

////////////////////////////////////////////////////////////////
// Property "delayActive"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>delayActive</code> property.
   * boolean property which indicates if the component is actively delaying a report
   * @see com.vykon.vykonPro.report.BReportDelay#getDelayActive
   * @see com.vykon.vykonPro.report.BReportDelay#setDelayActive
   */
  public static final Property delayActive = newProperty(Flags.READONLY, false,null);
  
  /**
   * Get the <code>delayActive</code> property.
   * @see com.vykon.vykonPro.report.BReportDelay#delayActive
   */
  public boolean getDelayActive() { return getBoolean(delayActive); }
  
  /**
   * Set the <code>delayActive</code> property.
   * @see com.vykon.vykonPro.report.BReportDelay#delayActive
   */
  public void setDelayActive(boolean v) { setBoolean(delayActive,v,null); }

////////////////////////////////////////////////////////////////
// Property "nextTriggerTime"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>nextTriggerTime</code> property.
   * The time at which the component will forward the currently
   * delayed report to the recipient.
   * @see com.vykon.vykonPro.report.BReportDelay#getNextTriggerTime
   * @see com.vykon.vykonPro.report.BReportDelay#setNextTriggerTime
   */
  public static final Property nextTriggerTime = newProperty(Flags.READONLY, BAbsTime.NULL,null);
  
  /**
   * Get the <code>nextTriggerTime</code> property.
   * @see com.vykon.vykonPro.report.BReportDelay#nextTriggerTime
   */
  public BAbsTime getNextTriggerTime() { return (BAbsTime)get(nextTriggerTime); }
  
  /**
   * Set the <code>nextTriggerTime</code> property.
   * @see com.vykon.vykonPro.report.BReportDelay#nextTriggerTime
   */
  public void setNextTriggerTime(BAbsTime v) { set(nextTriggerTime,v,null); }

////////////////////////////////////////////////////////////////
// Action "route"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>route</code> action.
   * action slot which is linked to a BReportSource component
   * @see com.vykon.vykonPro.report.BReportDelay#route()
   */
  public static final Action route = newAction(Flags.SUMMARY,new BReport(),null);
  
  /**
   * Invoke the <code>route</code> action.
   * @see com.vykon.vykonPro.report.BReportDelay#route
   */
  public void route(BReport report) { invoke(route,report,null); }

////////////////////////////////////////////////////////////////
// Action "delayExpired"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>delayExpired</code> action.
   * @see com.vykon.vykonPro.report.BReportDelay#delayExpired()
   */
  public static final Action delayExpired = newAction(Flags.HIDDEN,null);
  
  /**
   * Invoke the <code>delayExpired</code> action.
   * @see com.vykon.vykonPro.report.BReportDelay#delayExpired
   */
  public void delayExpired() { invoke(delayExpired,null,null); }

////////////////////////////////////////////////////////////////
// Topic "out"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>out</code> topic.
   * topic slot which is linked to a report recipient to
   * route the report
   * @see com.vykon.vykonPro.report.BReportDelay#fireOut
   */
  public static final Topic out = newTopic(Flags.READONLY|Flags.SUMMARY,null);
  
  /**
   * Fire an event for the <code>out</code> topic.
   * @see com.vykon.vykonPro.report.BReportDelay#out
   */
  public void fireOut(BReport event) { fire(out, event, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BReportDelay.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  public void stopped()
  {
    if(ticket!=null) ticket.cancel();
  }
  
  public void doRoute(BReport report)
  {
    reportHold = report;
    
    if(getReportDelay().getMillis() > 1000L)
    {
      if(ticket != null) ticket.cancel();
      ticket = Clock.schedule(this, getReportDelay(), delayExpired, null);
      
      setDelayActive(true);
      setNextTriggerTime(Clock.time().add(getReportDelay()));
    }
    else
    {
      delayExpired();
    }
  }
  
  public void doDelayExpired()
  {
    if(ticket != null) ticket.cancel();
    fireOut(reportHold);
    
    setDelayActive(false);
  }

  ///////////////////////////////////////////
  // local variables 
  //////////////////////////////////////////
  
  Clock.Ticket ticket = null;
  BReport reportHold = null;
}
