package com.vykon.vykonPro.util;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import javax.baja.sys.BBitString;
import javax.baja.sys.BObject;
import javax.baja.sys.Context;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;


public final class BDisabledLoadsBitString
    extends BBitString
{
  



////////////////////////////////////////////////////////////////
//Constructors
////////////////////////////////////////////////////////////////

  /**
   * Construct an instance using the given bits
   */
  public BDisabledLoadsBitString(int bits)
  {
    this.bits = bits;
  }

  /**
   *  Construct an instance using the given bits
   */
  public static BDisabledLoadsBitString make(int bits)
  {                           
    return (BDisabledLoadsBitString)(new BDisabledLoadsBitString(bits).intern());
  }
  
  // //////////////////////////////////////////////////////////////
  // Access
  // //////////////////////////////////////////////////////////////

  /**
   * Get the bits.
   */
  public int getBits()
  {
    return bits;
  }

  // //////////////////////////////////////////////////////////////
  // BBitString
  // //////////////////////////////////////////////////////////////

  /**
   * Return if the bit specified by the given ordinal is set.
   */
  public boolean getBit(int ordinal)
  {
    return (bits & ordinal) != 0;
  }

  /**
   * Return if the bit specified by the given tag is set.
   */
  public boolean getBit(String tag)
  {
    return getBit(tagToOrdinal(tag));
  }
  
  /**
   * Get an array enumerating the list of all known
   * ordinal values of this bitstring instance.
   */
  public int[] getOrdinals()
  {
    return support.getOrdinals();
  }
  
  /**
   * Is the specified ordinal value included in this
   * bitstring's range of valid ordinals.
   */
  public boolean isOrdinal(int ordinal)
  {
    return support.isOrdinal(ordinal);
  }
  
  /**
   * Get the tag identifier for an ordinal value.
   */
  public String getTag(int ordinal)
  {
    return support.getTag(ordinal);
  }

  /**
   * Get the user readable tag for an ordinal value.
   */
  public String getDisplayTag(int ordinal, Context cx)
  {
    return support.getDisplayTag(ordinal, cx);
  }
  
  /**
   * Get the BBitString instance which maps to the 
   * specified set of ordinal values.
   */
  public BBitString getInstance(int[] ordinals)
  {
    int mask = 0;
    for(int i=0; i<ordinals.length; ++i) mask |= ordinals[i];
    return make(mask);
  }
  
  /**
   * Return true if the specified tag is contained by the range.
   */
  public boolean isTag(String tag)
  {
    return support.isTag(tag);
  }
  
  /**
   * Get the ordinal associated with the specified tag.
   */
  public int tagToOrdinal(String tag)
  {
    return support.tagToOrdinal(tag);
  }
  
  /**
   * Return if true if no bits set.
   */
  public boolean isEmpty()
  {
    return bits == 0;
  }
  
  public String getEmptyTag()
  {
    return "none";
  }
  
  
////////////////////////////////////////////////////////////////
//Comparsion
////////////////////////////////////////////////////////////////

  /**
   * BRoombaLedBits uses its bits as the hash code.
   */
  public int hashCode()
  {
    return bits;
  }

  /**
   * Equality is based on bitmask equality.
   */
  public boolean equals(Object obj)
  {
    if (obj instanceof BDisabledLoadsBitString)
      return ((BDisabledLoadsBitString) obj).bits == bits;
    return false;
  }  
  
  public BObject decode(DataInput in) throws IOException
  {
    return make( in.readInt() );
  }

  public BObject decodeFromString(String s) throws IOException
  {
    try
    {
      return make( Integer.parseInt(s, 16) );
    }
    catch(Exception e)
    {
      throw new IOException("Invalid bits: " + s);
    }
  }

  public void encode(DataOutput out) throws IOException
  {
    out.writeInt(bits);
  }

  public String encodeToString() throws IOException
  {
    return Integer.toHexString(bits);
  }

  
  
  public static final int OUT_A = 0x0001;
  public static final int OUT_B = 0x0002;
  public static final int OUT_C = 0x0004;
  public static final int OUT_D = 0x0008;
  public static final int OUT_E = 0x0010;
  public static final int OUT_F = 0x0020;
  public static final int OUT_G = 0x0030;
  public static final int OUT_H = 0x0040;
  public static final int OUT_I = 0x0080;
  public static final int OUT_J = 0x0200;
  
  public static final BDisabledLoadsBitString outA    = (BDisabledLoadsBitString)(new BDisabledLoadsBitString(OUT_A).intern());
  public static final BDisabledLoadsBitString outB    = (BDisabledLoadsBitString)(new BDisabledLoadsBitString(OUT_B).intern());
  public static final BDisabledLoadsBitString outC    = (BDisabledLoadsBitString)(new BDisabledLoadsBitString(OUT_C).intern());
  public static final BDisabledLoadsBitString outD    = (BDisabledLoadsBitString)(new BDisabledLoadsBitString(OUT_D).intern());
  public static final BDisabledLoadsBitString outE    = (BDisabledLoadsBitString)(new BDisabledLoadsBitString(OUT_E).intern());
  public static final BDisabledLoadsBitString outF    = (BDisabledLoadsBitString)(new BDisabledLoadsBitString(OUT_F).intern());
  public static final BDisabledLoadsBitString outG    = (BDisabledLoadsBitString)(new BDisabledLoadsBitString(OUT_G).intern());
  public static final BDisabledLoadsBitString outH    = (BDisabledLoadsBitString)(new BDisabledLoadsBitString(OUT_H).intern());
  public static final BDisabledLoadsBitString outI    = (BDisabledLoadsBitString)(new BDisabledLoadsBitString(OUT_I).intern());
  public static final BDisabledLoadsBitString outJ    = (BDisabledLoadsBitString)(new BDisabledLoadsBitString(OUT_J).intern());
  
  public static final BDisabledLoadsBitString DEFAULT = new BDisabledLoadsBitString(0);
  
  private static Support support = new Support(DEFAULT);
  static
  {
    support.add(OUT_A,    "outA");
    support.add(OUT_B,    "outB");
    support.add(OUT_C,    "outC");
    support.add(OUT_D,    "outD");
    support.add(OUT_E,    "outE");
    support.add(OUT_F,    "outF");
    support.add(OUT_G,    "outG");
    support.add(OUT_H,    "outH");
    support.add(OUT_I,    "outI");
    support.add(OUT_J,    "outJ");
  }
  

  
  
  private int bits;
  
////////////////////////////////////////////////////////////////
//Type
////////////////////////////////////////////////////////////////

public Type getType() { return TYPE; }
public static final Type TYPE = Sys.loadType(BDisabledLoadsBitString.class);



}
