package com.vykon.vykonProTransform.functions;

import javax.baja.seriestransform.exceptions.SourceException;
import javax.baja.seriestransform.exceptions.TransformException;
import javax.baja.seriestransform.graph.BTransformFunction;
import javax.baja.sys.*;
import javax.baja.units.BUnit;
import javax.baja.util.Lexicon;
import java.util.List;
import java.util.Map;

/**
 * Created by james on 11/23/2016.
 */
public class BQuotientFunction
    extends BTransformFunction 
{
	
	  /*-
  class BQuotientFunction
  {
  }
  -*/
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.vykon.vykonProTransform.functions.BQuotientFunction(354756992)1.0$ @*/
/* Generated Wed Nov 23 15:01:51 EST 2016 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BQuotientFunction.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/
  


    @Override
    public void applyFunction(Map<String, List<? extends BComplex>> series,
                              String[] srcProps, Property dstSlot,
                              BComplex resultRecord) throws TransformException
    {
      BNumber dividend = null;
      BNumber divisor = null;
      BNumber quotient = null;
      BUnit dstUnits = getUnits(dstSlot);
      BUnit srcUnits = null;
      BFacets srcFacets = null;

      //iterate through each input field name and check for series name
      //qualification prefixing the field name (seriesName.fieldName)
      for (String name : srcProps)
      {
        String seriesName = getSeriesName(name);
        String fieldName = getFieldName(name);

        //get our records for the series associated with this input field
        List<? extends BComplex> records = getSeriesRecords(series, seriesName);

        // iterate over our list of records and find our sum for the
        // given field value. Need to do this for rollup nodes or multiple schemas marked
		// as dividend or divisor with facet keys
        for (BComplex record : records)
        {
          Property srcProp = record.getProperty(fieldName);
          srcFacets = srcProp.getFacets();

          if(srcFacets!=null && srcFacets.getFacet("dividend")!=null)
          {
            //If the value is an NaN, we will discard unless it's caused by invalid units
            try
            {
              BNumber check = sumValues(dividend, record, srcProp, dstUnits);
              if (!check.equals(BDouble.NaN))
              {
                dividend = check;
              }
            }
            catch (SourceException e)
            {
              dividend = BDouble.NaN;
            }
          }
          else if(srcFacets!=null && srcFacets.getFacet("divisor")!=null)
          {
            //If the value is an NaN, we will discard unless it's caused by invalid units
            try
            {
              BNumber check = sumValues(divisor, record, srcProp, dstUnits);
              if (!check.equals(BDouble.NaN))
              {
                divisor = check;
              }
            }
            catch (SourceException e)
            {
              divisor = BDouble.NaN;
            }
          }
        }
      }

      //If the value is an NaN, we will discard unless it's caused by invalid units
	  // calculate the quotient after summing any records in a rollup or multiple input
	  // schemas for both the dividend and divisor
      try
      {
        BNumber check = divideValues(dividend, divisor);
        if (!check.equals(BDouble.NaN))
        {
          quotient = check;
        }
      }
      catch (Exception e)
      {
        quotient = BDouble.NaN;
      }

      setRecordValue(resultRecord, dstSlot, quotient);
    }

/**
   * creates a sum of the given current sum and the BNumber value found in
   * the given record at the given field.
   * 
   * @param curSum 
   *          - {@link BNumber} current numeric value we will add new record
   *            value to.
   * @param source 
   *          - {@link BComplex} source to retrieve value we are adding to our
   *            sum from.
   * @param field
   *          - The {@link Property} slot we are retrieving the value from 
   *            in our record.
   * @param dstUnits
   *          - The {@link BUnit} of the destination property we must convert
   *            our summed values to.
   *                       
   * @return {@link BNumber}
   */
	protected BNumber sumValues(BNumber curSum, BComplex source, Property field, BUnit dstUnits) throws SourceException
    {
      BValue value = source.get(field);
      BFacets facets = field.getFacets();
      BUnit origUnits = facets != null ? (BUnit)facets.get(BFacets.UNITS) : BUnit.NULL;

      if (value instanceof BNumber)
      {
        BNumber number = convertToUnits((BNumber)value, origUnits, dstUnits);

        if (null == curSum)
        {
          curSum = number;
        }
        else
        {
          curSum = addValues(number, curSum);
        }
      }
      return curSum;
    }
  
  
  /**
   * This method adds the two {@link BNumber} objects by converting both
   * number objects to the {@link BSimple} numeric type that argument <b>a</b>
   * represents. A {@link BSimple} of the same type as argument <b>a</b> is
   * returned that is the sum value of the two arguments.
   *
   * @param a first number
   * @param b second number
   * @return {@link BNumber}
   */
  protected BNumber addValues(BNumber a, BNumber b)
  {
    //sanity check
    if (null == a || null == b || Double.isNaN(a.getNumeric()) || Double.isNaN(b.getNumeric()))
    {
      return BDouble.NaN;
    }

    if (a instanceof BDouble)
    {
      return BDouble.make(a.getDouble() + b.getDouble());
    }
    if (a instanceof BFloat)
    {
      return BFloat.make(a.getFloat() + b.getFloat());
    }
    if (a instanceof BLong)
    {
      return BLong.make(a.getLong() + b.getLong());
    }
    if (a instanceof BInteger)
    {
      return BInteger.make(a.getInt() + b.getInt());
    }

    return BDouble.make(a.getDouble() + b.getDouble());
  }
  
  
  /**
   * This method divides the two {@link BNumber} objects by converting both
   * number objects to the {@link BSimple} numeric type that argument <b>a</b>
   * represents. A {@link BSimple} of the same type as argument <b>a</b> is
   * returned that is the quotient value of the two arguments.
   *
   * @param a dividend number
   * @param b divisor number
   * @return {@link BNumber}
   */
    protected BNumber divideValues(BNumber a, BNumber b)
    {
        //sanity check
        if (null == a || null == b || b.getNumeric()==0 || Double.isNaN(a.getNumeric()) || Double.isNaN(b.getNumeric()))
        {
            return BDouble.NaN;
        }

        if (a instanceof BDouble)
        {
            return BDouble.make(a.getDouble() / b.getDouble());
        }
        if (a instanceof BFloat)
        {
            return BFloat.make(a.getFloat() / b.getFloat());
        }
        if (a instanceof BLong)
        {
            return BLong.make(a.getLong() / b.getLong());
        }
        if (a instanceof BInteger)
        {
            return BInteger.make(a.getInt() / b.getInt());
        }

        return BDouble.make(a.getDouble() / b.getDouble());
    }

    @Override
    public String getFunctionName() {
        return FUNCTION_NAME;
    }

    @Override
    public String getFunctionDesc() {
        return FUNCTION_DESC;
    }

    @Override
    public Type getReturnType() {
        return BNumber.TYPE;
    }

    @Override
    public Type[] getArgumentTypes() {
        return new Type[]{BNumber.TYPE};
    }

    private static final Lexicon lex = Lexicon.make(BQuotientFunction.class);
    private static final String FUNCTION_NAME = lex.getText("functions.name.quotient");
	private static final String FUNCTION_DESC = lex.getText("functions.desc.quotient");
}

