package com.vykon.vykonProTransform.source;

import java.util.LinkedList;
import java.util.List;

import javax.baja.collection.BITable;
import javax.baja.history.BHistoryId;
import javax.baja.history.BHistoryService;
import javax.baja.history.BIHistory;
import javax.baja.history.HistoryQuery;
import javax.baja.history.db.BHistoryDatabase;
import javax.baja.history.db.HistoryDatabaseConnection;
import javax.baja.naming.BOrd;
import javax.baja.naming.OrdQuery;
import javax.baja.security.BPermissions;
import javax.baja.seriestransform.BSeriesTransformTable;
import javax.baja.seriestransform.exceptions.SourceException;
import javax.baja.seriestransform.exceptions.TransformException;
import javax.baja.seriestransform.graph.GraphNodeParams;
import javax.baja.status.BStatus;
import javax.baja.sys.BAbsTime;
import javax.baja.sys.BObject;
import javax.baja.sys.Context;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;
import javax.baja.util.BFormat;
import javax.baja.util.Lexicon;

import com.tridium.bql.util.BDynamicTimeRange;
import com.tridium.seriestransform.interval.BSeriesInterval;
import com.tridium.seriestransform.interval.BSeriesIntervalEnum;
import com.tridium.seriestransform.quantize.BNonQuantizedTable;
import com.tridium.seriestransform.quantize.BQuantizationTable;
import com.tridium.seriestransform.source.BHistorySourceNode;

public class BHistorySourceNodeDelta
    extends BHistorySourceNode
{
  /*-
   class BHistorySourceNodeDelta
   {
     properties
     {
       deltaValues: boolean
         -- If history values are totalized (ever increasing) setting delta values to true
         -- will return interval values (change of value) instead of the raw totalized values
         default {[ false ]}
     }
     actions
     {
     }
     topics
     {
     }
   }
   -*/
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.vykon.vykonProTransform.source.BHistorySourceNodeDelta(3773957618)1.0$ @*/
/* Generated Thu Dec 04 10:10:29 EST 2014 by Slot-o-Matic 2000 (c) Tridium, Inc. 2000 */

////////////////////////////////////////////////////////////////
// Property "deltaValues"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the <code>deltaValues</code> property.
   * If history values are totalized (ever increasing) setting delta values to true will return interval values (change of value) instead of the raw totalized values
   * @see com.vykon.vykonProTransform.source.BHistorySourceNodeDelta#getDeltaValues
   * @see com.vykon.vykonProTransform.source.BHistorySourceNodeDelta#setDeltaValues
   */
  public static final Property deltaValues = newProperty(0, false,null);
  
  /**
   * Get the <code>deltaValues</code> property.
   * If history values are totalized (ever increasing) setting delta values to true will return interval values (change of value) instead of the raw totalized values
   * @see com.vykon.vykonProTransform.source.BHistorySourceNodeDelta#deltaValues
   */
  public boolean getDeltaValues() { return getBoolean(deltaValues); }
  
  /**
   * Set the <code>deltaValues</code> property.
   * If history values are totalized (ever increasing) setting delta values to true will return interval values (change of value) instead of the raw totalized values
   * @see com.vykon.vykonProTransform.source.BHistorySourceNodeDelta#deltaValues
   */
  public void setDeltaValues(boolean v) { setBoolean(deltaValues,v,null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BHistorySourceNodeDelta.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  /*
   * (non-Javadoc)
   * @see javax.baja.seriestransform.graph.BGraphNode#doResolve(javax.baja.seriestransform.BSeriesTransformTable[], javax.baja.seriestransform.graph.GraphNodeParams, javax.baja.naming.BOrd, javax.baja.sys.Context)
   */
  
  protected BSeriesTransformTable[] doResolve(BSeriesTransformTable[] inputs, GraphNodeParams args,
                                              BOrd base, Context cx) throws TransformException
  {
    
    //check if our input data is linked in from the output of a transform
    //graph
    if( inputs.length > 0) return inputs;
    
    //attempt to construct a history ORD from our node settings
    BOrd schemaOrd = BOrd.NULL;
    BFormat relativeSource = (BFormat)args.get(dataSource.getName(), getDataSource());
    BDynamicTimeRange range = (BDynamicTimeRange)args.get(dateRange.getName(), getDateRange());
    String historyOrdParams = getHistoryOrdParams(range,getDeltaValues());
    
    //construct our ORD from our format string and date range
    String formatStr = relativeSource.getFormat();
    if( null != formatStr && formatStr.length() > 0)
    {
      BObject object = base.get();
      String format = relativeSource.format(object);
      try
      {
        schemaOrd = BOrd.make(format + historyOrdParams);
      }catch(RuntimeException re){
        /* do nothing*/
      }
    }

    // If we have a source ORD, process the source
    List results = new LinkedList();
    if( schemaOrd != BOrd.DEFAULT )
    {
      //parse the schema ORD to get our history ORD. We will use the history
      //ORD to retrieve the history itself and check that the user has 
      //permissions to access that history.
      OrdQuery[] queries = schemaOrd.parse();
      for( int i = 0; i < queries.length; i++)
      {
        OrdQuery query = queries[i];
        if( query instanceof HistoryQuery)
        {
          //resolve the history query to a BIHistory
          HistoryQuery historyQuery = (HistoryQuery)query;
          BHistoryId historyId = historyQuery.getHistoryId();
          BHistoryDatabase localDb = 
              ((BHistoryService)Sys.getService(BHistoryService.TYPE)).getDatabase();
              
          BIHistory history = null;
          try(HistoryDatabaseConnection conn = localDb.getDbConnection(null))
          {
            history = conn.getHistory(historyId);
          }
          catch(Exception e)
          {
            System.out.println(this.getSlotPath());
            e.printStackTrace();
          }
          
          //get our user permissions for this history. If the context has 
          // a NULL user, by default they have full permissions. This is to
          // conform with the current standard in Niagara of giving full 
          // permissions for Program Objects. 
          // XXX: Workaround for JIRA issue NCCB-4508 until proper context
          // handling can be implemented
          if( null != cx && null != cx.getUser())
          {
            BPermissions permissions = cx.getUser().getPermissionsFor(history);

            // if our user doesn't have the correct permissions, throw an exception
            if (!permissions.hasOperatorRead())
            {
              String text = lex.getText(INVALID_PERMISSIONS_KEY, new Object[] {
                historyQuery.toString()
              });
              throw new SourceException(text);
            }
          }
            
        }
      }
      
      //resolve our history ORD to a collection of history records.
      BObject object = schemaOrd.get(base.get());
      if( object instanceof BITable)
      {
        BITable table = (BITable)object;
        setStatus(BStatus.ok);

        //create our quantization cursor for this source
        BSeriesInterval interval = getQuantInterval();
        if( interval.getDesc() == BSeriesIntervalEnum.none)
          results.add( new BNonQuantizedTable(getName(),getSchema(),table));
        else
          results.add( new BQuantizationTable(getName(),getSchema(),interval,table) );

        return (BSeriesTransformTable[]) results.toArray(new BSeriesTransformTable[]{});
      }

    }

    //If we reach this point, we have an invalid data source
    throw new SourceException(INVALID_DATA_SRC);
  }
  
  
  /**
   * This method returns the history ORD parameters based on the date range
   * information set for this node.
   * 
   * @param range
   *          - {@link BDynamicTimeRange} used to form our history query params.
   * 
   * @param delta values
   *          - {@link BBoolean } use to form our history query params.
   *                   
   * @return String history ORD query parameters to attach to history ORD.
   */
  private String getHistoryOrdParams(BDynamicTimeRange range, boolean delta)
  {
    StringBuffer sb = new StringBuffer();
    
    if( range != BDynamicTimeRange.DEFAULT) 
    {
      BAbsTime currentTime = BAbsTime.now();
      BAbsTime startTime = range.getStartTime(currentTime);
      BAbsTime endTime = range.getEndTime(currentTime);
      
      sb.append("?start=" + startTime.encodeToString() + ";");
      sb.append("end=" + endTime.encodeToString());
    }
    
    if(delta)
    {
      if(sb.length()>0) sb.append(";delta=true");
      else sb.append("?delta=true");
    }
    return sb.toString();
    
  }
  
////////////////////////////////////////////////////////////////
//Constants
////////////////////////////////////////////////////////////////

private static final Lexicon lex = Lexicon.make(BHistorySourceNode.class);
private static final String INVALID_DATA_SRC = lex.getText("fault.msg.invalidHistorySource");
private static final String INVALID_PERMISSIONS_KEY = "fault.msg.invalidPermissions";
}
